#ifndef OPTIMIZER_H
#define OPTIMIZER_H

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cmath>
#include "Eigen/Eigen"
#include "Vec.h"

class Optimizer
{
public:
	Optimizer();
	~Optimizer();

private:
	std::vector<double>										m_IniteValue;
	std::vector<double>										m_ResultValue;
	
	//std::vector<std::vector<double>>						m_IniteValueV;
	//std::vector<std::vector<double>>						m_ResultValueV;
	//std::vector<std::vector<trimesh::ivec3>>				m_RegularityV;

	//std::vector<std::vector<double>>						m_MatrixNewton;
	Eigen::MatrixXd											m_MatrixNewton;
	std::vector<std::vector<double>>						m_MatrixA;

	std::vector<trimesh::vec2>								m_OriginalPosition;
	std::vector<trimesh::vec2>								m_OriginalScale;
	std::vector<trimesh::vec2>								m_OriginalScaleStore;
	std::vector<trimesh::vec4>								m_ScaleThreshold;
	std::vector<trimesh::vec4>								m_ScaleThresholdStore;
	std::vector<trimesh::vec4>								m_DynamicScaleThreshold;
	std::vector<trimesh::ivec3>								m_AdjacentPair;
	std::vector<std::vector<trimesh::ivec2>>				m_ApartBBoxPair;
	std::vector<double>										m_ApartDistance;
	std::vector<trimesh::ivec2>								m_AffiliatedPair;
	std::vector<std::vector<int>>							m_AffiliatedVertex;
	std::vector<trimesh::ivec3>								m_BBoxPositionStructure;
	std::vector<trimesh::vec3>								m_BBoxStructureDistance;
	std::vector<trimesh::ivec2>								m_StructurePair;
	std::vector<double>										m_StructureDistanceSquare;
	std::vector<trimesh::ivec3>								m_GroupPositionStructure;
	std::vector<trimesh::vec3>								m_GroupStructureDistance;
	std::vector<trimesh::ivec2>								m_ObserveRelatedID;
	//std::vector<trimesh::vec3>							m_ObserveField;
	std::vector<std::vector<trimesh::vec2>>					m_ObserveField;
	std::vector<trimesh::vec2>								m_TargetPoint;
	std::vector<std::vector<int>>							m_OcclusionRelatedID;
	std::vector<trimesh::ivec3>								m_RegularityStructure;
	std::vector<int>										m_DeleteRegularityID;
	//ivec2 1st is component id has regularity, 2nd is num of regularity, 3rd is regularity direction---0 along x,1 along y

	double													m_WeightPosition;
	double													m_WeightScale;
	double													m_WeightPathWidth;
	double													m_WeightAffiliatePosition;
	double													m_WeightAffiliateScale;
	double													m_WeightVisible;
	double													m_WeightStructure;
	std::vector<double>										m_WeightVector;

	double													m_phiDelta;
	double													m_barrierSigma;
	double													m_distanceEpsilon;
	double													m_PathWidth;
	double													m_MoveRangeR;
	double													m_MoveExpS;
	double													m_NeiborDistanceSquare;
	double													m_barrierEpsilon;
	double													m_barrierEpsilon2;
	double													m_deleteRegularityThreshold;

	double													m_ScalePenalty;
	double													m_PathWidthPenalty;

public:
	void IniteValue(std::vector<double> x);
	void IniteValue(double scale);
	void ImportOriginalPosition(std::vector<trimesh::vec2> p);
	void ImportOriginalScale(std::vector<trimesh::vec2> s);
	void ImportScaleThreshold(std::vector<trimesh::vec4> t);
	void ImportAffiliatedPair(std::vector<trimesh::ivec2> af_p, std::vector<std::vector<int>> af_v);
	void ImportApartBBoxPair(std::vector<std::vector<trimesh::ivec2>> ap_p, std::vector<double> ap_d);
	void ImportAdjacentPair(std::vector<trimesh::ivec3> ad_p);
	void ImportStructure(std::vector<trimesh::ivec3> s);
	void ImportStructure(std::vector<std::vector<int>> s_g);
	void ImportGroupStructure(std::vector<trimesh::ivec3> g_s, std::vector<trimesh::vec3> g_d);
	//void ImportVisibleInfo(std::vector<trimesh::ivec2> observe_related_ID, std::vector<trimesh::vec3> observe_field);
	void ImportVisibleInfo(std::vector<trimesh::ivec2> observe_related_ID, std::vector<std::vector<trimesh::vec2>> observe_field,
		std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_ID);

	void ImportRegularityStructure(std::vector<trimesh::ivec3> r);

	void OptimizeNewton(std::vector<double> &optimal_energy);
	void OptimizeRegularity(std::vector<double> &optimal_energy);
	void OptimizeRegularity(std::vector<double> &optimal_energy, int &stop_flag);
	void OptimizeRegularity(std::vector<double> &optimal_energy, int &stop_flag, std::ofstream &info_file);

	std::vector<double> getResult();
	void getStructure(std::vector<trimesh::ivec2> &structure_pair, std::vector<trimesh::vec2> &structure_distance);
	void OutputResult(std::string result_path);
	void OutputPAndS(std::string result_path);
	void testScaleThreshold(std::string test_path);
	void testPathWidth(std::string test_path);
	void testVisibleConstrain(std::string test_path);
	void testEnergy(std::string value_path);

	void testFunction_phi();
	void testFunctionBarrier();
	void testNeiborDistance();//to do

	double BBoxDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2);
	double BBoxDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2, int &apart_mode);

	void deleteRegularitySpecial();
private:
	double FunctionPosition(std::vector<double> &x);
	double FunctionPosition(std::vector<double> &x, std::ofstream &energy_test);
	void difFunctionPosition(std::vector<double> &x, std::vector<double> &dif);
	void HessianFunctionPosition(std::vector<double> &x, std::vector<std::vector<double>> &hessian);

	double FunctionPosition_Old(std::vector<double> &x);
	double FunctionPosition_Old(std::vector<double> &x, std::ofstream &energy_test);
	void difFunctionPosition_Old(std::vector<double> &x, std::vector<double> &dif);
	void HessianFunctionPosition_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian);

	void updateDynamicScaleThreshold(std::vector<double> &x);
	trimesh::vec2 adjustThreshold(double s, double s_min, double s_max);

	double FunctionScale(std::vector<double> &x);
	double FunctionScale(std::vector<double> &x, std::ofstream &energy_test);
	void difFunctionScale(std::vector<double> &x, std::vector<double> &dif);
	void HessianFunctionScale(std::vector<double> &x, std::vector<std::vector<double>> &hessian);

	void updateScaleTolerance(std::vector<double> &x);

	double FunctionScale_Old(std::vector<double> &x);
	double FunctionScale_Old(std::vector<double> &x, std::ofstream &energy_test);
	void difFunctionScale_Old(std::vector<double> &x, std::vector<double> &dif);
	void HessianFunctionScale_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian);

	double FunctionPathWidth(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
		std::vector<double> &apart_distance);
	double FunctionPathWidth(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
		std::vector<double> &apart_distance, std::ofstream &energy_test);
	void difFunctionPathWidth(std::vector<double> &x, std::vector<double> &dif,
		std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance);
	void HessianFunctionPathWidth(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance);

	double FunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
		std::vector<double> &apart_distance);
	double FunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
		std::vector<double> &apart_distance, std::ofstream &energy_test);
	void difFunctionPathWidth_Old(std::vector<double> &x, std::vector<double> &dif,
		std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance);
	void HessianFunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance);

	double FunctionAffiliatedPosition(std::vector<double> &x, std::vector<trimesh::ivec2> &affiliated_pair,
		std::vector<std::vector<int>> &affiliated_vertex);
	void difFunctionAffiliatedPosition(std::vector<double> &x, std::vector<double> &dif,
		std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex);
	void HessianFunctionAffiliatedPosition(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex);

	double FunctionAffiliatedScale(std::vector<double> &x, std::vector<trimesh::ivec2> &affiliated_pair,
		std::vector<std::vector<int>> &affiliated_vertex);
	void difFunctionAffiliatedScale(std::vector<double> &x, std::vector<double> &dif,
		std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex);
	void HessianFunctionAffiliatedScale(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex);

	double FunctionStructure(std::vector<double> &x, std::vector<trimesh::ivec2> &structure_pair,
		std::vector<double>  &structure_distance_square);
	void difFunctionStructure(std::vector<double> &x, std::vector<double> &dif,
		std::vector<trimesh::ivec2> &structure_pair, std::vector<double> &structure_distance_square);
	void PositiveHessianFunctionStructure(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec2> &structure_pair, std::vector<double> &structure_distance_square);

	double FunctionStructure_Old(std::vector<double> &x, std::vector<trimesh::ivec3> &structure_triangle,
		std::vector<trimesh::vec3> &triangle_edge);
	double FunctionStructure_Old(std::vector<double> &x, std::vector<trimesh::ivec3> &structure_triangle,
		std::vector<trimesh::vec3> &triangle_edge, std::ofstream &energy_test);
	void difFunctionStructure_Old(std::vector<double> &x, std::vector<double> &dif,
		std::vector<trimesh::ivec3> &structure_triangle, std::vector<trimesh::vec3> &triangle_edge);
	void HessianFunctionStructure_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec3> &structure_triangle, std::vector<trimesh::vec3> &triangle_edge);

	double FunctionVisible(std::vector<double> &x, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point,
		std::vector<std::vector<int>> occlusion_related_id);
	void difFunctionVisible(std::vector<double> &x, std::vector<double> &dif, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point,
		std::vector<std::vector<int>> occlusion_related_id);
	void HessianFunctionVisible(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec2> &observe_related_id, std::vector<std::vector<trimesh::vec2>> observe_field,
		std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_id);

	double FunctionVisible_Old(std::vector<double> &x, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<trimesh::vec3> observe_field);
	void difFunctionVisible_Old(std::vector<double> &x, std::vector<double> &dif,
		std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field);
	void HessianFunctionVisible_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
		std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field);

	double FunctionPenalty(std::vector<double> &x);
	double FunctionPenalty(std::vector<double> &x, std::ofstream &info_file);
	double ScalePenalty(double scale, double threshold_min, double threshold_max);
	double PathWidthPenalty(trimesh::vec4 bbox1, trimesh::vec4 bbox2, double path_width);

	double function_phi(double x, double x_min, double x_max);
	double dif_function_phi(double x, double x_min, double x_max);
	double ddif_function_phi(double x, double x_min, double x_max);

	double function_phi_Old(double x, double x_min, double x_max);
	double dif_function_phi_Old(double x, double x_min, double x_max);
	double ddif_function_phi_Old(double x, double x_min, double x_max);

	double function_barrier_d(double d1, double d2, double l, double epsilon);
	double dif_function_barrier_d(double d1, double d2, double l, double epsilon);
	double ddif_function_barrier_d(double d1, double d2, double l, double epsilon);
	double function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon);
	double function_barrier_test(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon);
	void dif_function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon,
		std::vector<double> &dif);
	void positive_hessian_function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
		double epsilon, std::vector<std::vector<double>> &hessian);

	double function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
		double barrier_delta);
	double function_elliptic_barrier(double Ew_coe, double Eh_coe, trimesh::vec2 p, double barrier_delta);
	trimesh::vec4 dif_function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
		double barrier_delta);
	void hessian_function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
		double barrier_delta, std::vector<std::vector<double>> &hessian);

	double function_distance_energy(double distance);
	double dif_function_distance_energy(double distance);
	double ddif_function_distance_energy(double distance);
	inline double sign_distance(trimesh::vec3 boundary_coe, trimesh::vec2 p);

	double function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
		std::vector<int> vertex_id);
	void dif_function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
		std::vector<int> vertex_id, std::vector<double> &dif);
	void ddif_function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
		std::vector<int> vertex_id, std::vector<std::vector<double>> &hessian);

	void testDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<int> vertex_id,
		std::vector<std::vector<double>> &vertex_distance);

	double function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2);
	void dif_function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
		std::vector<double> &dif);
	void ddif_function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
		std::vector<std::vector<double>> &hessian);

	double function_triangle_similarity(std::vector<trimesh::vec2> triangle1, trimesh::vec3 triangle2);
	void dif_function_triangle_similarity(std::vector<trimesh::vec2>triangle1, trimesh::vec3 triangle2,
		std::vector<double> &dif);
	void ddif_function_triangle_similarity(std::vector<trimesh::vec2> triangle1, trimesh::vec3 triangle2,
		std::vector<std::vector<double>> &hessian);

	double function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field,
		trimesh::vec2 target_position);
	void visible_boundary(trimesh::vec2 p, trimesh::vec3 field, trimesh::vec3 &b1, trimesh::vec3 &b2);
	void dif_function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field,
		trimesh::vec2 target_position, std::vector<double> &dif);
	void ddif_function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field,
		trimesh::vec2 target_position, std::vector<std::vector<double>> &hessian);

	double function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 target_bbox, trimesh::vec2 target_point);
	void dif_function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 target_bbox, trimesh::vec2 target_point, std::vector<double> &dif);
	void positive_hessian_function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 target_bbox, trimesh::vec2 target_point, std::vector<std::vector<double>> &hessian);

	double function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 occlusion_bbox);
	void dif_function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 occlusion_bbox, std::vector<double> &dif);
	void positive_hessian_function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
		trimesh::vec4 occlusion_bbox, std::vector<std::vector<double>> &hessian);

	void ObjectFunction(std::vector<double> &x, std::vector<double> &energy);
	void ObjectFunction(std::vector<double> &x, std::vector<double> &energy, std::ofstream &energy_test);
	void difObjectFunction(std::vector<double> &x, std::vector<double> &dif);
	void HessianObjectFunction(std::vector<double> &x, std::vector<std::vector<double>> &hessian);

	void LinearConstraint(int variable_num);
	void addSpecialLinearConstraint(std::string constraint_filepath, int variable_num);
	void ConstractNewtonMatrix(std::vector<std::vector<double>> &hessian);
	void UpdateNewtonMatrix(std::vector<std::vector<double>> &hessian);

	void CalculateDescentDirection(std::vector<double> &p, std::vector<double> &dif,
		std::vector<std::vector<double>> &hessian);
	bool CalculateDescentDirection2(std::vector<double> &p, std::vector<double> &dif,
		std::vector<std::vector<double>> &hessian);
	void CalculateDescentDirectionLU(std::vector<double> &p, std::vector<double> &dif,
		std::vector<std::vector<double>> &hessian);
	double CalculateTmax(std::vector<double> &x,std::vector<double> &p);
	double CalculateAffiliatedTMax(std::vector<double> &x, std::vector<double> &p,
		std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex);
	double CalculateVisibleTMax(std::vector<double> &x, std::vector<double> &p, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point,
		std::vector<std::vector<int>> occlusion_related_id);
	double CalculateKeepVisibleTmax(std::vector<double> &x, std::vector<double> &p, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point);
	double CalculatePreventOcclusionTMax(std::vector<double> &x, std::vector<double> &p, std::vector<trimesh::ivec2> &observe_related_id,
		std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<std::vector<int>> occlusion_related_id);
	double CalculateVisibleTMaxOld(std::vector<double> &x, std::vector<double> &p,
		std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field);
	double CalculatePathBarrierTmax(std::vector<double> &x, std::vector<double> &p,
		std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance);
	void LineSearch(std::vector<double> &x, std::vector<double> &p, double tmax,
		trimesh::vec2 &stepsize, double energy_pre, std::vector<double> &energy, std::vector<double> &result);
	void LineSearch(std::vector<double> &x, std::vector<double> &p, double energy_pre,
		std::vector<double> &energy, double &stepsize, std::vector<double> &result);
	void LineSearch(std::vector<double> &x, std::vector<double> &p, double tmax,
		double energy_pre, std::vector<double> &energy, double &stepsize, std::vector<double> &result);

	void testMatrixRank(std::vector<std::vector<double>> matrix);
	void testMatrixRank(Eigen::MatrixXd matrix);
	bool isHessianPositive(std::vector<std::vector<double>> &hessian);
	bool PositiveHessian(std::vector<std::vector<double>> &hessian);

	bool isPinTriangle(trimesh::vec2 p, trimesh::vec2 t1, trimesh::vec2 t2, trimesh::vec2 t3);
	double triangle_area(trimesh::vec2 t1, trimesh::vec2 t2, trimesh::vec2 t3);

	void vl_Coefficient(std::vector<trimesh::vec2> &lx_C, std::vector<trimesh::vec2> &ly_C,
		std::vector<trimesh::vec2> &vx_C, std::vector<trimesh::vec2> &vy_C,
		std::vector<double> &x, std::vector<double> &p, int bbox1_ID, int bbox2_ID, double distance);
	double vertex2bboxTmax(std::vector<trimesh::vec2> lx_C, std::vector<trimesh::vec2> ly_C,
		trimesh::vec2 vx_C, trimesh::vec2 vy_C);
	bool isOnSegment(trimesh::vec2 bound1, trimesh::vec2 bound2, trimesh::vec2 v, double t);

	void deleteRegularity(std::vector<trimesh::ivec3> &regularity_structure, int id);
	void resetRegularity();

	int quadratic_root(double a, double b, double c,trimesh::vec2 &q_root);
};




#endif