#include "Scene.h"

Scene::Scene()
{
}

Scene::Scene(std::string scene_path)
{
	loadScene(scene_path);
}

Scene::~Scene()
{
}

void Scene::loadScene(std::string scene_path)
{
	std::fstream scene_file(scene_path);

	m_ComponentBBox.clear();
	m_ScaleThreshold.clear();
	m_AdjacentPair.clear();
	m_AffiliatedPair.clear();
	m_AffiliatedVertexID.clear();
	m_VisibleConstrain.clear();
	m_ApartPair.clear();
	m_ApartDistance.clear();

	std::string str;
	int component_num;
	int component_id;
	int vertex_num;
	int pair_num;
	int apart_class_num;
	double apart_distance;
	trimesh::vec4 BBox;
	trimesh::vec4 ScaleThreshold;
	trimesh::vec2 position;
	trimesh::vec2 direction;
	trimesh::vec4 field;
	trimesh::ivec2 Pair;
	trimesh::ivec3 Pair_3;
	VisibleInfo ObserveConstrain;

	scene_file >> str;
	scene_file >> m_SceneBound[0] >> m_SceneBound[1];

	scene_file >> str;
	scene_file >> component_num;
	for (int i = 0; i < component_num; ++i)
	{
		scene_file >> BBox[0] >> BBox[1] >> BBox[2] >> BBox[3];
		scene_file >> ScaleThreshold[0] >> ScaleThreshold[1] >> ScaleThreshold[2] >> ScaleThreshold[3];
		m_ComponentBBox.push_back(BBox);
		m_ScaleThreshold.push_back(ScaleThreshold);
	}

	scene_file >> str;
	scene_file >> pair_num;
	for (int i = 0; i < pair_num; ++i)
	{
		scene_file >> Pair_3[0] >> Pair_3[1] >> Pair_3[2];
		m_AdjacentPair.push_back(Pair_3);
	}

	scene_file >> str;
	scene_file >> pair_num;
	for (int i = 0; i < pair_num; ++i)
	{
		scene_file >> Pair[0] >> Pair[1];
		m_AffiliatedPair.push_back(Pair);
		scene_file >> vertex_num;
		std::vector<int> vertexID(vertex_num);
		for (int j = 0; j < vertex_num; ++j)
		{
			scene_file >> vertexID[j];
		}
		m_AffiliatedVertexID.push_back(vertexID);
	}

	scene_file >> str;
	scene_file >> pair_num;
	for (int i = 0; i < pair_num; ++i)
	{
		scene_file >> component_id;
		ObserveConstrain.observe_bbox_id = component_id - 1;
		scene_file >> position[0] >> position[1];
		ObserveConstrain.observe_point_position = position;
		scene_file >> field[0] >> field[1] >> field[2] >> field[3];
		ObserveConstrain.observe_field = field;
		scene_file >> component_id;
		ObserveConstrain.target_bbox_id = component_id - 1;
		scene_file >> position[0] >> position[1];
		ObserveConstrain.target_point_position = position;
		scene_file >> component_num;
		std::vector<int> component_id_v(component_num, 0);
		for (int j = 0; j < component_num; ++j)
		{
			scene_file >> component_id;
			component_id_v[j] = component_id - 1;
		}
		ObserveConstrain.occlusion_bbox_id = component_id_v;

		m_VisibleConstrain.push_back(ObserveConstrain);
	}

	scene_file >> str;
	scene_file >> apart_class_num;
	for (int i = 0; i < apart_class_num; ++i)
	{
		std::vector<trimesh::ivec2> current_apartPair;
		scene_file >> apart_distance;
		m_ApartDistance.push_back(apart_distance);
		scene_file >> pair_num;
		for (int j = 0; j < pair_num; ++j)
		{
			scene_file >> Pair[0] >> Pair[1];

			current_apartPair.push_back(trimesh::ivec2(Pair[0] - 1, Pair[1] - 1));
			current_apartPair.push_back(trimesh::ivec2(Pair[1] - 1, Pair[0] - 1));
		}
		m_ApartPair.push_back(current_apartPair);
	}

	detectAdjacent();
	//detectAdjacentNoLineareConstraint();
	detectAffiliated();
	//selectApart();
}

void Scene::loadScene_Old(std::string scene_path)
{
	std::fstream scene_file(scene_path);

	m_ComponentBBox.clear();
	m_ScaleThreshold.clear();
	m_AdjacentPair.clear();
	m_AffiliatedPair.clear();
	m_AffiliatedVertexID.clear();
	m_VisibleConstrain.clear();
	m_ApartPair.clear();
	m_ApartDistance.clear();

	std::string str;
	int component_num;
	int component_id;
	int vertex_num;
	int pair_num;
	int observe_type;
	int apart_class_num;
	double apart_distance;
	trimesh::vec4 BBox;
	trimesh::vec4 ScaleThreshold;
	trimesh::vec2 position;
	trimesh::vec2 direction;
	double field;
	trimesh::ivec2 Pair;
	trimesh::ivec3 Pair_3;
	VisibleInfo ObserveConstrain;

	scene_file >> str;
	scene_file >> m_SceneBound[0] >> m_SceneBound[1];

	scene_file >> str;
	scene_file >> component_num;
	for (int i = 0; i < component_num; ++i)
	{
		scene_file >> BBox[0] >> BBox[1] >> BBox[2] >> BBox[3];
		scene_file >> ScaleThreshold[0] >> ScaleThreshold[1] >> ScaleThreshold[2] >> ScaleThreshold[3];
		m_ComponentBBox.push_back(BBox);
		m_ScaleThreshold.push_back(ScaleThreshold);
	}

	scene_file >> str;
	scene_file >> pair_num;
	for (int i = 0; i < pair_num; ++i)
	{
		scene_file >> Pair_3[0] >> Pair_3[1] >> Pair_3[2];
		m_AdjacentPair.push_back(Pair_3);
	}

	scene_file >> str;
	scene_file >> pair_num;
	for (int i = 0; i < pair_num; ++i)
	{
		scene_file >> Pair[0] >> Pair[1];
		m_AffiliatedPair.push_back(Pair);
		scene_file >> vertex_num;
		std::vector<int> vertexID(vertex_num);
		for (int j = 0; j < vertex_num; ++j)
		{
			scene_file >> vertexID[j];
		}
		m_AffiliatedVertexID.push_back(vertexID);
	}

// 	scene_file >> str;
// 	scene_file >> pair_num;
// 	for (int i = 0; i < pair_num; ++i)
// 	{
// 		scene_file >> observe_type;
// 		ObserveConstrain.observe_type = observe_type;
// 		if (observe_type == 0)
// 		{
// 			scene_file >> position[0] >> position[1];
// 			ObserveConstrain.observe_position_point = position;
// 		}
// 		else
// 		{
// 			scene_file >> component_id;
// 			ObserveConstrain.observe_position_id = component_id;
// 		}
// 		scene_file >> direction[0] >> direction[1];
// 		ObserveConstrain.observe_direction = direction;
// 		scene_file >> field;
// 		ObserveConstrain.observe_field = field;
// 		scene_file >> component_id;
// 		ObserveConstrain.target_id = component_id;
// 
// 		m_VisibleConstrain.push_back(ObserveConstrain);
// 	}

	scene_file >> str;
	scene_file >> apart_class_num;
	for (int i = 0; i < apart_class_num; ++i)
	{
		std::vector<trimesh::ivec2> current_apartPair;
		scene_file >> apart_distance;
		m_ApartDistance.push_back(apart_distance);
		scene_file >> pair_num;
		for (int j = 0; j < pair_num; ++j)
		{
			scene_file >> Pair[0] >> Pair[1];
			////////////////////////////////
			/*if (Pair[0]>17)
			{
			Pair[0] = Pair[0] + 6;
			}
			else if (Pair[0]>9)
			{
			Pair[0] = Pair[0] + 2;
			}
			else if (Pair[0] == 2)
			{
			Pair[0] = 10;
			}
			if (Pair[1]>17)
			{
			Pair[1] = Pair[1] + 6;
			}
			else if (Pair[1] > 9)
			{
			Pair[1] = Pair[1] + 2;
			}
			else if (Pair[1] == 2)
			{
			Pair[1] = 10;
			}*/
			////////////////////////////////
			current_apartPair.push_back(trimesh::ivec2(Pair[0] - 1, Pair[1] - 1));
			current_apartPair.push_back(trimesh::ivec2(Pair[1] - 1, Pair[0] - 1));
		}
		m_ApartPair.push_back(current_apartPair);
	}

	detectAdjacent();
	detectAffiliated();
	//selectApart();
}

void Scene::loadBBoxStructure(std::string structure_path)
{
	m_BBoxPositionStructure.clear();

	std::fstream structure_file(structure_path);
	int structure_num;
	trimesh::ivec3 triangle_v(0, 0, 0);
	structure_file >> structure_num;
	for (int i = 0; i < structure_num; ++i)
	{
		structure_file >> triangle_v[0] >> triangle_v[1] >> triangle_v[2];
		triangle_v[0] = triangle_v[0] - 1;
		triangle_v[1] = triangle_v[1] - 1;
		triangle_v[2] = triangle_v[2] - 1;
		m_BBoxPositionStructure.push_back(triangle_v);
	}
}

void Scene::loadStructureGroup(std::string structure_path)
{
	std::fstream structure_file(structure_path);
	int group_num;
	int bbox_num;
	int bbox_id;
	structure_file >> group_num;
	for (int i = 0; i < group_num; ++i)
	{
		structure_file >> bbox_num;
		std::vector<int> structure_group(bbox_num, 0);
		for (int j = 0; j < bbox_num; ++j)
		{
			structure_file >> bbox_id;
			structure_group[j] = bbox_id - 1;
		}
		m_BBoxStructureGroup.push_back(structure_group);
	}
}

void Scene::loadGroupStructure(std::string structure_path)
{
	m_GroupPositionStructure.clear();

	std::fstream structure_file(structure_path);
	int structure_num;
	trimesh::ivec3 triangle_v(0, 0, 0);
	structure_file >> structure_num;
	for (int i = 0; i < structure_num; ++i)
	{
		structure_file >> triangle_v[0] >> triangle_v[1] >> triangle_v[2];
		triangle_v[0] = triangle_v[0] - 1;
		triangle_v[1] = triangle_v[1] - 1;
		triangle_v[2] = triangle_v[2] - 1;
		m_GroupPositionStructure.push_back(triangle_v);
	}
}

void Scene::loadRegularityStructure(std::string structure_path)
{
	m_RegularityStructure.clear();

	std::fstream structure_file(structure_path);
	int structure_num;
	trimesh::ivec3 regularity(0, 0, 0);
	structure_file >> structure_num;
	for (int i = 0; i < structure_num; ++i)
	{
		structure_file >> regularity[0] >> regularity[1] >> regularity[2];
		regularity[0] = regularity[0] - 1;
		m_RegularityStructure.push_back(regularity);
	}
}

void Scene::initeOptimizer(double scale)
{
	std::vector<trimesh::vec2> original_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> scale_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> original_scale(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		original_position[i][0] = m_ComponentBBox[i][0];
		original_position[i][1] = m_ComponentBBox[i][1];
		scale_position[i][0] = m_ComponentBBox[i][0] * scale;
		scale_position[i][1] = m_ComponentBBox[i][1] * scale;
		original_scale[i][0] = m_ComponentBBox[i][2];
		original_scale[i][1] = m_ComponentBBox[i][3];
	}
	m_Optimizer.ImportOriginalPosition(scale_position);
	m_Optimizer.ImportOriginalScale(original_scale);
	m_Optimizer.ImportScaleThreshold(m_ScaleThreshold);
	m_Optimizer.ImportAffiliatedPair(m_AffiliatedPair, m_AffiliatedVertexID);
	m_Optimizer.ImportApartBBoxPair(m_ApartPair, m_ApartDistance);
	m_Optimizer.ImportAdjacentPair(m_AdjacentPair);
	//m_Optimizer.ImportStructure(m_BBoxPositionStructure);
	m_Optimizer.ImportStructure(m_BBoxStructureGroup);
	m_Optimizer.ImportRegularityStructure(m_RegularityStructure);

// 	std::ofstream testGroupStructure("data/test/testGroupStucture.txt");
// 	std::vector<trimesh::vec3> group_structure_distance(m_GroupPositionStructure.size(), trimesh::vec3(0, 0, 0));
// 	for (int i = 0; i < m_GroupPositionStructure.size(); ++i)
// 	{
// 		trimesh::vec3 triangle_edge;
// 
// 		triangle_edge[0] = (original_position[m_GroupPositionStructure[i][1]] - original_position[m_GroupPositionStructure[i][0]]).length();
// 		triangle_edge[1] = (original_position[m_GroupPositionStructure[i][2]] - original_position[m_GroupPositionStructure[i][1]]).length();
// 		triangle_edge[2] = (original_position[m_GroupPositionStructure[i][0]] - original_position[m_GroupPositionStructure[i][2]]).length();
// 		group_structure_distance.push_back(triangle_edge);
// 		testGroupStructure << m_GroupPositionStructure[i][0] << " " << m_GroupPositionStructure[i][1] << " " << m_GroupPositionStructure[i][2] << "   ";
// 		testGroupStructure << triangle_edge[0] << " " << triangle_edge[1] << " " << triangle_edge[2] << std::endl;
// 	}
// 	m_Optimizer.ImportGroupStructure(m_GroupPositionStructure, group_structure_distance);


	std::vector<trimesh::ivec2> observe_related_ID(m_VisibleConstrain.size(), trimesh::ivec2(0, 0));
	std::vector<std::vector<trimesh::vec2>> observe_field(m_VisibleConstrain.size(), std::vector<trimesh::vec2>(3, trimesh::vec2(0, 0)));
	std::vector<trimesh::vec2> target_point(m_VisibleConstrain.size(), trimesh::vec2(0, 0));
	std::vector<std::vector<int>> occlusion_related_ID(m_VisibleConstrain.size(), std::vector<int>());
	for (int i = 0; i < m_VisibleConstrain.size(); ++i)
	{
		observe_related_ID[i][0] = m_VisibleConstrain[i].observe_bbox_id;
		observe_related_ID[i][1] = m_VisibleConstrain[i].target_bbox_id;

		observe_field[i][0] = m_VisibleConstrain[i].observe_point_position;
		observe_field[i][1] = trimesh::vec2(m_VisibleConstrain[i].observe_field[0], m_VisibleConstrain[i].observe_field[1]);
		observe_field[i][2] = trimesh::vec2(m_VisibleConstrain[i].observe_field[2], m_VisibleConstrain[i].observe_field[3]);

		target_point[i] = m_VisibleConstrain[i].target_point_position;

		occlusion_related_ID[i] = m_VisibleConstrain[i].occlusion_bbox_id;
	}
	m_Optimizer.ImportVisibleInfo(observe_related_ID, observe_field, target_point, occlusion_related_ID);

	//m_Optimizer.testFunction_phi();
	//m_Optimizer.testFunctionBarrier();

}

void Scene::initeOptimizer(trimesh::vec2 scale)
{
	std::vector<trimesh::vec2> original_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> scale_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> original_scale(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		original_position[i][0] = m_ComponentBBox[i][0];
		original_position[i][1] = m_ComponentBBox[i][1];
		scale_position[i][0] = m_ComponentBBox[i][0] * scale[0];
		scale_position[i][1] = m_ComponentBBox[i][1] * scale[1];
		original_scale[i][0] = m_ComponentBBox[i][2];
		original_scale[i][1] = m_ComponentBBox[i][3];
	}
	m_Optimizer.ImportOriginalPosition(scale_position);
	m_Optimizer.ImportOriginalScale(original_scale);
	m_Optimizer.ImportScaleThreshold(m_ScaleThreshold);
	m_Optimizer.ImportAffiliatedPair(m_AffiliatedPair, m_AffiliatedVertexID);
	m_Optimizer.ImportApartBBoxPair(m_ApartPair, m_ApartDistance);
	m_Optimizer.ImportAdjacentPair(m_AdjacentPair);
	//m_Optimizer.ImportStructure(m_BBoxPositionStructure);
	m_Optimizer.ImportStructure(m_BBoxStructureGroup);
	m_Optimizer.ImportRegularityStructure(m_RegularityStructure);


	std::vector<trimesh::ivec2> observe_related_ID(m_VisibleConstrain.size(), trimesh::ivec2(0, 0));
	std::vector<std::vector<trimesh::vec2>> observe_field(m_VisibleConstrain.size(), std::vector<trimesh::vec2>(3, trimesh::vec2(0, 0)));
	std::vector<trimesh::vec2> target_point(m_VisibleConstrain.size(), trimesh::vec2(0, 0));
	std::vector<std::vector<int>> occlusion_related_ID(m_VisibleConstrain.size(), std::vector<int>());
	for (int i = 0; i < m_VisibleConstrain.size(); ++i)
	{
		observe_related_ID[i][0] = m_VisibleConstrain[i].observe_bbox_id;
		observe_related_ID[i][1] = m_VisibleConstrain[i].target_bbox_id;

		observe_field[i][0] = m_VisibleConstrain[i].observe_point_position;
		observe_field[i][1] = trimesh::vec2(m_VisibleConstrain[i].observe_field[0], m_VisibleConstrain[i].observe_field[1]);
		observe_field[i][2] = trimesh::vec2(m_VisibleConstrain[i].observe_field[2], m_VisibleConstrain[i].observe_field[3]);

		target_point[i] = m_VisibleConstrain[i].target_point_position;

		occlusion_related_ID[i] = m_VisibleConstrain[i].occlusion_bbox_id;
	}
	m_Optimizer.ImportVisibleInfo(observe_related_ID, observe_field, target_point, occlusion_related_ID);
}

void Scene::initeOptimizer_Old(double scale)
{
	std::vector<trimesh::vec2> original_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> scale_position(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> original_scale(m_ComponentBBox.size(), trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		original_position[i][0] = m_ComponentBBox[i][0];
		original_position[i][1] = m_ComponentBBox[i][1];
		scale_position[i][0] = m_ComponentBBox[i][0] * scale;
		scale_position[i][1] = m_ComponentBBox[i][1] * scale;
		original_scale[i][0] = m_ComponentBBox[i][2];
		original_scale[i][1] = m_ComponentBBox[i][3];
	}
	m_Optimizer.ImportOriginalPosition(scale_position);
	m_Optimizer.ImportOriginalScale(original_scale);
	m_Optimizer.ImportScaleThreshold(m_ScaleThreshold);
	m_Optimizer.ImportAffiliatedPair(m_AffiliatedPair, m_AffiliatedVertexID);
	m_Optimizer.ImportApartBBoxPair(m_ApartPair, m_ApartDistance);
	m_Optimizer.ImportAdjacentPair(m_AdjacentPair);
	//m_Optimizer.ImportStructure(m_BBoxPositionStructure);
	m_Optimizer.ImportStructure(m_BBoxStructureGroup);
	m_Optimizer.ImportRegularityStructure(m_RegularityStructure);

	// 	std::ofstream testGroupStructure("data/test/testGroupStucture.txt");
	// 	std::vector<trimesh::vec3> group_structure_distance(m_GroupPositionStructure.size(), trimesh::vec3(0, 0, 0));
	// 	for (int i = 0; i < m_GroupPositionStructure.size(); ++i)
	// 	{
	// 		trimesh::vec3 triangle_edge;
	// 
	// 		triangle_edge[0] = (original_position[m_GroupPositionStructure[i][1]] - original_position[m_GroupPositionStructure[i][0]]).length();
	// 		triangle_edge[1] = (original_position[m_GroupPositionStructure[i][2]] - original_position[m_GroupPositionStructure[i][1]]).length();
	// 		triangle_edge[2] = (original_position[m_GroupPositionStructure[i][0]] - original_position[m_GroupPositionStructure[i][2]]).length();
	// 		group_structure_distance.push_back(triangle_edge);
	// 		testGroupStructure << m_GroupPositionStructure[i][0] << " " << m_GroupPositionStructure[i][1] << " " << m_GroupPositionStructure[i][2] << "   ";
	// 		testGroupStructure << triangle_edge[0] << " " << triangle_edge[1] << " " << triangle_edge[2] << std::endl;
	// 	}
	// 	m_Optimizer.ImportGroupStructure(m_GroupPositionStructure, group_structure_distance);


// 	std::vector<trimesh::ivec2> observe_related_ID(m_VisibleConstrain.size(), trimesh::ivec2(0, 0));
// 	std::vector<trimesh::vec3> observe_field(m_VisibleConstrain.size(), trimesh::vec3(0, 0, 0));
// 	for (int i = 0; i < m_VisibleConstrain.size(); ++i)
// 	{
// 		observe_related_ID[i][0] = m_VisibleConstrain[i].observe_position_id;
// 		observe_related_ID[i][1] = m_VisibleConstrain[i].target_id;
// 		observe_field[i][0] = m_VisibleConstrain[i].observe_direction[0];
// 		observe_field[i][1] = m_VisibleConstrain[i].observe_direction[1];
// 		observe_field[i][2] = m_VisibleConstrain[i].observe_field;
// 	}
// 	m_Optimizer.ImportVisibleInfo(observe_related_ID, observe_field);

	//m_Optimizer.testFunction_phi();
	//m_Optimizer.testFunctionBarrier();

}

void Scene::initeRetargetting(double scale)
{
	int bbox_num = m_ComponentBBox.size();
	std::vector<double> initeValue(bbox_num * 4);
	for (int i = 0; i < bbox_num; ++i)
	{
		initeValue[i * 4] = m_ComponentBBox[i][0] * scale;
		initeValue[i * 4 + 1] = m_ComponentBBox[i][1] * scale;
		initeValue[i * 4 + 2] = m_ComponentBBox[i][2] * scale;
		initeValue[i * 4 + 3] = m_ComponentBBox[i][3] * scale;
	}

	/////////////initial 0.6//////////////

//		initeValue[45 * 4] = 5.3;
// 

// 	initeValue[37 * 4] = 0.93;
// 	initeValue[37 * 4 + 2] = 0.8;
// 
//   	initeValue[51 * 4] = 6.1;
//  	  	initeValue[51 * 4 + 2] = 0.1;
//  	  	initeValue[52 * 4] = 6.1;
//   	initeValue[52 * 4 + 2] = 0.1;

	/////////////initial 0.6//////////////

	/////////////initial 0.70//////////////
//   		initeValue[37 * 4] = 0.94;
//    		initeValue[37 * 4 + 2] = 0.8;
// // // 
// 		// // 
// 	   		initeValue[43 * 4] = 4.42;
// 	   	  	initeValue[43 * 4+2] = 0.5;
//    		//initeValue[45 * 4] = 5.7;
//    	  	initeValue[51 * 4] = 7.1;
//    	  	initeValue[51 * 4 + 2] = 0.1;
//    	  	initeValue[52 * 4] = 7.1;
//    	 	initeValue[52 * 4 + 2] = 0.1;
	/////////////initial 0.65//////////////
	//initeValue[34 * 4 + 3] = 0.4;

	initeValue[37 * 4] = 0.5+0.2*0.45;
	initeValue[37 * 4 + 2] = 0.5;
	// // 
	// // 
	//initeValue[43 * 4] = 0.5 + 5.6*0.75;
	//initeValue[43 * 4+2] = 0.5;
	initeValue[45 * 4] = 3.98;
	initeValue[51 * 4] = 0.05 + 10 * 0.45;
	initeValue[51 * 4 + 2] = 0.05;
	initeValue[52 * 4] = 0.05 + 10 * 0.45;
	initeValue[52 * 4 + 2] = 0.05;

		/////////////initial 0.65 linear//////////////
// 		initeValue[39 * 4] = 0.93;
// 		initeValue[39 * 4 + 2] = 0.8;
// 		// // 
// 		// // 
// 		//initeValue[43 * 4] = 4.14;
// 		//initeValue[43 * 4+2] = 0.5;
// 		initeValue[46 * 4 + 2] = 0.161;
// 
// 		initeValue[47 * 4] = 5.7;
// 		initeValue[53 * 4] = 6.6;
// 		initeValue[53 * 4 + 2] = 0.1;
// 		initeValue[54 * 4] = 6.6;
// 		initeValue[54 * 4 + 2] = 0.1;

// 		// 	/////////////initial 0.45//////////////
// 
// 		initeValue[27 * 4] = 2.6;
// 		initeValue[27 * 4 + 1] = 2.7;
// 		initeValue[27 * 4+2] = 0.1;
// 		initeValue[27 * 4 + 3] = 0.1;
// 
// 		initeValue[37 * 4] = 0.49;
// 		initeValue[37 * 4 + 2] = 0.4;
// 		// // 
// 		// // 
// 		initeValue[43 * 4] = 2.82;
// 		initeValue[43 * 4+2] = 0.3;
// 		//initeValue[44 * 4 + 2] = 0.161;
// 
// 		//initeValue[45 * 4] = 5.7;
// 		initeValue[51 * 4] = 4.51;
// 		initeValue[51 * 4 + 2] = 0.01;
// 		initeValue[52 * 4] = 4.51;
// 		initeValue[52 * 4 + 2] = 0.01;

	/////////////initial 0.7//////////////
// 	 		initeValue[37 * 4] = 0.94;
// 	  		initeValue[37 * 4 + 2] = 0.8;
// 	// // 
// 	// // 
// 	  		initeValue[45 * 4] = 6.23;
// 	  	  	initeValue[51 * 4] = 7.1;
// 	  	  	initeValue[51 * 4 + 2] = 0.1;
// 	  	  	initeValue[52 * 4] = 7.1;
// 	  	 	initeValue[52 * 4 + 2] = 0.1;
	/////////////initial 0.65//////////////

	int bbox_id1, bbox_id2;
	double inite_distance;
	for (int i = 0; i < m_ApartPair.size(); ++i)
	{
		for (int j = 0; j < m_ApartPair[i].size(); ++j)
		{
			bbox_id1 = m_ApartPair[i][j][0];
			bbox_id2 = m_ApartPair[i][j][1];
			trimesh::vec4 bbox1(initeValue[bbox_id1 * 4], initeValue[bbox_id1 * 4 + 1], initeValue[bbox_id1 * 4 + 2], initeValue[bbox_id1 * 4 + 3]);
			trimesh::vec4 bbox2(initeValue[bbox_id2 * 4], initeValue[bbox_id2 * 4 + 1], initeValue[bbox_id2 * 4 + 2], initeValue[bbox_id2 * 4 + 3]);
			int apart_mode;
			inite_distance = m_Optimizer.BBoxDistance(bbox1, bbox2, apart_mode);
			
			if (inite_distance < m_ApartDistance[i])
			{
				trimesh::vec2 bbox_center_dis(std::abs(bbox1[0] - bbox2[0]),std::abs(bbox1[1] - bbox2[1]));

				if (apart_mode == 0)
				{
					if (bbox_center_dis[0] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}
				if (apart_mode == 1)
				{
					if (bbox_center_dis[1] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}
				if (apart_mode == 2)
				{
					if (bbox_center_dis[0] < m_ApartDistance[i] && bbox_center_dis[1] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}

				if (!isBBoxConstrained(bbox_id1, apart_mode))
				{
					if (!isBBoxConstrained(bbox_id2, apart_mode))
					{
						if (apart_mode == 0)
						{
							initeValue[bbox_id1 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox1[2];
							initeValue[bbox_id2 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox2[2];
							initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						}
						if (apart_mode == 1)
						{
							initeValue[bbox_id1 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox1[3];
							initeValue[bbox_id2 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox2[3];
							initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						}
						if (apart_mode == 2)
						{
							if (bbox_center_dis[0] > bbox_center_dis[1])
							{
								initeValue[bbox_id1 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox1[2];
								initeValue[bbox_id2 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox2[2];
								initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
								initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
							}
							else
							{
								initeValue[bbox_id1 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox1[3];
								initeValue[bbox_id2 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox2[3];
								initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
								initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
							}
						}
					}
					else
					{
						if (apart_mode == 0)
						{
							initeValue[bbox_id1 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox2[2];
							initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							if (initeValue[bbox_id1 * 4 + 2] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
						if (apart_mode == 1)
						{
							initeValue[bbox_id1 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox2[3];
							initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							if (initeValue[bbox_id1 * 4 + 3] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
						if (apart_mode == 2)
						{
							if (bbox_center_dis[0] > bbox_center_dis[1])
							{
								initeValue[bbox_id1 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox2[2];
								initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							}
							else
							{
								initeValue[bbox_id1 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox2[3];
								initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							}
							if (initeValue[bbox_id1 * 4 + 2] <= 0 || initeValue[bbox_id1 * 4 + 3] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
					}
				}
				else if (!isBBoxConstrained(bbox_id2, apart_mode))
				{
					if (apart_mode == 0)
					{
						initeValue[bbox_id2 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox1[2];
						initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						if (initeValue[bbox_id2 * 4 + 2] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
					if (apart_mode == 1)
					{
						initeValue[bbox_id2 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox1[3];
						initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						if (initeValue[bbox_id2 * 4 + 3] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
					if (apart_mode == 2)
					{
						if (bbox_center_dis[0] > bbox_center_dis[1])
						{
							initeValue[bbox_id2 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox1[2];
							initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						}
						else
						{
							initeValue[bbox_id2 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox1[3];
							initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						}
						if (initeValue[bbox_id2 * 4 + 2] <= 0 || initeValue[bbox_id2 * 4 + 3] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
				}
				else
				{
					std::cout << "INITE FAILED!!!!!!!!!!" <<"  bbox1id = "<<bbox_id1<<"   bbox2id = "<<bbox_id2<< std::endl;
				}
			}
		}
	}

	//initeValue[17 * 4 + 2] = 0.05;
	//initeValue[17 * 4 + 3] = 0.05;

	m_Optimizer.IniteValue(initeValue);
	DrawIniteScene(initeValue, "data/initeScene.png");
}

void Scene::initeRetargetting(trimesh::vec2 scale)
{
	int bbox_num = m_ComponentBBox.size();
	std::vector<double> initeValue(bbox_num * 4);
	for (int i = 0; i < bbox_num; ++i)
	{
		initeValue[i * 4] = m_ComponentBBox[i][0] * scale[0];
		initeValue[i * 4 + 1] = m_ComponentBBox[i][1] * scale[1];
		initeValue[i * 4 + 2] = m_ComponentBBox[i][2] * scale[0];
		initeValue[i * 4 + 3] = m_ComponentBBox[i][3] * scale[1];
	}

	/////////////initial 0.6//////////////

// 	initeValue[37 * 4] = 0.82;
// 	initeValue[37 * 4 + 2] = 0.7;
// 	// 
// 	  	initeValue[51 * 4] = 6.1;
// 	 	  	initeValue[51 * 4 + 2] = 0.1;
// 	 	  	initeValue[52 * 4] = 6.1;
// 	  	initeValue[52 * 4 + 2] = 0.1;

	/////////////initial 0.6//////////////

	/////////////initial 0.65//////////////
// 	initeValue[37 * 4] = 0.93;
// 	initeValue[37 * 4 + 2] = 0.8;
// 
// 
// 	//initeValue[45 * 4] = 5.7;
// 	initeValue[51 * 4] = 6.6;
// 	initeValue[51 * 4 + 2] = 0.1;
// 	initeValue[52 * 4] = 6.6;
// 	initeValue[52 * 4 + 2] = 0.1;
	/////////////initial 0.65//////////////

	/////////////initial 0.65, 0.87//////////////
// 	 	initeValue[37 * 4] = 0.72;
// 		initeValue[37 * 4 + 2] = 0.6;
// 
// 		//initeValue[43 * 4] = 3.738;
// 		//initeValue[43 * 4 + 2] = 0.49;
// 
// 	 		initeValue[45 * 4 + 2] = 0.1;
// 	// 
// 	 		initeValue[51 * 4] = 6.12;
// 	 		initeValue[51 * 4 + 2] = 0.12;
// 	 		initeValue[52 * 4] = 6.12;
// 	 		initeValue[52 * 4 + 2] = 0.12;

	/////////////initial obstacle//////////////
//  	 	 		initeValue[53 * 4] = 4.0;
//  				initeValue[53 * 4 + 1] = 3.667;
//  				initeValue[53 * 4 + 2] = 0.3;
//  				initeValue[53 * 4 + 3] = 0.3;
//  
//  				initeValue[51 * 4] = 20.0/3.0+0.12;
//  				initeValue[51 * 4 + 2] = 0.12;
//  				initeValue[52 * 4] = 20.0 / 3.0 + 0.12;
//  				initeValue[52 * 4 + 2] = 0.12;
//  
//  				initeValue[43 * 4] = 11.2/3.0+0.49;
//  				initeValue[43 * 4 + 2] = 0.49;
//  				initeValue[45 * 4 + 2] = 0.1;
//  
//  				initeValue[37 * 4] = 0.4 / 3.0 + 0.6;
//  				initeValue[37 * 4 + 2] = 0.6;
//  
//  				initeValue[24 * 4 + 1] = 8.2 / 3.0 + 0.05;
//  				initeValue[24 * 4 + 3] = 0.05;

	/////////////complex input//////////////
	initeValue[53 * 4] = 0.575;
	initeValue[53 * 4 + 1] = 5.333;
	initeValue[53 * 4 + 2] = 0.425;
	initeValue[53 * 4 + 3] = 0.5333;

	initeValue[54 * 4] = 8.20;
	initeValue[54 * 4 + 1] = 1.1 / 3.0;
	initeValue[54 * 4 + 2] = 0.20;
	initeValue[54 * 4 + 3] = 0.70 / 3.0;

	initeValue[51 * 4] = 7.62;
	initeValue[51 * 4 + 2] = 0.12;
	initeValue[52 * 4] = 7.62;
	initeValue[52 * 4 + 2] = 0.12;
	initeValue[50 * 4] = 11.8*0.75 - 0.12;
	initeValue[50 * 4 + 2] = 0.12;

	initeValue[43 * 4] = 4.69;
	initeValue[43 * 4 + 2] = 0.49;
	initeValue[45 * 4 + 2] = 0.1;

	initeValue[31 * 4] = 3.9 * 0.75 - 0.6;
	initeValue[31 * 4 + 2] = 0.6;

	initeValue[33 * 4] = 1.25 * 0.75 + 0.6;
	//initeValue[33 * 4 + 2] = 0.05;


	int bbox_id1, bbox_id2;
	double inite_distance;
	for (int i = 0; i < m_ApartPair.size(); ++i)
	{
		for (int j = 0; j < m_ApartPair[i].size(); ++j)
		{
			bbox_id1 = m_ApartPair[i][j][0];
			bbox_id2 = m_ApartPair[i][j][1];
			trimesh::vec4 bbox1(initeValue[bbox_id1 * 4], initeValue[bbox_id1 * 4 + 1], initeValue[bbox_id1 * 4 + 2], initeValue[bbox_id1 * 4 + 3]);
			trimesh::vec4 bbox2(initeValue[bbox_id2 * 4], initeValue[bbox_id2 * 4 + 1], initeValue[bbox_id2 * 4 + 2], initeValue[bbox_id2 * 4 + 3]);
			int apart_mode;
			inite_distance = m_Optimizer.BBoxDistance(bbox1, bbox2, apart_mode);

			if (inite_distance < m_ApartDistance[i])
			{
				trimesh::vec2 bbox_center_dis(std::abs(bbox1[0] - bbox2[0]), std::abs(bbox1[1] - bbox2[1]));

				if (apart_mode == 0)
				{
					if (bbox_center_dis[0] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}
				if (apart_mode == 1)
				{
					if (bbox_center_dis[1] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}
				if (apart_mode == 2)
				{
					if (bbox_center_dis[0] < m_ApartDistance[i] && bbox_center_dis[1] < m_ApartDistance[i])
					{
						std::cout << "INITE FAILED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
					}
				}

				if (!isBBoxConstrained(bbox_id1, apart_mode))
				{
					if (!isBBoxConstrained(bbox_id2, apart_mode))
					{
						if (apart_mode == 0)
						{
							initeValue[bbox_id1 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox1[2];
							initeValue[bbox_id2 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox2[2];
							initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						}
						if (apart_mode == 1)
						{
							initeValue[bbox_id1 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox1[3];
							initeValue[bbox_id2 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox2[3];
							initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						}
						if (apart_mode == 2)
						{
							if (bbox_center_dis[0] > bbox_center_dis[1])
							{
								initeValue[bbox_id1 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox1[2];
								initeValue[bbox_id2 * 4 + 2] = (bbox_center_dis[0] - m_ApartDistance[i]) / (bbox1[2] + bbox2[2]) * bbox2[2];
								initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
								initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
							}
							else
							{
								initeValue[bbox_id1 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox1[3];
								initeValue[bbox_id2 * 4 + 3] = (bbox_center_dis[1] - m_ApartDistance[i]) / (bbox1[3] + bbox2[3]) * bbox2[3];
								initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
								initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
							}
						}
					}
					else
					{
						if (apart_mode == 0)
						{
							initeValue[bbox_id1 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox2[2];
							initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							if (initeValue[bbox_id1 * 4 + 2] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
						if (apart_mode == 1)
						{
							initeValue[bbox_id1 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox2[3];
							initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							if (initeValue[bbox_id1 * 4 + 3] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
						if (apart_mode == 2)
						{
							if (bbox_center_dis[0] > bbox_center_dis[1])
							{
								initeValue[bbox_id1 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox2[2];
								initeValue[bbox_id1 * 4 + 2] = initeValue[bbox_id1 * 4 + 2] * 0.99;
							}
							else
							{
								initeValue[bbox_id1 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox2[3];
								initeValue[bbox_id1 * 4 + 3] = initeValue[bbox_id1 * 4 + 3] * 0.99;
							}
							if (initeValue[bbox_id1 * 4 + 2] <= 0 || initeValue[bbox_id1 * 4 + 3] <= 0)
							{
								std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
							}
						}
					}
				}
				else if (!isBBoxConstrained(bbox_id2, apart_mode))
				{
					if (apart_mode == 0)
					{
						initeValue[bbox_id2 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox1[2];
						initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						if (initeValue[bbox_id2 * 4 + 2] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
					if (apart_mode == 1)
					{
						initeValue[bbox_id2 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox1[3];
						initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						if (initeValue[bbox_id2 * 4 + 3] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
					if (apart_mode == 2)
					{
						if (bbox_center_dis[0] > bbox_center_dis[1])
						{
							initeValue[bbox_id2 * 4 + 2] = bbox_center_dis[0] - m_ApartDistance[i] - bbox1[2];
							initeValue[bbox_id2 * 4 + 2] = initeValue[bbox_id2 * 4 + 2] * 0.99;
						}
						else
						{
							initeValue[bbox_id2 * 4 + 3] = bbox_center_dis[1] - m_ApartDistance[i] - bbox1[3];
							initeValue[bbox_id2 * 4 + 3] = initeValue[bbox_id2 * 4 + 3] * 0.99;
						}
						if (initeValue[bbox_id2 * 4 + 2] <= 0 || initeValue[bbox_id2 * 4 + 3] <= 0)
						{
							std::cout << "INITE FAILED!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
						}
					}
				}
				else
				{
					std::cout << "INITE FAILED!!!!!!!!!!" << "  bbox1id = " << bbox_id1 << "   bbox2id = " << bbox_id2 << std::endl;
				}
			}
		}
	}

	m_Optimizer.IniteValue(initeValue);
	DrawIniteScene(initeValue, "data/initeScene.png");
}

void Scene::initeRetargettingNaive(double scale)
{
	int bbox_num = m_ComponentBBox.size();
	std::vector<double> initeValue(bbox_num * 4);
	for (int i = 0; i < bbox_num; ++i)
	{
		initeValue[i * 4] = m_ComponentBBox[i][0] * scale;
		initeValue[i * 4 + 1] = m_ComponentBBox[i][1] * scale;
		initeValue[i * 4 + 2] = m_ComponentBBox[i][2] * scale;
		initeValue[i * 4 + 3] = m_ComponentBBox[i][3] * scale;
	}

	m_Optimizer.IniteValue(initeValue);
	DrawIniteScene(initeValue, "data/initeScene.png");
}

void Scene::initeRetargetting(std::string inite_path)
{
	int value_num = m_ComponentBBox.size() * 4;
	std::vector<double> initeValue(value_num);

	std::fstream inite_file(inite_path);
	for (int i = 0; i < value_num; ++i)
	{
		inite_file >> initeValue[i];
	}

	m_Optimizer.IniteValue(initeValue);
	DrawIniteScene(initeValue, "data/initeScene.png");
	//DrawIniteSceneX(initeValue, "data/SceneX.png");
	DrawScaleScene("data/scaleScene.png", 0.8);
}

void Scene::Retarget()
{
	m_Optimizer.testVisibleConstrain("data/test/initVisible.txt");

	std::vector<double> retarget_energy(6, 0);

	//m_Optimizer.deleteRegularitySpecial();

  	m_Optimizer.OptimizeNewton(retarget_energy);

	std::ofstream regularity_process_file("data/test/regularity_delete.txt");
    	int optimize_stop_flag = 1;
    	while (optimize_stop_flag)
    	{
 		m_Optimizer.OptimizeRegularity(retarget_energy, optimize_stop_flag, regularity_process_file);
    	}
    	m_Optimizer.OptimizeNewton(retarget_energy);


	DrawIniteScene(m_Optimizer.getResult(), "data/resultScene.png");
	//DrawIniteSceneX(m_Optimizer.getResult(), "data/SceneX.png");
	m_Optimizer.OutputResult("data/result.txt");
	m_Optimizer.OutputPAndS("data/info.txt");
	m_Optimizer.testScaleThreshold("data/test/Scale.txt");
	m_Optimizer.testPathWidth("data/test/pathwidth.txt");
	m_Optimizer.testVisibleConstrain("data/test/finalVisible.txt");
}

void Scene::testEnergy(std::string value_path)
{
	m_Optimizer.testEnergy(value_path);
}

void Scene::DrawOriginalScene(std::string image_path)
{
	int sceneW, offsetW, sceneH, offsetH, sceneScale,line_width;
	
	sceneW = 1600;
	offsetW = 80;
	sceneH = 1200;
	offsetH = 60;
	sceneScale = 120;
	line_width = 3;
	cv::Scalar bgr_white(255, 255, 255);
	cv::Scalar bgr_black(0, 0, 0);
	cv::Scalar bgr_blue(255, 0, 0);
	cv::Scalar bgr_red(0, 0, 255);
	cv::Mat OriginalScene(sceneH, sceneW, CV_8UC3, bgr_white);

	std::vector<trimesh::vec2> component_vertex(4, trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		BBox2Vertex(m_ComponentBBox[i], component_vertex);
		for (int j = 0; j < 4; ++j)
		{
			cv::line(OriginalScene, vec2cvPoint(component_vertex[(j + 1) % 4], sceneScale, offsetW, offsetH),
				vec2cvPoint(component_vertex[j], sceneScale, offsetW, offsetH), bgr_black, line_width);
		}
		trimesh::vec2 boxcenter(m_ComponentBBox[i][0], m_ComponentBBox[i][1]);
		cv::circle(OriginalScene, vec2cvPoint(boxcenter, sceneScale, offsetW, offsetH), 5, bgr_red, -1);
	}

// 	for (int i = 0; i < m_BBoxPositionStructure.size(); ++i)
// 	{
// 		trimesh::vec2 t1(m_ComponentBBox[m_BBoxPositionStructure[i][0]][0], m_ComponentBBox[m_BBoxPositionStructure[i][0]][1]);
// 		trimesh::vec2 t2(m_ComponentBBox[m_BBoxPositionStructure[i][1]][0], m_ComponentBBox[m_BBoxPositionStructure[i][1]][1]);
// 		trimesh::vec2 t3(m_ComponentBBox[m_BBoxPositionStructure[i][2]][0], m_ComponentBBox[m_BBoxPositionStructure[i][2]][1]);
// 		cv::line(OriginalScene, vec2cvPoint(t1, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t2, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 		cv::line(OriginalScene, vec2cvPoint(t2, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t3, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 		cv::line(OriginalScene, vec2cvPoint(t3, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t1, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 	}

	cv::imwrite(image_path, OriginalScene);
}

void Scene::DrawOriginalSceneX(std::string image_path)
{
}

void Scene::DrawScaleScene(std::string image_path, double scale)
{
	int sceneW, offsetW, sceneH, offsetH, sceneScale, line_width;

	sceneW = 1600;
	offsetW = 50;
	sceneH = 1300;
	offsetH = 50;
	sceneScale = (int)(150 * scale);
	line_width = 4;
	cv::Scalar bgr_white(255, 255, 255);
	cv::Scalar bgr_black(0, 0, 0);
	cv::Scalar bgr_blue(255, 0, 0);
	cv::Scalar bgr_red(0, 0, 255);
	cv::Mat OriginalScene(sceneH, sceneW, CV_8UC3, bgr_white);

	std::vector<trimesh::vec2> component_vertex(4, trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		BBox2Vertex(m_ComponentBBox[i], component_vertex);
		for (int j = 0; j < 4; ++j)
		{
			cv::line(OriginalScene, vec2cvPoint(component_vertex[(j + 1) % 4], sceneScale, offsetW, offsetH),
				vec2cvPoint(component_vertex[j], sceneScale, offsetW, offsetH), bgr_black, line_width);
		}
		trimesh::vec2 boxcenter(m_ComponentBBox[i][0], m_ComponentBBox[i][1]);
		cv::circle(OriginalScene, vec2cvPoint(boxcenter, sceneScale, offsetW, offsetH), 5, bgr_red, -1);
	}

	for (int i = 0; i < m_BBoxPositionStructure.size(); ++i)
	{
		trimesh::vec2 t1(m_ComponentBBox[m_BBoxPositionStructure[i][0]][0], m_ComponentBBox[m_BBoxPositionStructure[i][0]][1]);
		trimesh::vec2 t2(m_ComponentBBox[m_BBoxPositionStructure[i][1]][0], m_ComponentBBox[m_BBoxPositionStructure[i][1]][1]);
		trimesh::vec2 t3(m_ComponentBBox[m_BBoxPositionStructure[i][2]][0], m_ComponentBBox[m_BBoxPositionStructure[i][2]][1]);
		cv::line(OriginalScene, vec2cvPoint(t1, sceneScale, offsetW, offsetH),
			vec2cvPoint(t2, sceneScale, offsetW, offsetH), bgr_blue, 2);
		cv::line(OriginalScene, vec2cvPoint(t2, sceneScale, offsetW, offsetH),
			vec2cvPoint(t3, sceneScale, offsetW, offsetH), bgr_blue, 2);
		cv::line(OriginalScene, vec2cvPoint(t3, sceneScale, offsetW, offsetH),
			vec2cvPoint(t1, sceneScale, offsetW, offsetH), bgr_blue, 2);
	}
	cv::imwrite(image_path, OriginalScene);
}

void Scene::DrawIniteScene(std::vector<double> initeValue, std::string image_path)
{
	int sceneW, offsetW, sceneH, offsetH, sceneScale, line_width;

	sceneW = 1600;
	offsetW = 80;
	sceneH = 1200;
	offsetH = 60;
	sceneScale = 120;
	line_width = 3;
	cv::Scalar bgr_white(255, 255, 255);
	cv::Scalar bgr_black(0, 0, 0);
	cv::Scalar bgr_blue(255, 0, 0);
	cv::Scalar bgr_red(0, 0, 255);
	cv::Mat IniteScene(sceneH, sceneW, CV_8UC3, bgr_white);

	std::vector<trimesh::vec2> component_vertex(4, trimesh::vec2(0, 0));
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		trimesh::vec4 tempBBox(initeValue[i * 4], initeValue[i * 4 + 1], initeValue[i * 4 + 2], initeValue[i * 4 + 3]);
		BBox2Vertex(tempBBox, component_vertex);
		for (int j = 0; j < 4; ++j)
		{
			cv::line(IniteScene, vec2cvPoint(component_vertex[(j + 1) % 4], sceneScale, offsetW, offsetH),
				vec2cvPoint(component_vertex[j], sceneScale, offsetW, offsetH), bgr_black, line_width);
		}
		trimesh::vec2 boxcenter(initeValue[i * 4], initeValue[i * 4 + 1]);
		cv::circle(IniteScene, vec2cvPoint(boxcenter, sceneScale, offsetW, offsetH), 5, bgr_red, -1);
	
	}

	std::vector<trimesh::ivec2> observe_related_ID(m_VisibleConstrain.size(), trimesh::ivec2(0, 0));
	std::vector<std::vector<trimesh::vec2>> observe_field(m_VisibleConstrain.size(), std::vector<trimesh::vec2>(3, trimesh::vec2(0, 0)));
	std::vector<trimesh::vec2> target_point(m_VisibleConstrain.size(), trimesh::vec2(0, 0));
	std::vector<std::vector<int>> occlusion_related_ID(m_VisibleConstrain.size(), std::vector<int>());
	for (int i = 0; i<m_VisibleConstrain.size(); ++i)
	{
		observe_related_ID[i][0] = m_VisibleConstrain[i].observe_bbox_id;

		observe_field[i][0] = m_VisibleConstrain[i].observe_point_position;
		observe_field[i][1] = trimesh::vec2(m_VisibleConstrain[i].observe_field[0], m_VisibleConstrain[i].observe_field[1]);
		observe_field[i][2] = trimesh::vec2(m_VisibleConstrain[i].observe_field[2], m_VisibleConstrain[i].observe_field[3]);

		trimesh::vec4 observe_bbox(initeValue[observe_related_ID[i][0] * 4], initeValue[observe_related_ID[i][0] * 4 + 1],
			initeValue[observe_related_ID[i][0] * 4 + 2], initeValue[observe_related_ID[i][0] * 4 + 3]);

		std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0)); 
		for (int j = 0; j < 3; ++j)
		{
			region_v[j][0] = observe_bbox[0] + observe_bbox[2] * observe_field[i][j][0];
			region_v[j][1] = observe_bbox[1] + observe_bbox[3] * observe_field[i][j][1];
		}

		for (int j = 0; j < 3; ++j)
		{
			cv::line(IniteScene, vec2cvPoint(region_v[j], sceneScale, offsetW, offsetH),
				vec2cvPoint(region_v[(j + 1) % 3], sceneScale, offsetW, offsetH), bgr_blue, 2);
		}

	}

// 	for (int i = 0; i < m_BBoxPositionStructure.size(); ++i)
// 	{
// 		trimesh::vec2 t1(initeValue[m_BBoxPositionStructure[i][0] * 4], initeValue[m_BBoxPositionStructure[i][0] * 4 + 1]);
// 		trimesh::vec2 t2(initeValue[m_BBoxPositionStructure[i][1] * 4], initeValue[m_BBoxPositionStructure[i][1] * 4 + 1]);
// 		trimesh::vec2 t3(initeValue[m_BBoxPositionStructure[i][2] * 4], initeValue[m_BBoxPositionStructure[i][2] * 4 + 1]);
// 		cv::line(IniteScene, vec2cvPoint(t1, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t2, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 		cv::line(IniteScene, vec2cvPoint(t2, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t3, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 		cv::line(IniteScene, vec2cvPoint(t3, sceneScale, offsetW, offsetH),
// 			vec2cvPoint(t1, sceneScale, offsetW, offsetH), bgr_blue, 2);
// 	}

	cv::imwrite(image_path, IniteScene);
}

void Scene::DrawIniteSceneX(std::vector<double> initeValue, std::string image_path)
{
}

void Scene::DrawRetargetScene(std::string Scene_Path, std::string Attribute_Path, std::string Image_Path)
{
	std::ifstream Attribute_File(Attribute_Path);
	double sceneW, sceneH;
	int offsetW, offsetH, sceneScale, line_width;
	int canvaW, canvaH;
	cv::Scalar bgra_canva(255, 255, 255, 255);
	cv::Scalar bgra_box(255, 255, 255, 255);
	Attribute_File >> sceneW >> sceneH >> sceneScale >> offsetW >> offsetH >> line_width;
	Attribute_File >> bgra_canva[0] >> bgra_canva[1] >> bgra_canva[2] >> bgra_canva[3];
	Attribute_File >> bgra_box[0] >> bgra_box[1] >> bgra_box[2] >> bgra_box[3];

	canvaW = (int)(offsetW * 2 + sceneW * sceneScale);
	canvaH = (int)(offsetH * 2 + sceneH * sceneScale);

	std::cout << sceneW << " " << sceneH << " " << sceneScale << " " << offsetW << " " << offsetH << " " << line_width << std::endl;
	std::cout << canvaW << " " << canvaH << std::endl;
	std::cout << bgra_canva << std::endl;
	std::cout << bgra_box << std::endl;

	std::ifstream Scene_File(Scene_Path);
	int SceneElementNum;
	Scene_File >> SceneElementNum;
	std::cout << "bbox num = " << SceneElementNum << std::endl;
	std::vector<trimesh::vec4> SceneElement(SceneElementNum, trimesh::vec4(0, 0, 0, 0));
	for (int i = 0; i < SceneElementNum; ++i)
	{
		Scene_File >> SceneElement[i][0] >> SceneElement[i][1] >> SceneElement[i][2] >> SceneElement[i][3];
	}

	cv::Mat SceneResult(canvaH, canvaW, CV_8UC4, bgra_canva);

	std::vector<trimesh::vec2> component_vertex(4, trimesh::vec2(0, 0));
	for (int i = 0; i < SceneElementNum; ++i)
	{
		BBox2Vertex(SceneElement[i], component_vertex);
// 		std::cout << component_vertex[0] << " " << component_vertex[1] << " " << component_vertex[2] << " " << component_vertex[3] << std::endl;
// 		std::cout << vec2cvPoint(component_vertex[0], sceneScale, offsetW, offsetH) << " " 
// 			<< vec2cvPoint(component_vertex[1], sceneScale, offsetW, offsetH) << " " 
// 			<< vec2cvPoint(component_vertex[2], sceneScale, offsetW, offsetH) << " "
// 			<< vec2cvPoint(component_vertex[3], sceneScale, offsetW, offsetH) << std::endl;
		for (int j = 0; j < 4; ++j)
		{
			cv::line(SceneResult, vec2cvPoint(component_vertex[(j + 1) % 4], sceneScale, offsetW, offsetH),
				vec2cvPoint(component_vertex[j], sceneScale, offsetW, offsetH), bgra_box, line_width);
		}

	}
	
	cv::imwrite(Image_Path, SceneResult);
	//cv::imshow(Image_Path, SceneResult);
}

void Scene::detectAdjacent()
{
	int component_num = m_ComponentBBox.size();
	int adjacent_mod = -1;

	for (int i = 0; i < component_num; ++i)
	{
		for (int j = i + 1; j < component_num; ++j)
		{
			if (isAdjacent(m_ComponentBBox[i], m_ComponentBBox[j],adjacent_mod))
			{
				m_AdjacentPair.push_back(trimesh::ivec3(i, j, adjacent_mod));
			}
		}
	}
}

void Scene::detectAdjacentNoLineareConstraint()
{
	int component_num = m_ComponentBBox.size();
	int adjacent_mod = -1;

	m_ApartDistance.push_back(0.001);
	std::vector<trimesh::ivec2> adjacentPair;

	for (int i = 0; i < component_num; ++i)
	{
		for (int j = i + 1; j < component_num; ++j)
		{
			if (isAdjacent(m_ComponentBBox[i], m_ComponentBBox[j], adjacent_mod))
			{
				if (i < 24 && j < 24)
				{
					m_AdjacentPair.push_back(trimesh::ivec3(i, j, adjacent_mod));
				}
				else
				{
					adjacentPair.push_back(trimesh::ivec2(i, j));
					adjacentPair.push_back(trimesh::ivec2(j, i));
				}
			}
		}
	}

	m_ApartPair.push_back(adjacentPair);
}

void Scene::detectAffiliated()
{
	int component_num = m_ComponentBBox.size();
	std::vector<int> AffiliatedVertex;

	for (int i = 0; i < component_num; ++i)
	{
		for (int j = 0; j < component_num; ++j)
		{
			if (i == j)
			{
				continue;
			}
			if (isAffiliated(m_ComponentBBox[i], m_ComponentBBox[j], AffiliatedVertex))
			{
				m_AffiliatedPair.push_back(trimesh::ivec2(i, j));
				m_AffiliatedVertexID.push_back(AffiliatedVertex);
			}
		}
	}
}

void Scene::selectApart()
{
	m_ApartPair.clear();
	int component_num = m_ComponentBBox.size();
	int pair_num = component_num * (component_num - 1);
	std::vector<int> one_v(component_num, 1);
	std::vector<std::vector<int>>	apart_flag(component_num, one_v);
	for (int i = 0; i < m_AffiliatedPair.size(); ++i)
	{
		apart_flag[m_AffiliatedPair[i][0]][m_AffiliatedPair[i][1]] = 0;
	}
	for (int i = 0; i < m_AdjacentPair.size(); ++i)
	{
		apart_flag[m_AdjacentPair[i][0]][m_AdjacentPair[i][1]] = 0;
	}

	for (int i = 0; i < component_num; ++i)
	{
		for (int j = 0; j < component_num; ++j)
		{
			if (apart_flag[i][j] == 1)
			{
				if (i != j)
				{
					//m_ApartPair.push_back(trimesh::ivec2(i, j));
				}
			}
		}
	}
}

void Scene::construactBboxStructure()
{
	m_BBoxPositionStructure.clear();

	std::vector<trimesh::vec2> bbox_center;
	for (int i = 0; i < m_ComponentBBox.size(); ++i)
	{
		bbox_center.push_back(trimesh::vec2(m_ComponentBBox[i][0], m_ComponentBBox[i][1]));
	}
	Delaunay constructor(bbox_center);
	constructor.Process();
	m_BBoxPositionStructure = constructor.getTriangleGraph();
}

bool Scene::isAdjacent(trimesh::vec4 bbox1, trimesh::vec4 bbox2)
{
	trimesh::vec2 box_offset = trimesh::vec2(std::abs(bbox1[0] - bbox2[0]) - bbox1[2] - bbox2[2],
		std::abs(bbox1[1] - bbox2[1]) - bbox1[3] - bbox2[3]);
	if ((std::abs(box_offset[0]) < 0.0001) && (box_offset[1] < 0))
	{
		return true;
	}
	else if ((std::abs(box_offset[1]) < 0.0001) && (box_offset[0] < 0))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Scene::isAdjacent(trimesh::vec4 bbox1, trimesh::vec4 bbox2, int &adjacentMode)
{
	trimesh::vec2 box_offset = trimesh::vec2(std::abs(bbox1[0] - bbox2[0]) - bbox1[2] - bbox2[2],
		std::abs(bbox1[1] - bbox2[1]) - bbox1[3] - bbox2[3]);
	if ((std::abs(box_offset[0]) < 0.0001) && (box_offset[1] < 0))
	{
		if (bbox1[0] > bbox2[0])
		{
			adjacentMode = 0;
		}
		else
		{
			adjacentMode = 1;
		}
		return true;
	}
	else if ((std::abs(box_offset[1]) < 0.0001) && (box_offset[0] < 0))
	{
		if (bbox1[1] > bbox2[1])
		{
			adjacentMode = 2;
		}
		else
		{
			adjacentMode = 3;
		}
		return true;
	}
	else
	{
		return false;
	}
}

bool Scene::isAffiliated(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<int> &AffiliatedVertex)
{
	AffiliatedVertex.clear();

	std::vector<trimesh::vec2> bbox_v(4, trimesh::vec2(0, 0));
	bbox_v[0] = trimesh::vec2(bbox2[0] - bbox2[2], bbox2[1] - bbox2[3]);
	bbox_v[1] = trimesh::vec2(bbox2[0] + bbox2[2], bbox2[1] - bbox2[3]);
	bbox_v[2] = trimesh::vec2(bbox2[0] + bbox2[2], bbox2[1] + bbox2[3]);
	bbox_v[3] = trimesh::vec2(bbox2[0] - bbox2[2], bbox2[1] + bbox2[3]);

	for (int i = 0; i < 4; ++i)
	{
		if (inBox(bbox1, bbox_v[i]))
		{
			AffiliatedVertex.push_back(i);
		}
	}
	if (AffiliatedVertex.size() > 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Scene::inBox(trimesh::vec4 bbox, trimesh::vec2 p)
{
	trimesh::vec2 offset(std::abs(bbox[0] - p[0]) - bbox[2],
		std::abs(bbox[1] - p[1]) - bbox[3]);
	if ((offset[0] < -1e-6) && (offset[1] < -1e-6))
	//if ((offset[0] < 0) && (offset[1] < 0))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Scene::isBBoxConstrained(int bbox_id)
{
	for (int i = 0; i < m_AdjacentPair.size(); ++i)
	{
		if ((bbox_id == m_AdjacentPair[i][0]) || (bbox_id == m_AdjacentPair[i][1]))
		{
			return true;
		}
	}
	return false;
}

bool Scene::isBBoxConstrained(int bbox_id, int constraint_mode)
{
	for (int i = 0; i < m_AdjacentPair.size(); ++i)
	{
		if ((bbox_id == m_AdjacentPair[i][0]) || (bbox_id == m_AdjacentPair[i][1]))
		{
			if (((m_AdjacentPair[i][2])/2 == constraint_mode) || constraint_mode == 2)
			{
				return true;
			}
		}
	}
	return false;
}

void Scene::testInfo()
{
	std::ofstream test_info("data/test/testscene.txt");
	int com_num = m_ComponentBBox.size();
	test_info << "component num is " << com_num << std::endl;
	for (int i = 0; i < com_num; ++i)
	{
		test_info << m_ComponentBBox[i][0] << " " << m_ComponentBBox[i][1] << " " << m_ComponentBBox[i][2] << " " << m_ComponentBBox[i][3] << std::endl;
	}
	
	int pair_num = m_AdjacentPair.size();
	test_info << "adjacent num is " << pair_num << std::endl;
	for (int i = 0; i < pair_num; ++i)
	{
		test_info << m_AdjacentPair[i][0] << " " << m_AdjacentPair[i][1] << " " << m_AdjacentPair[i][2] << std::endl;
	}
	
	pair_num = m_AffiliatedPair.size();
	test_info << "affiliated num is " << pair_num << std::endl;
	for (int i = 0; i < pair_num; ++i)
	{
		test_info << m_AffiliatedPair[i][0] << " " << m_AffiliatedPair[i][1] << "  vertex are  ";
		for (int j = 0; j < m_AffiliatedVertexID[i].size(); ++j)
		{
			test_info << m_AffiliatedVertexID[i][j] << " ";
		}
		test_info << std::endl;
	}

	int class_num = m_ApartPair.size();
	test_info << "apart class num is " << class_num << std::endl;
	for (int i = 0; i < class_num; ++i)
	{
		pair_num = m_ApartPair[i].size();
		test_info << "apart distance is " << m_ApartDistance[i] << "  ";
		test_info << "apart class " << i << " has " << pair_num << " pairs" << std::endl;
		for (int j = 0; j < pair_num; ++j)
		{
			test_info << m_ApartPair[i][j][0] + 1 << " " << m_ApartPair[i][j][1] + 1 << std::endl;
		}
		test_info << std::endl;
	}
	
	class_num = m_BBoxStructureGroup.size();
	test_info << "structure group num = " << class_num << std::endl;
	for (int i = 0; i < class_num; ++i)
	{
		pair_num = m_BBoxStructureGroup[i].size();
		test_info << "group " << i << " has " << pair_num << "components" << std::endl;
		for (int j = 0; j < pair_num; ++j)
		{
			test_info << m_BBoxStructureGroup[i][j] << std::endl;
		}
	}
	test_info << std::endl;

	class_num = m_VisibleConstrain.size();
	test_info << "visible constraint num = " << class_num << std::endl;
	for (int i = 0; i < class_num; ++i)
	{
		test_info << "observe_id = " << m_VisibleConstrain[i].observe_bbox_id << "   observe_fild = " << m_VisibleConstrain[i].observe_point_position << " "
			<< m_VisibleConstrain[i].observe_field << std::endl;
		test_info << "target_id = " << m_VisibleConstrain[i].target_bbox_id << "   target_position = " << m_VisibleConstrain[i].target_point_position << std::endl;
		test_info << "occlusion_id = ";
		for (int j = 0; j < m_VisibleConstrain[i].occlusion_bbox_id.size(); ++j)
		{
			test_info << m_VisibleConstrain[i].occlusion_bbox_id[j] << "  ";
		}
		test_info << std::endl;
	}
	test_info << std::endl;
}

void Scene::bbox2info(std::string bbox_path, std::string info_path)
{
	std::ifstream bbox_file(bbox_path);
	std::ofstream info_file(info_path);
	int object_num;
	bbox_file >> object_num;
	info_file << object_num << std::endl;
	trimesh::vec4 bbox(0, 0, 0, 0);
	trimesh::vec4 threshold(0, 0, 0, 0);
	for (int i = 0; i < object_num; ++i)
	{
		bbox_file >> bbox[0] >> bbox[1] >> bbox[2] >> bbox[3];
		bbox_file >> threshold[0] >> threshold[1] >> threshold[2] >> threshold[3];
		info_file << bbox[0] << " 0.0 " << bbox[1] << "		1.0 1.0 1.0" << std::endl;
	}
	info_file.close();
}

void Scene::organizeSubInfo(std::string result_path, std::string data_path, std::string output_path)
{
	std::ifstream result_file(result_path);
	std::ifstream data_file(data_path);
	std::ofstream out_file(output_path);

	int group_num;
	result_file >> group_num;

	trimesh::vec4 result_v(0, 0, 0, 0);
	trimesh::vec4 data_v(0, 0, 0, 0);

	for (int i = 0; i < group_num; ++i)
	{
		int object_num;
		double result_h;
		trimesh::vec2 origin_position(0, 0);
		trimesh::vec2 center_position(0, 0);
		trimesh::vec2 sub_orientation(0, 0);
		result_file >> object_num >> result_h >> origin_position[0] >> origin_position[1]
			>> center_position[0] >> center_position[1] >> sub_orientation[0] >> sub_orientation[1];

		for (int j = 0; j < object_num; ++j)
		{
			result_file >> result_v[0] >> result_v[1] >> result_v[2] >> result_v[3];
			data_file >> data_v[0] >> data_v[1] >> data_v[2] >> data_v[3];

			trimesh::vec2 offset(result_v[0] - center_position[0], result_v[1] - center_position[1]);
			if (std::abs(sub_orientation[0]) > 0.5)
			{
				if (sub_orientation[0] > 0)
				{
					offset[0] = result_v[0] - center_position[0];
					offset[1] = result_v[1] - center_position[1];
				}
				else
				{
					offset[0] = -result_v[0] + center_position[0];
					offset[1] = -result_v[1] + center_position[1];
				}
			}
			else
			{
				if (sub_orientation[1] > 0)
				{
					offset[0] = -result_v[1] + center_position[1];
					offset[1] = -result_v[0] + center_position[0];
				}
				else
				{
					offset[0] = result_v[1] - center_position[1];
					offset[1] = result_v[0] - center_position[0];
				}
			}

			out_file << offset[0] + origin_position[0] << " " << result_h << " " << offset[1] + origin_position[1] << std::endl;
			out_file << sub_orientation[0] << " " << sub_orientation[1] << std::endl;
			out_file << result_v[2] / data_v[2] << " 1.0 " << result_v[3] / data_v[3] << std::endl;
			out_file << "props/" << std::endl;
			out_file << std::endl;
		}
	}
}

void Scene::organizeSubInfo(std::string result_path, std::string data_path, std::string refrence_path, std::string output_path)
{
	std::ifstream result_file(result_path);
	std::ifstream data_file(data_path);
	std::ifstream reference_file(refrence_path);
	std::ofstream out_file(output_path);

	int group_num;
	result_file >> group_num;

	trimesh::vec4 result_v(0, 0, 0, 0);
	trimesh::vec4 data_v(0, 0, 0, 0);

	trimesh::vec3 reference_position(0, 0, 0);
	trimesh::vec2 reference_orientation(0, 0);
	trimesh::vec3 reference_scale(0, 0, 0);

	for (int i = 0; i < group_num; ++i)
	{
		int object_num;
		double result_h;
		trimesh::vec2 origin_position(0, 0);
		trimesh::vec2 center_position(0, 0);
		trimesh::vec2 sub_orientation(0, 0);
		result_file >> object_num >> result_h >> origin_position[0] >> origin_position[1]
			>> center_position[0] >> center_position[1] >> sub_orientation[0] >> sub_orientation[1];

		for (int j = 0; j < object_num; ++j)
		{
			std::string object_string;
			reference_file >> reference_position[0] >> reference_position[1] >> reference_position[2];
			reference_file >> reference_orientation[0] >> reference_orientation[1];
			reference_file >> reference_scale[0] >> reference_scale[1] >> reference_scale[2];
			reference_file >> object_string;

			result_file >> result_v[0] >> result_v[1] >> result_v[2] >> result_v[3];
			data_file >> data_v[0] >> data_v[1] >> data_v[2] >> data_v[3];

			trimesh::vec2 offset(result_v[0] - center_position[0], result_v[1] - center_position[1]);
			if (std::abs(sub_orientation[0]) > 0.5)
			{
				if (sub_orientation[0] < 0)
				{
					offset[0] = -result_v[0] + center_position[0];
					offset[1] = result_v[1] - center_position[1];
				}
				else
				{
					offset[0] = result_v[0] - center_position[0];
					offset[1] = -result_v[1] + center_position[1];
				}
			}
			else
			{
				if (sub_orientation[1] > 0)
				{
					offset[0] = -result_v[1] + center_position[1];
					offset[1] = -result_v[0] + center_position[0];
				}
				else
				{
					offset[0] = result_v[1] - center_position[1];
					offset[1] = result_v[0] - center_position[0];
				}
			}

			out_file << offset[0] + origin_position[0] << " " << result_h << " " << offset[1] + origin_position[1] << std::endl;
			out_file << sub_orientation[0] << " " << sub_orientation[1] << std::endl;
		//	out_file << reference_orientation[0] << " " << reference_orientation[1] << std::endl;
			out_file << result_v[2] / data_v[2] << " 1.0 " << result_v[3] / data_v[3] << std::endl;
			out_file << object_string << std::endl;
			out_file << std::endl;
		}
	}
}

void Scene::reorganizeInfo(std::string info_path, std::string output_path)
{
	std::ifstream info_file(info_path);
	std::ofstream output(output_path);

	int object_num;
	info_file >> object_num;
	trimesh::vec3 position(0, 0, 0);
	trimesh::vec3 scale(0, 0, 0);
	for (int i = 0; i < object_num; ++i)
	{
		info_file >> position[0] >> position[1] >> position[2] >> scale[0] >> scale[1] >> scale[2];
		output << position[0] << " " << position[1] << " " << position[2] << std::endl;
		output << "1.0 0.0" << std::endl;
		output << scale[0] << " " << scale[1] << " " << scale[2] << std::endl;
		output << std::endl;
	}
}

void Scene::reorganizeInfo(std::string info_path, std::string reference_path, std::string output_path)
{
	std::ifstream info_file(info_path);
	std::ifstream reference_file(reference_path);
	std::ofstream output(output_path);

	int object_num;
	info_file >> object_num;
	trimesh::vec3 position(0, 0, 0);
	trimesh::vec3 scale(0, 0, 0);

	trimesh::vec3 reference_position(0, 0, 0);
	trimesh::vec2 reference_orientation(0, 0);
	trimesh::vec3 reference_scale(0, 0, 0);

	for (int i = 0; i < object_num; ++i)
	{
		std::string object_string;
		reference_file >> reference_position[0] >> reference_position[1] >> reference_position[2];
		reference_file >> reference_orientation[0] >> reference_orientation[1];
		reference_file >> reference_scale[0] >> reference_scale[1] >> reference_scale[2];
		reference_file >> object_string;

		info_file >> position[0] >> position[1] >> position[2] >> scale[0] >> scale[1] >> scale[2];
		output << position[0] << " " << position[1] << " " << position[2] << std::endl;
		output << reference_orientation[0] << " " << reference_orientation[1] << std::endl;;
		output << scale[0] << " " << scale[1] << " " << scale[2] << std::endl;
		output << object_string << std::endl;
		output << std::endl;
	}
}

void Scene::reorganizeInfo(std::string origin_path, std::string result_path, std::string reference_path, std::string output_path)
{
	std::ifstream origin_file(origin_path);
	std::ifstream result_file(result_path);
	std::ifstream reference_file(reference_path);
	std::ofstream output(output_path);

	int object_num;
	origin_file >> object_num;
	result_file >> object_num;
	trimesh::vec3 position(0, 0, 0);
	trimesh::vec4 threshold(0, 0, 0, 0);
	trimesh::vec3 origin_scale(0, 1, 0);
	trimesh::vec3 result_scale(0, 1, 0);

	trimesh::vec3 reference_position(0, 0, 0);
	trimesh::vec2 reference_orientation(0, 0);
	trimesh::vec3 reference_scale(0, 0, 0);

	for (int i = 0; i < object_num; ++i)
	{
		std::string object_string;
		reference_file >> reference_position[0] >> reference_position[1] >> reference_position[2];
		reference_file >> reference_orientation[0] >> reference_orientation[1];
		reference_file >> reference_scale[0] >> reference_scale[1] >> reference_scale[2];
		reference_file >> object_string;

		origin_file >> position[0] >> position[2] >> origin_scale[0] >> origin_scale[2];
		origin_file >> threshold[0] >> threshold[1] >> threshold[2] >> threshold[3];
		result_file >> position[0] >> position[2] >> result_scale[0] >> result_scale[2];
		result_file >> threshold[0] >> threshold[1] >> threshold[2] >> threshold[3];
		output << position[0] << " " << position[1] << " " << position[2] << std::endl;
		output << reference_orientation[0] << " " << reference_orientation[1] << std::endl;;
		output << result_scale[0] / origin_scale[0] << " " << result_scale[1] / origin_scale[1]
			<< " " << result_scale[2] / origin_scale[2] << std::endl;
		output << object_string << std::endl;
		output << std::endl;
	}
}

void Scene::scaleInfo(trimesh::vec3 scale_v, std::string info_path, std::string scale_path)
{
	std::ifstream info_file(info_path);
	std::ofstream scale_file(scale_path);

	std::string object_string;
	int object_num;
	info_file >> object_string;
	info_file >> object_num;
	scale_file << object_string << std::endl;
	scale_file << object_num << std::endl;
	scale_file << std::endl;

	trimesh::vec3 info_position(0, 0, 0);
	trimesh::vec2 info_orientation(0, 0);
	trimesh::vec3 info_scale(0, 0, 0);
	for (int i = 0; i < object_num; ++i)
	{
		info_file >> info_position[0] >> info_position[1] >> info_position[2];
		info_file >> info_orientation[0] >> info_orientation[1];
		info_file >> info_scale[0] >> info_scale[1] >> info_scale[2];
		info_file >> object_string;

		scale_file << info_position[0] * scale_v[0] << " " << info_position[1] * scale_v[1]
			<< " " << info_position[2] * scale_v[2] << std::endl;
		scale_file << info_orientation[0] << " " << info_orientation[1] << std::endl;
		scale_file << scale_v[0] << " " << scale_v[1] << " " << scale_v[2] << std::endl;
		scale_file << object_string << std::endl;
		scale_file << std::endl;
	}
}

void Scene::BBox2Vertex(trimesh::vec4 bbox, std::vector<trimesh::vec2> &vertex)
{
	vertex[0][0] = bbox[0] - bbox[2];
	vertex[0][1] = bbox[1] - bbox[3];
	vertex[1][0] = bbox[0] - bbox[2];
	vertex[1][1] = bbox[1] + bbox[3];
	vertex[2][0] = bbox[0] + bbox[2];
	vertex[2][1] = bbox[1] + bbox[3];
	vertex[3][0] = bbox[0] + bbox[2];
	vertex[3][1] = bbox[1] - bbox[3];
}

void Scene::BBox2Vertex2(trimesh::vec4 bbox, std::vector<trimesh::vec2> &vertex)
{
	vertex[0][0] = bbox[0] - bbox[2];
	vertex[0][1] = bbox[1] - bbox[3];
	vertex[1][0] = bbox[0] + bbox[2];
	vertex[1][1] = bbox[1] + bbox[3];
}

cv::Point Scene::vec2cvPoint(trimesh::vec2 p, int scale, int offset_w, int offset_h)
{
	return cv::Point(p[0] * scale + offset_w, p[1] * scale + offset_h);
}


void Scene::TVCarbinet(std::string SubPath)
{
	std::vector<trimesh::vec4> obj_init(3, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> obj_final(3, trimesh::vec4(0, 0, 0, 0));
	obj_final[0] = trimesh::vec4(0.96, 0.178, 0.50, 0.12);
	obj_final[1] = trimesh::vec4(0.20, 0.178, 0.15, 0.15);
	obj_final[2] = trimesh::vec4(1.72, 0.178, 0.15, 0.15);
	obj_init[0] = trimesh::vec4(0.96, 0.178, 0.4, 0.107);
	obj_init[1] = trimesh::vec4(0.32, 0.178, 0.133, 0.133);
	obj_init[2] = trimesh::vec4(1.6, 0.178, 0.133, 0.133);
	for (int i = 0; i < 300; ++i)
	{
		int id_a = i / 100;
		int id_b = (i - id_a * 100) / 10;
		int id_c = i % 10;

		std::string out_path = SubPath + "/" + std::to_string(id_a)
			+ std::to_string(id_b) + std::to_string(id_c) + ".txt";
		std::ofstream out_file(out_path);
		double ratio = (double)i / 300.0;
		out_file << "1.0 1.0 1.0 1.0" << std::endl;
		out_file << "1.0 1.0 1.0 1.0" << std::endl;
		out_file << "1.0 1.0 1.0 1.0" << std::endl;
		out_file << "1.0 1.0 1.0 1.0" << std::endl;
		for (int j = 0; j < 3; ++j)
		{
			for (int k = 0; k < 4; ++k)
			{
				out_file << obj_init[j][k] * (1.0 - ratio) + obj_final[j][k] * ratio << " ";
			}
			out_file << std::endl;
		}
		out_file.close();
	}
}

void Scene::SmoothProcess(std::string scene_data_path, std::string smooth_path)
{
	std::string scene_folder_path = scene_data_path + "/Scene";
	std::string subscene_folder_path = scene_data_path + "/SubScene";
	std::string smooth_scene_folder_path = smooth_path + "/Scene";
	std::string smooth_subscene_folder_path = smooth_path + "/SubScene";

	std::ifstream subscene_info_file(scene_data_path + "/Reference/subscene_info.txt");
	int subscene_num;
	subscene_info_file >> subscene_num;
	std::vector<int> subscene_id(subscene_num, 0);
	std::vector<int> subscene_obj_num(subscene_num, 0);
	for (int i = 0; i < subscene_num; ++i)
	{
		subscene_info_file >> subscene_id[i] >> subscene_obj_num[i];
		subscene_obj_num[i] = subscene_obj_num[i] + 4;
	}

	int id_a = 0;
	int id_b = 0;
	int id_c = 0;
	std::string scene_path = scene_folder_path + "/" + std::to_string(id_a)
		+ std::to_string(id_b) + std::to_string(id_c) + ".txt";
	std::ifstream init_scene_info_file(scene_path);
	std::vector<trimesh::vec4> scene_bbox1(53, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> scene_bbox2(53, trimesh::vec4(0, 0, 0, 0));
	for (int i = 0; i < 53; ++i)
	{
		init_scene_info_file >> scene_bbox1[i][0] >> scene_bbox1[i][1] >> scene_bbox1[i][2] >> scene_bbox1[i][3];
	}
	init_scene_info_file.close();

	std::string smooth_scene_path;
	int smooth_id = 0;
	int smooth_id_a = 0;
	int smooth_id_b = 0;
	int smooth_id_c = 0;
	int smooth_id_d = 0;

	for (int i = 1; i < 500; ++i)
	{
		id_a = i / 100;
		id_b = (i - id_a * 100) / 10;
		id_c = i % 10;
		scene_path = scene_folder_path + "/" + std::to_string(id_a)
			+ std::to_string(id_b) + std::to_string(id_c) + ".txt";
		std::ifstream scene_info_file(scene_path);
		for (int j = 0; j < 53; ++j)
		{
			scene_info_file >> scene_bbox2[j][0] >> scene_bbox2[j][1] >> scene_bbox2[j][2] >> scene_bbox2[j][3];
			//std::cout << scene_bbox2[j][0] << " " << scene_bbox2[j][1] << std::endl;
		}

		for (int j = 0; j < 6; ++j)
		{
			double smooth_ratio = (double)j / 6.0;
			smooth_id = (i - 1) * 6 + j;

			smooth_id_a = smooth_id / 1000;
			smooth_id_b = (smooth_id % 1000) / 100;
			smooth_id_c = (smooth_id % 100) / 10;
			smooth_id_d = smooth_id % 10;
			smooth_scene_path = smooth_scene_folder_path + "/" + std::to_string(smooth_id_a)
				+ std::to_string(smooth_id_b) + std::to_string(smooth_id_c) + std::to_string(smooth_id_d) + ".txt";
			std::ofstream smooth_scene_file(smooth_scene_path);
			for (int k = 0; k < 53; ++k)
			{
				smooth_scene_file << scene_bbox2[k][0] * smooth_ratio + scene_bbox1[k][0] * (1 - smooth_ratio) << " "
					<< scene_bbox2[k][1] * smooth_ratio + scene_bbox1[k][1] * (1 - smooth_ratio) << " "
					<< scene_bbox2[k][2] * smooth_ratio + scene_bbox1[k][2] * (1 - smooth_ratio) << " "
					<< scene_bbox2[k][3] * smooth_ratio + scene_bbox1[k][3] * (1 - smooth_ratio) << std::endl;
			}
			smooth_scene_file.close();
		}
		scene_bbox1 = scene_bbox2;
		subscene_info_file.close();
	}

	for (int a = 0; a < subscene_num; ++a)
	{
		id_a = 0;
		id_b = 0;
		id_c = 0;
		std::string subscene_path = subscene_folder_path + "/Sub" + std::to_string(subscene_id[a])+"/" + std::to_string(id_a)
			+ std::to_string(id_b) + std::to_string(id_c) + ".txt";
		std::ifstream init_subscene_info_file(subscene_path);
		std::vector<trimesh::vec4> subscene_bbox1(subscene_obj_num[a], trimesh::vec4(0, 0, 0, 0));
		std::vector<trimesh::vec4> subscene_bbox2(subscene_obj_num[a], trimesh::vec4(0, 0, 0, 0));
		for (int i = 0; i < subscene_obj_num[a]; ++i)
		{
			init_subscene_info_file >> subscene_bbox1[i][0] >> subscene_bbox1[i][1] >> subscene_bbox1[i][2] >> subscene_bbox1[i][3];
		}
		init_subscene_info_file.close();

		std::string smooth_subscene_path;
		smooth_id = 0;
		smooth_id_a = 0;
		smooth_id_b = 0;
		smooth_id_c = 0;
		smooth_id_d = 0;

		for (int i = 1; i < 300; ++i)
		{
			id_a = i / 100;
			id_b = (i - id_a * 100) / 10;
			id_c = i % 10;
			subscene_path = subscene_folder_path + "/Sub" + std::to_string(subscene_id[a]) + "/" + std::to_string(id_a)
				+ std::to_string(id_b) + std::to_string(id_c) + ".txt";
			std::ifstream subscene_info_file(subscene_path);
			for (int j = 0; j < subscene_obj_num[a]; ++j)
			{
				subscene_info_file >> subscene_bbox2[j][0] >> subscene_bbox2[j][1] >> subscene_bbox2[j][2] >> subscene_bbox2[j][3];
			}

			for (int j = 0; j < 7; ++j)
			{
				double smooth_ratio = (double)j / 7.0;
				smooth_id = (i - 1) * 7 + j;

				smooth_id_a = smooth_id / 1000;
				smooth_id_b = (smooth_id % 1000) / 100;
				smooth_id_c = (smooth_id % 100) / 10;
				smooth_id_d = smooth_id % 10;
				smooth_subscene_path = smooth_subscene_folder_path + "/Sub" + std::to_string(subscene_id[a]) + "/" + std::to_string(smooth_id_a)
					+ std::to_string(smooth_id_b) + std::to_string(smooth_id_c) + std::to_string(smooth_id_d) + ".txt";
				std::ofstream smooth_subscene_file(smooth_subscene_path);
				for (int k = 0; k < subscene_obj_num[a]; ++k)
				{
					smooth_subscene_file << subscene_bbox2[k][0] * smooth_ratio + subscene_bbox1[k][0] * (1 - smooth_ratio) << " "
						<< subscene_bbox2[k][1] * smooth_ratio + subscene_bbox1[k][1] * (1 - smooth_ratio) << " "
						<< subscene_bbox2[k][2] * smooth_ratio + subscene_bbox1[k][2] * (1 - smooth_ratio) << " "
						<< subscene_bbox2[k][3] * smooth_ratio + subscene_bbox1[k][3] * (1 - smooth_ratio) << std::endl;
				}
				smooth_subscene_file.close();
			}
			subscene_bbox1 = subscene_bbox2;
			subscene_info_file.close();
		}
	}
}

void Scene::RetargetProcess(std::string input_data_path, std::string output_path)
{
	std::string scene_folder_path = input_data_path + "/Scene";
	std::string subscene_folder_path = input_data_path + "/SubScene";
	std::string mergescene_folder_path = output_path;
	std::string scenefloor_folder_path = output_path + "/Floor";
	
	std::ifstream subscene_info_file(input_data_path + "/Reference/subscene_info.txt");
	int subscene_num;
	subscene_info_file >> subscene_num;
	std::vector<int> subscene_id(subscene_num, 0);
	std::vector<trimesh::vec2> subscene_orientation(subscene_num, trimesh::vec2(1, 0));
	std::vector<trimesh::vec2> subscene_aimbound(subscene_num, trimesh::vec2(0, 0));
	for (int i = 0; i < subscene_num; ++i)
	{
		subscene_info_file >> subscene_id[i];
		subscene_info_file >> subscene_orientation[i][0] >> subscene_orientation[i][1];
		subscene_info_file >> subscene_aimbound[i][0] >> subscene_aimbound[i][1];
	}

	std::ifstream scene_data_reference_file(input_data_path + "/Reference/scene_data_reference.txt");
	int scene_data_reference_num;
	scene_data_reference_file >> scene_data_reference_num;
	std::vector<trimesh::vec4> scene_data_reference(scene_data_reference_num, trimesh::vec4(0, 0, 0, 0));
	for (int i = 0; i < scene_data_reference_num; ++i)
	{
		scene_data_reference_file >> scene_data_reference[i][0] >> scene_data_reference[i][1] >> scene_data_reference[i][2] >> scene_data_reference[i][3];
	}

	std::ifstream subscene_data_reference_file(input_data_path + "/Reference/props_data_reference.txt");
	std::vector<std::vector<trimesh::vec4>> subscene_data_reference;
	for (int i = 0; i < subscene_num; ++i)
	{
		int props_num;
		subscene_data_reference_file >> props_num;
		std::vector<trimesh::vec4> props_data_reference(props_num, trimesh::vec4(0, 0, 0, 0));
		for (int j = 0; j < props_num; ++j)
		{
			subscene_data_reference_file >> props_data_reference[j][0] >> props_data_reference[j][1] >> props_data_reference[j][2] >> props_data_reference[j][3];
		}
		subscene_data_reference.push_back(props_data_reference);
	}

	std::ifstream special_furniture_file(input_data_path + "/Reference/special_furniture.txt");
	int special_furniture_num;
	special_furniture_file >> special_furniture_num;
	std::vector<trimesh::vec2> inite_scale(special_furniture_num, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> final_scale(special_furniture_num, trimesh::vec2(0, 0));

	for (int i = 0; i < special_furniture_num; ++i)
	{
		special_furniture_file >> inite_scale[i][0] >> inite_scale[i][1] >> final_scale[i][0] >> final_scale[i][1];
	}

	std::string info_reference_path = input_data_path + "/Reference/scene_info_reference.txt";

	for (int i = 0; i < 1; ++i)
	{
		std::cout << "merge process: " << i << std::endl;
		mergeSceneAndSubScene(i, scene_folder_path, subscene_folder_path, subscene_id, mergescene_folder_path, scenefloor_folder_path, scene_data_reference, subscene_data_reference,
			subscene_orientation, subscene_aimbound, inite_scale, final_scale, info_reference_path);
	}
}

void Scene::mergeSceneAndSubScene(int merge_id, std::string scene_folder_path, std::string subscene_folder_path,
	std::vector<int> subscene_id, std::string mergescene_folder_path, std::string scenefloor_folder_path,
	std::vector<trimesh::vec4> &scene_data_reference, std::vector<std::vector<trimesh::vec4>> &subscene_data_reference,
	std::vector<trimesh::vec2> subscene_orientation, std::vector<trimesh::vec2> subscene_aimbound, std::vector<trimesh::vec2> inite_scale,
	std::vector<trimesh::vec2> final_scale, std::string info_reference_path)
{
	int merge_id_a = merge_id / 1000;
	int merge_id_b = (merge_id % 1000) / 100;
	int merge_id_c = (merge_id % 100) / 10;
	int merge_id_d = merge_id % 10;

	int merge_sub_id = std::max(0, merge_id - 901);
	int merge_sub_id_a = merge_sub_id / 1000;
	int merge_sub_id_b = (merge_sub_id % 1000) / 100;
	int merge_sub_id_c = (merge_sub_id % 100) / 10;
	int merge_sub_id_d = merge_sub_id % 10;


	std::string scene_path = scene_folder_path + "/" + std::to_string(merge_id_a)
		+ std::to_string(merge_id_b) + std::to_string(merge_id_c) + std::to_string(merge_id_d) + ".txt";
	//std::cout << merge_id_a << " " << merge_id_b << " " << merge_id_c << " "<<scene_path;
	std::ifstream scene_info_file(scene_path);

	std::vector<std::vector<trimesh::vec4>> subscene_result;
	for (int i = 0; i < subscene_id.size(); ++i)
	{
		std::string temp_subscene_path = subscene_folder_path + "/Sub" + std::to_string(subscene_id[i]) + "/"
			+ std::to_string(merge_sub_id_a) + std::to_string(merge_sub_id_b) + std::to_string(merge_sub_id_c) + std::to_string(merge_sub_id_d) + ".txt";
		//std::cout << "subscene" << subscene_id[i] << " " << temp_subscene_path << std::endl;
		std::ifstream temp_subscene_file(temp_subscene_path);
		trimesh::vec4 temp_bbox(0, 0, 0, 0);
		std::vector<trimesh::vec4> temp_subscene;
		for (int j = 0; j < 4; ++j)
		{
			temp_subscene_file >> temp_bbox[0] >> temp_bbox[1] >> temp_bbox[2] >> temp_bbox[3];
		}
		for (int j = 0; j < subscene_data_reference[i].size(); ++j)
		{
			temp_subscene_file >> temp_bbox[0] >> temp_bbox[1] >> temp_bbox[2] >> temp_bbox[3];
			temp_subscene.push_back(temp_bbox);
		}
		//std::cout << temp_subscene.size() << std::endl;
		subscene_result.push_back(temp_subscene);
	}

	std::string merge_path = mergescene_folder_path + "/" + std::to_string(merge_id_a)
		+ std::to_string(merge_id_b) + std::to_string(merge_id_c) + std::to_string(merge_id_d) + ".txt";
	std::ofstream merge_file(merge_path);

	std::string scenefloor_path = scenefloor_folder_path + "/" + std::to_string(merge_id_a)
		+ std::to_string(merge_id_b) + std::to_string(merge_id_c) + std::to_string(merge_id_d) + ".txt";
	std::ofstream scenefloor_file(scenefloor_path);

	int object_num = 53;
	//scene_info_file >> object_num;
	
	std::vector<trimesh::vec4> wall_bbox(11, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> door_bbox(13, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> result_bbox(object_num - 24, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> furniture_bbox;

	scenefloor_file << 40 << " " << 53 << std::endl;

	for (int i = 0; i < 11; ++i)
	{
		scene_info_file >> wall_bbox[i][0] >> wall_bbox[i][1] >> wall_bbox[i][2] >> wall_bbox[i][3];

		scenefloor_file << wall_bbox[i][0] << " " << wall_bbox[i][1] << " " << wall_bbox[i][2] << " " << wall_bbox[i][3] << std::endl;
	}
	for (int i = 0; i < 13; ++i)
	{
		scene_info_file >> door_bbox[i][0] >> door_bbox[i][1] >> door_bbox[i][2] >> door_bbox[i][3];
	}
	for (int i = 0; i < object_num - 24; ++i)
	{
		scene_info_file >> result_bbox[i][0] >> result_bbox[i][1] >> result_bbox[i][2] >> result_bbox[i][3];

		scenefloor_file << result_bbox[i][0] << " " << result_bbox[i][1] << " " << result_bbox[i][2] << " " << result_bbox[i][3] << std::endl;
	}

	//generateWallandFloor(wall_bbox, door_bbox, merge_file);
	processSpecialFurniture(inite_scale, final_scale, (double)merge_id / 2994.0, result_bbox, furniture_bbox);

	for (int i = 0; i < subscene_id.size(); ++i)
	{
		trimesh::vec4 subscene_bound = furniture_bbox[subscene_id[i]];
		trimesh::vec2 subscene_o = subscene_orientation[i];
		trimesh::vec2 subscene_ab = subscene_aimbound[i];
		generateSubScene(subscene_bound, subscene_o, subscene_ab, subscene_result[i], subscene_data_reference[i], scenefloor_file);
	}

	for (int i = 0; i < furniture_bbox.size(); ++i)
	{
		furniture_bbox[i][2] = furniture_bbox[i][2] / scene_data_reference[i][2];
		furniture_bbox[i][3] = furniture_bbox[i][3] / scene_data_reference[i][3];
	}

	std::ifstream info_reference_file(info_reference_path);
	std::string info_head;
	info_reference_file >> info_head;
	merge_file << info_head << std::endl;
	int obj_num;
	info_reference_file >> obj_num;
	merge_file << obj_num + 22 << std::endl;
	merge_file << std::endl;

	generateWallandFloor(wall_bbox, door_bbox, merge_file);

	trimesh::vec3 reference_position(0, 0, 0);
	trimesh::vec2 reference_orientation(0, 0);
	trimesh::vec3 reference_scale(0, 0, 0);
	std::string reference_obj;
	for (int i = 0; i < furniture_bbox.size(); ++i)
	{
		info_reference_file >> reference_position[0] >> reference_position[1] >> reference_position[2];
		info_reference_file >> reference_orientation[0] >> reference_orientation[1];
		info_reference_file >> reference_scale[0] >> reference_scale[1] >> reference_scale[2];
		info_reference_file >> reference_obj;

		merge_file << furniture_bbox[i][0] << " " << reference_position[1] << " " << furniture_bbox[i][1] << std::endl;
		merge_file << reference_orientation[0] << " " << reference_orientation[1] << std::endl;;
		merge_file << furniture_bbox[i][2] << " " << 1.0 << " " << furniture_bbox[i][3] << std::endl;
		merge_file << reference_obj << std::endl;
		merge_file << std::endl;
	}
	for (int i = 0; i < subscene_id.size(); ++i)
	{
		for (int j = 0; j < subscene_result[i].size(); ++j)
		{
			info_reference_file >> reference_position[0] >> reference_position[1] >> reference_position[2];
			info_reference_file >> reference_orientation[0] >> reference_orientation[1];
			info_reference_file >> reference_scale[0] >> reference_scale[1] >> reference_scale[2];
			info_reference_file >> reference_obj;

			merge_file << subscene_result[i][j][0] << " " << reference_position[1] << " " << subscene_result[i][j][1] << std::endl;
			merge_file << reference_orientation[0] << " " << reference_orientation[1] << std::endl;;
			merge_file << subscene_result[i][j][2] << " " << 1.0 << " " << subscene_result[i][j][3] << std::endl;
			merge_file << reference_obj << std::endl;
			merge_file << std::endl;
		}
	}

	merge_file.close();
}

void Scene::processSpecialFurniture(std::vector<trimesh::vec2> inite_scale, std::vector<trimesh::vec2> final_scale, double progress_rate,
	std::vector<trimesh::vec4> result_bbox, std::vector<trimesh::vec4> &furniture_bbox)
{
	furniture_bbox.clear();
	std::vector<trimesh::vec2> special_size(6, trimesh::vec2(0, 0));
	// diningtable_size diningchair_size desk1_size chair1_size desk2_size chair2_size
	for (int i = 0; i < 6; ++i)
	{
		special_size[i] = trimesh::vec2(final_scale[i][0] * progress_rate + inite_scale[i][0] * (1 - progress_rate),
			final_scale[i][1] * progress_rate + inite_scale[i][1] * (1 - progress_rate));
	}


	for (int i = 0; i < 7; ++i)
	{
		furniture_bbox.push_back(result_bbox[i]);
	}
	trimesh::vec4 cupboard1_bbox(result_bbox[7][0] + result_bbox[7][2] * 5.0 / 12.0, result_bbox[7][1],
		result_bbox[7][2] * 7.0 / 12.0, result_bbox[7][3]);
	trimesh::vec4 sink_bbox(result_bbox[7][0] - result_bbox[7][2] * 7.0 / 12.0, result_bbox[7][1],
		result_bbox[7][2] * 5.0 / 12.0, result_bbox[7][3]);
	trimesh::vec4 stove_bbox(result_bbox[8][0], result_bbox[8][1] + result_bbox[8][3] * 0.5,
		result_bbox[8][2], result_bbox[8][3] * 0.5);
	trimesh::vec4 cupboard2_bbox(result_bbox[8][0], result_bbox[8][1] - result_bbox[8][3] * 0.5,
		result_bbox[8][2], result_bbox[8][3] * 0.5);
	furniture_bbox.push_back(cupboard1_bbox);
	furniture_bbox.push_back(sink_bbox);
	furniture_bbox.push_back(stove_bbox);
	furniture_bbox.push_back(cupboard2_bbox);
	furniture_bbox.push_back(result_bbox[9]);

	special_size[0][0] = std::min(special_size[0][0], result_bbox[10][2] * 0.9f);
	special_size[0][1] = std::min(special_size[0][1], result_bbox[10][3] * 0.9f);
	special_size[1][0] = std::min(special_size[0][0], result_bbox[10][2] * 0.48f);
	special_size[1][1] = std::min(special_size[0][1], result_bbox[10][3] * 0.24f);

	trimesh::vec4 dining_table(result_bbox[10][0], result_bbox[10][1], special_size[0][0], special_size[0][1]);
	trimesh::vec4 dining_chair1(result_bbox[10][0], result_bbox[10][1] - result_bbox[10][3] + special_size[1][1], special_size[1][0], special_size[1][1]);
	trimesh::vec4 dining_chair2(result_bbox[10][0] - result_bbox[10][2] + special_size[1][0], result_bbox[10][1] - special_size[1][1] * 1.02, special_size[1][0], special_size[1][1]);
	trimesh::vec4 dining_chair3(result_bbox[10][0] - result_bbox[10][2] + special_size[1][0], result_bbox[10][1] + special_size[1][1] * 1.02, special_size[1][0], special_size[1][1]);
	trimesh::vec4 dining_chair4(result_bbox[10][0], result_bbox[10][1] + result_bbox[10][3] - special_size[1][1], special_size[1][0], special_size[1][1]);
	trimesh::vec4 dining_chair5(result_bbox[10][0] + result_bbox[10][2] - special_size[1][0], result_bbox[10][1] + special_size[1][1] * 1.02, special_size[1][0], special_size[1][1]);
	trimesh::vec4 dining_chair6(result_bbox[10][0] + result_bbox[10][2] - special_size[1][0], result_bbox[10][1] - special_size[1][1] * 1.02, special_size[1][0], special_size[1][1]);
	
	
	furniture_bbox.push_back(dining_table);
	furniture_bbox.push_back(dining_chair1);
	furniture_bbox.push_back(dining_chair2);
	furniture_bbox.push_back(dining_chair3);
	furniture_bbox.push_back(dining_chair4);
	furniture_bbox.push_back(dining_chair5);
	furniture_bbox.push_back(dining_chair6);
	for (int i = 11; i < 18; ++i)
	{
		furniture_bbox.push_back(result_bbox[i]);
	}


	special_size[2][0] = std::min(special_size[2][0], result_bbox[18][2] * 0.9f);
	special_size[2][1] = std::min(special_size[2][1], result_bbox[18][3] * 0.9f);
	special_size[3][0] = std::min(special_size[3][0], special_size[2][0] * 0.42f);
	special_size[3][1] = std::min(special_size[3][1], special_size[2][0] * 0.42f);
	trimesh::vec4 desk1(result_bbox[18][0], result_bbox[18][1] + result_bbox[18][3] - special_size[2][1], result_bbox[18][2], special_size[2][1]);
	trimesh::vec4 chair1(result_bbox[18][0], result_bbox[18][1] - result_bbox[18][3] + special_size[3][1], special_size[3][0], special_size[3][1]);


	furniture_bbox.push_back(desk1);
	furniture_bbox.push_back(chair1);
	for (int i = 19; i < 27; ++i)
	{
		furniture_bbox.push_back(result_bbox[i]);
	}

	special_size[4][0] = std::min(special_size[4][0], result_bbox[27][2] * 0.9f);
	special_size[4][1] = std::min(special_size[4][1], result_bbox[27][3] * 0.9f);
	special_size[5][0] = std::min(special_size[5][0], special_size[4][1] * 0.42f);
	special_size[5][1] = std::min(special_size[5][1], special_size[4][1] * 0.42f);
	trimesh::vec4 desk2(result_bbox[27][0] - result_bbox[27][2] + special_size[4][0], result_bbox[27][1], special_size[4][0], result_bbox[27][3]);
	trimesh::vec4 chair2(result_bbox[27][0] + result_bbox[27][2] - special_size[5][0], result_bbox[27][1], special_size[5][0], special_size[5][1]);
	furniture_bbox.push_back(desk2);
	furniture_bbox.push_back(chair2);
	furniture_bbox.push_back(result_bbox[28]);
}

void Scene::generateWallandFloor(std::vector<trimesh::vec4> wall_bbox, std::vector<trimesh::vec4> door_bbox, std::ofstream &out_file)
{
	trimesh::vec4 floor1, floor2, floor3;
	std::vector<trimesh::vec4> wall_block_type1(12, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> wall_block_type2(4, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec4> wall_block_type3(3, trimesh::vec4(0, 0, 0, 0));

	double wall_block_type1_y = 1.04921;
	double wall_block_type1_h = 2.09842;
	double wall_block_type2_y = 2.697895;
	double wall_block_type2_h = 1.19895;
	double wall_block_type3_y = 1.64868;
	double wall_block_type3_h = 3.29737;

	floor1[0] = (wall_bbox[9][0] + wall_bbox[9][2] + (wall_bbox[3][0] + wall_bbox[3][2])) / 2.0;
	floor1[1] = (wall_bbox[2][1] + wall_bbox[2][3] + (wall_bbox[4][1] - wall_bbox[4][3])) / 2.0;
	floor1[2] = (wall_bbox[9][0] + wall_bbox[9][2] - (wall_bbox[3][0] + wall_bbox[3][2])) / 2.0;
	floor1[3] = (wall_bbox[2][1] + wall_bbox[2][3] - (wall_bbox[4][1] - wall_bbox[4][3])) / 2.0;

	floor2[0] = (wall_bbox[9][0] + wall_bbox[9][2] + (wall_bbox[3][0] + wall_bbox[3][2])) / 2.0;
	floor2[1] = (wall_bbox[4][1] - wall_bbox[4][3] + (wall_bbox[0][1] - wall_bbox[0][3])) / 2.0;
	floor2[2] = (wall_bbox[9][0] + wall_bbox[9][2] - (wall_bbox[3][0] + wall_bbox[3][2])) / 2.0;
	floor2[3] = (wall_bbox[4][1] - wall_bbox[4][3] - (wall_bbox[0][1] - wall_bbox[0][3])) / 2.0;

	floor3[0] = (wall_bbox[1][0] + wall_bbox[1][2] + (wall_bbox[9][0] + wall_bbox[9][2])) / 2.0;
	floor3[1] = (wall_bbox[2][1] + wall_bbox[2][3] + (wall_bbox[0][1] - wall_bbox[0][3])) / 2.0;
	floor3[2] = (wall_bbox[1][0] + wall_bbox[1][2] - (wall_bbox[9][0] + wall_bbox[9][2])) / 2.0;
	floor3[3] = (wall_bbox[2][1] + wall_bbox[2][3] - (wall_bbox[0][1] - wall_bbox[0][3])) / 2.0;


	wall_block_type1[0][0] = ((door_bbox[12][0] - door_bbox[12][2]) + (wall_bbox[2][0] - wall_bbox[2][2])) / 2.0;
	wall_block_type1[0][1] = wall_bbox[2][1];
	wall_block_type1[0][2] = ((door_bbox[12][0] - door_bbox[12][2]) - (wall_bbox[2][0] - wall_bbox[2][2])) / 2.0;
	wall_block_type1[0][3] = wall_bbox[2][3];

	wall_block_type1[1][0] = ((door_bbox[12][0] + door_bbox[12][2]) + (wall_bbox[2][0] + wall_bbox[2][2])) / 2.0;
	wall_block_type1[1][1] = wall_bbox[2][1];
	wall_block_type1[1][2] = (-(door_bbox[12][0] + door_bbox[12][2]) + (wall_bbox[2][0] + wall_bbox[2][2])) / 2.0;
	wall_block_type1[1][3] = wall_bbox[2][3];



	wall_block_type1[2][0] = ((door_bbox[2][0] - door_bbox[2][2]) + (wall_bbox[4][0] - wall_bbox[4][2])) / 2.0;
	wall_block_type1[2][1] = wall_bbox[4][1];
	wall_block_type1[2][2] = ((door_bbox[2][0] - door_bbox[2][2]) - (wall_bbox[4][0] - wall_bbox[4][2])) / 2.0;
	wall_block_type1[2][3] = wall_bbox[4][3];

	wall_block_type1[3][0] = ((door_bbox[2][0] + door_bbox[2][2]) + (door_bbox[6][0] - door_bbox[6][2])) / 2.0;
	wall_block_type1[3][1] = wall_bbox[4][1];
	wall_block_type1[3][2] = (-(door_bbox[2][0] + door_bbox[2][2]) + (door_bbox[6][0] - door_bbox[6][2])) / 2.0;
	wall_block_type1[3][3] = wall_bbox[4][3];

	wall_block_type1[4][0] = ((door_bbox[6][0] + door_bbox[6][2]) + (wall_bbox[4][0] + wall_bbox[4][2])) / 2.0;
	wall_block_type1[4][1] = wall_bbox[4][1];
	wall_block_type1[4][2] = (-(door_bbox[6][0] + door_bbox[6][2]) + (wall_bbox[4][0] + wall_bbox[4][2])) / 2.0;
	wall_block_type1[4][3] = wall_bbox[4][3];



	wall_block_type1[5][0] = wall_bbox[5][0]; 
	wall_block_type1[5][1] = ((door_bbox[0][1] - door_bbox[0][3]) + (wall_bbox[5][1] - wall_bbox[5][3])) / 2.0;
	wall_block_type1[5][2] = wall_bbox[5][2];
	wall_block_type1[5][3] = ((door_bbox[0][1] - door_bbox[0][3]) - (wall_bbox[5][1] - wall_bbox[5][3])) / 2.0;

	wall_block_type1[6][0] = wall_bbox[5][0];
	wall_block_type1[6][1] = ((door_bbox[0][1] + door_bbox[0][3]) + (wall_bbox[5][1] + wall_bbox[5][3])) / 2.0;
	wall_block_type1[6][2] = wall_bbox[5][2];
	wall_block_type1[6][3] = (-(door_bbox[0][1] + door_bbox[0][3]) + (wall_bbox[5][1] + wall_bbox[5][3])) / 2.0;



	wall_block_type1[7][0] = wall_bbox[8][0];
	wall_block_type1[7][1] = ((door_bbox[4][1] - door_bbox[4][3]) + (wall_bbox[8][1] - wall_bbox[8][3])) / 2.0;
	wall_block_type1[7][2] = wall_bbox[8][2];
	wall_block_type1[7][3] = ((door_bbox[4][1] - door_bbox[4][3]) - (wall_bbox[8][1] - wall_bbox[8][3])) / 2.0;

	wall_block_type1[8][0] = wall_bbox[8][0];
	wall_block_type1[8][1] = ((door_bbox[4][1] + door_bbox[4][3]) + (wall_bbox[8][1] + wall_bbox[8][3])) / 2.0;
	wall_block_type1[8][2] = wall_bbox[8][2];
	wall_block_type1[8][3] = (-(door_bbox[4][1] + door_bbox[4][3]) + (wall_bbox[8][1] + wall_bbox[8][3])) / 2.0;



	wall_block_type1[9][0] = wall_bbox[9][0];
	wall_block_type1[9][1] = ((door_bbox[10][1] - door_bbox[10][3]) + (wall_bbox[9][1] - wall_bbox[9][3])) / 2.0;
	wall_block_type1[9][2] = wall_bbox[9][2];
	wall_block_type1[9][3] = ((door_bbox[10][1] - door_bbox[10][3]) - (wall_bbox[9][1] - wall_bbox[9][3])) / 2.0;

	wall_block_type1[10][0] = wall_bbox[9][0];
	wall_block_type1[10][1] = ((door_bbox[10][1] + door_bbox[10][3]) + (door_bbox[8][1] - door_bbox[8][3])) / 2.0;
	wall_block_type1[10][2] = wall_bbox[9][2];
	wall_block_type1[10][3] = (-(door_bbox[10][1] + door_bbox[10][3]) + (door_bbox[8][1] - door_bbox[8][3])) / 2.0;

	wall_block_type1[11][0] = wall_bbox[9][0];
	wall_block_type1[11][1] = ((door_bbox[8][1] + door_bbox[8][3]) + (wall_bbox[9][1] + wall_bbox[9][3])) / 2.0;
	wall_block_type1[11][2] = wall_bbox[9][2];
	wall_block_type1[11][3] = (-(door_bbox[8][1] + door_bbox[8][3]) + (wall_bbox[9][1] + wall_bbox[9][3])) / 2.0;




	wall_block_type2[0] = wall_bbox[4];
	wall_block_type2[1] = wall_bbox[5];
	wall_block_type2[2] = wall_bbox[8];
	wall_block_type2[3] = wall_bbox[9];



	wall_block_type3[0] = wall_bbox[7];
	wall_block_type3[1] = wall_bbox[6];
	wall_block_type3[2] = wall_bbox[10];


	out_file << floor1[0] << " 0 " << floor1[1] << std::endl;
	out_file << "1 0" << std::endl;
	out_file << floor1[2] * 2.0 << " 1.0 " << floor1[3] * 2.0 << std::endl;
	out_file << "floor1.obj" << std::endl;
	out_file << std::endl;

	out_file << floor2[0] << " 0 " << floor2[1] << std::endl;
	out_file << "1 0" << std::endl;
	out_file << floor2[2] * 2.0 << " 1.0 " << floor2[3] * 2.0 << std::endl;
	out_file << "floor2.obj" << std::endl;
	out_file << std::endl;

	out_file << floor3[0] << " 0 " << floor3[1] << std::endl;
	out_file << "1 0" << std::endl;
	out_file << floor3[2] * 2.0 << " 1.0 " << floor3[3] * 2.0 << std::endl;
	out_file << "floor3.obj" << std::endl;
	out_file << std::endl;

	for (int i = 0; i < wall_block_type1.size(); ++i)
	{
		out_file << wall_block_type1[i][0] << " " << wall_block_type1_y << " " << wall_block_type1[i][1] << std::endl;
		out_file << "1 0" << std::endl;
		out_file << wall_block_type1[i][2] * 2.0 << " " << wall_block_type1_h << " " << wall_block_type1[i][3] * 2.0 << std::endl;
		out_file << "WallBlock.obj" << std::endl;
		out_file << std::endl;
	}
	for (int i = 0; i < wall_block_type2.size(); ++i)
	{
		out_file << wall_block_type2[i][0] << " " << wall_block_type2_y << " " << wall_block_type2[i][1] << std::endl;
		out_file << "1 0" << std::endl;
		out_file << wall_block_type2[i][2] * 2.0 << " " << wall_block_type2_h << " " << wall_block_type2[i][3] * 2.0 << std::endl;
		out_file << "WallBlock.obj" << std::endl;
		out_file << std::endl;
	}
	for (int i = 0; i < wall_block_type3.size(); ++i)
	{
		out_file << wall_block_type3[i][0] << " " << wall_block_type3_y << " " << wall_block_type3[i][1] << std::endl;
		out_file << "1 0" << std::endl;
		out_file << wall_block_type3[i][2] * 2.0 << " " << wall_block_type3_h << " " << wall_block_type3[i][3] * 2.0 << std::endl;
		out_file << "WallBlock.obj" << std::endl;
		out_file << std::endl;
	}

}

void Scene::generateSubScene(trimesh::vec4 subscene_bound, trimesh::vec2 subscene_oritation, trimesh::vec2 subscene_aim_bound, 
	std::vector<trimesh::vec4> &subscene_result, std::vector<trimesh::vec4> subscene_reference, std::ofstream &floor_file)
{
	
	trimesh::vec2 temp_scale(subscene_bound[2] / subscene_aim_bound[0], subscene_bound[3] / subscene_aim_bound[1]);

	for (int i = 0; i < subscene_result.size(); ++i)
	{
		trimesh::vec2 offset(0, 0);
		trimesh::vec4 bbox(0, 0, 0, 0);
		if (std::abs(subscene_oritation[0]) > 0.5)
		{
			if (subscene_oritation[0] < 0)
			{
				offset[0] = (-subscene_result[i][0] * temp_scale[0] + subscene_bound[2]);
				offset[1] = (subscene_result[i][1] * temp_scale[1] - subscene_bound[3]);
			}
			else
			{
				offset[0] = (subscene_result[i][0] * temp_scale[0] - subscene_bound[2]);
				offset[1] = (-subscene_result[i][1] * temp_scale[1] + subscene_bound[3]);
			}
			bbox[2] = subscene_result[i][2] * temp_scale[0];
			bbox[3] = subscene_result[i][3] * temp_scale[1];
			subscene_result[i][2] = subscene_result[i][2] / subscene_reference[i][2] * temp_scale[0];
			subscene_result[i][3] = subscene_result[i][3] / subscene_reference[i][3] * temp_scale[1];
		}
		else
		{
			if (subscene_oritation[1] > 0)
			{
				offset[0] = (-subscene_result[i][1] + subscene_aim_bound[0]) * temp_scale[0];
				offset[1] = (-subscene_result[i][0] + subscene_aim_bound[1]) * temp_scale[1];
			}
			else
			{
				offset[0] = (subscene_result[i][1] - subscene_aim_bound[0]) * temp_scale[0];
				offset[1] = (subscene_result[i][0] - subscene_aim_bound[1]) * temp_scale[1];
			}
			bbox[2] = subscene_result[i][3] * temp_scale[0];
			bbox[3] = subscene_result[i][2] * temp_scale[1];
			double temp_exchange = subscene_result[i][2] / subscene_reference[i][2] * temp_scale[1];
			subscene_result[i][2] = subscene_result[i][3] / subscene_reference[i][3] * temp_scale[0];
			subscene_result[i][3] = temp_exchange;
		}
		subscene_result[i][0] = subscene_bound[0] + offset[0];
		subscene_result[i][1] = subscene_bound[1] + offset[1];
		bbox[0] = subscene_result[i][0];
		bbox[1] = subscene_result[i][1];
		floor_file << bbox[0] << " " << bbox[1] << " " << bbox[2] << " " << bbox[3] << std::endl;
	}

}

// void Scene::generateProcessFloorVideo(std::string floor_folder_path)
// {
// 	std::string Attribute_Path = floor_folder_path + "/video_attribute.txt";
// 	std::ifstream Attribute_File(Attribute_Path);
// 	double sceneW, sceneH;
// 	int offsetW, offsetH, sceneScale, line_width;
// 	int canvaW, canvaH;
// 	int frame_num;
// 	cv::Scalar bgr_canva(255, 255, 255);
// 	cv::Scalar bgr_furniture(255, 255, 255);
// 	cv::Scalar bgr_props(255, 255, 255);
// 	Attribute_File >> sceneW >> sceneH >> sceneScale >> offsetW >> offsetH;
// 	Attribute_File >> bgr_canva[0] >> bgr_canva[1] >> bgr_canva[2];
// 	Attribute_File >> bgr_furniture[0] >> bgr_furniture[1] >> bgr_furniture[2];
// 	Attribute_File >> bgr_props[0] >> bgr_props[1] >> bgr_props[2];
// 	Attribute_File >> frame_num;
// 
// 	canvaW = (int)(offsetW * 2 + sceneW * sceneScale);
// 	canvaH = (int)(offsetH * 2 + sceneH * sceneScale);
// 
// 	std::cout << sceneW << " " << sceneH << " " << sceneScale << " " << offsetW << " " << offsetH  << std::endl;
// 	std::cout << canvaW << " " << canvaH << std::endl;
// 	std::cout << bgr_canva << std::endl;
// 	std::cout << bgr_furniture << std::endl;
// 	std::cout << bgr_props << std::endl;
// 	std::cout << frame_num << std::endl;
// 
// 	std::string video_path = floor_folder_path + "/floor.avi";
// 	cv::VideoWriter video_writer(video_path, CV_FOURCC('M', 'J', 'P', 'G'), 25.0, cv::Size(canvaW, canvaH));
// 
// 
// 	int frame_id_a, frame_id_b, frame_id_c, frame_id_d;
// 	int furniture_num, props_num;
// 	for (int i = 0; i < frame_num; ++i)
// 	{
// 		std::cout << "output frame:" << i << std::endl;
// 		frame_id_a = i / 1000;
// 		frame_id_b = (i % 1000) / 100;
// 		frame_id_c = (i % 100) / 10;
// 		frame_id_d = i % 10;
// 		std::string frame_info_path = floor_folder_path + "/" + std::to_string(frame_id_a) + std::to_string(frame_id_b)
// 			+ std::to_string(frame_id_c) + std::to_string(frame_id_d) + ".txt";
// 		std::ifstream frame_file(frame_info_path);
// 		frame_file >> furniture_num >> props_num;
// 
// 		cv::Mat CurrentFrame(canvaH, canvaW, CV_8UC3, bgr_canva);
// 		std::vector<trimesh::vec2> rect_vertex(2, trimesh::vec2(0, 0));
// 		trimesh::vec4 SceneElement(0, 0, 0, 0);
// 		for (int j = 0; j < furniture_num; ++j)
// 		{
// 			frame_file >> SceneElement[0] >> SceneElement[1] >> SceneElement[2] >> SceneElement[3];
// 			BBox2Vertex2(SceneElement, rect_vertex);
// 			cv::rectangle(CurrentFrame, vec2cvPoint(rect_vertex[0], sceneScale, offsetW, offsetH),
// 				vec2cvPoint(rect_vertex[1], sceneScale, offsetW, offsetH), bgr_furniture, -1);
// 		}
// 		for (int j = 0; j < props_num; ++j)
// 		{
// 			frame_file >> SceneElement[0] >> SceneElement[1] >> SceneElement[2] >> SceneElement[3];
// 			BBox2Vertex2(SceneElement, rect_vertex);
// 			cv::rectangle(CurrentFrame, vec2cvPoint(rect_vertex[0], sceneScale, offsetW, offsetH),
// 				vec2cvPoint(rect_vertex[1], sceneScale, offsetW, offsetH), bgr_props, -1);
// 		}
// 		video_writer << CurrentFrame;
// 
// 		//cv::imwrite(floor_folder_path + "/init.png", CurrentFrame);
// 	}
// 	//video_writer.release();
// }