#ifndef SCENE_H
#define SCENE_H

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cmath>
#include <ctime>
#include "Vec.h"
#include "Delaunay.h"
#include "Optimizer.h"
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>

struct VisibleInfo
{
	int					observe_bbox_id;
	trimesh::vec2		observe_point_position;
	trimesh::vec4		observe_field;
	int					target_bbox_id;
	trimesh::vec2		target_point_position;
	std::vector<int>	occlusion_bbox_id;
};

struct VisibleInfo_Old
{
	int					observe_type;
	// 0 present observe from a specific point, 1 present observe from a specific component
	trimesh::vec2		observe_position_point;
	int					observe_position_id;
	trimesh::vec2		observe_direction;
	double				observe_field;
	int					target_id;
};

class Scene
{
public:
	Scene();
	Scene(std::string scene_path);
	~Scene();

private:
	trimesh::vec2								m_SceneBound;
	std::vector<trimesh::vec4>					m_ComponentBBox;
	// vec4 for bbox, [0][1] is position of center, [2] is half of width, [3] is half of height
	std::vector<trimesh::vec4>					m_ScaleThreshold;
	std::vector<std::vector<trimesh::ivec2>>	m_ApartPair;
	std::vector<double>							m_ApartDistance;
	std::vector<trimesh::ivec3>					m_AdjacentPair;
	// adjacent mode, 0 is bbox1 left, 1 is bbox1 right, 2 is bbox1 down, 3 is bbox1 up
	std::vector<trimesh::ivec2>					m_AffiliatedPair;
	std::vector<std::vector<int>>				m_AffiliatedVertexID;
	// for vertexID, -1 represent no vertex in bbox, 0-3 represent lb, rb, rt, lt.
	std::vector<VisibleInfo>					m_VisibleConstrain;


	std::vector<trimesh::ivec3>					m_BBoxPositionStructure;
	std::vector<std::vector<int>>				m_BBoxStructureGroup;
	std::vector<trimesh::ivec3>					m_GroupPositionStructure;
	std::vector<trimesh::ivec3>					m_RegularityStructure;

	Optimizer									m_Optimizer;

public:
	void loadScene(std::string scene_path);
	void loadScene_Old(std::string scene_path);
	void loadBBoxStructure(std::string structure_path);
	void loadStructureGroup(std::string structure_path);
	void loadGroupStructure(std::string structure_path);
	void loadRegularityStructure(std::string structure_path);
	void initeOptimizer(double scale);
	void initeOptimizer_Old(double scale);
	void initeOptimizer(trimesh::vec2 scale);
	void testInfo();
	void bbox2info(std::string bbox_path, std::string info_path);
	void organizeSubInfo(std::string result_path, std::string data_path, std::string output_path);
	void organizeSubInfo(std::string result_path, std::string data_path, std::string reference_path, std::string output_path);
	void reorganizeInfo(std::string info_path, std::string output_path);
	void reorganizeInfo(std::string info_path, std::string reference_path, std::string output_path);
	void reorganizeInfo(std::string origin_path, std::string result_path, std::string reference_path, std::string output_path);
	void scaleInfo(trimesh::vec3 scale_v, std::string info_path, std::string scale_path);

	void TVCarbinet(std::string SubPath);
	void SmoothProcess(std::string scene_data_path, std::string smooth_path);
	void RetargetProcess(std::string input_data_path, std::string output_path);
	void mergeSceneAndSubScene(int merge_id, std::string scene_folder_path, std::string subscene_folder_path, std::vector<int> subscene_id,
		std::string mergescene_folder_path, std::string scenefloor_folder_path, std::vector<trimesh::vec4> &scene_data_reference,
		std::vector<std::vector<trimesh::vec4>> &subscene_data_reference,std::vector<trimesh::vec2> subscene_orientation, std::vector<trimesh::vec2> subscene_aimbound, 
		std::vector<trimesh::vec2> inite_scale, std::vector<trimesh::vec2> final_scale, std::string info_reference_path);
	void processSpecialFurniture(std::vector<trimesh::vec2> inite_scale, std::vector<trimesh::vec2> final_scale, double progress_rate,
		std::vector<trimesh::vec4> result_bbox, std::vector<trimesh::vec4> &furniture_bbox);
	void generateWallandFloor(std::vector<trimesh::vec4> wall_bbox, std::vector<trimesh::vec4> door_bbox, std::ofstream &out_file);
	void generateSubScene(trimesh::vec4 subscene_bound, trimesh::vec2 subscene_oritation, trimesh::vec2 subscene_aim_bound,
		std::vector<trimesh::vec4> &subscene_result, std::vector<trimesh::vec4> subscene_reference, std::ofstream &floor_file);

	/*void generateProcessFloorVideo(std::string floor_folder_path);*/

	void initeRetargetting(double scale);
	void initeRetargetting(trimesh::vec2 scale);
	void initeRetargettingNaive(double scale);
	void initeRetargetting(std::string inite_path);
	void Retarget();
	void testEnergy(std::string value_path);

	void DrawOriginalScene(std::string image_path);
	void DrawOriginalSceneX(std::string image_path);
	void DrawScaleScene(std::string image_path, double scale);
	void DrawIniteScene(std::vector<double> initeValue, std::string image_path);
	void DrawIniteSceneX(std::vector<double> initeValue, std::string image_path);

	void DrawRetargetScene(std::string Scene_Path, std::string Attribute_Path, std::string Image_Path);

private:
	void detectAdjacent();
	void detectAdjacentNoLineareConstraint();
	void detectAffiliated();
	void selectApart();
	void construactBboxStructure();

	bool isAdjacent(trimesh::vec4 bbox1, trimesh::vec4 bbox2);
	bool isAdjacent(trimesh::vec4 bbox1, trimesh::vec4 bbox2, int &adjacentMode);
	bool isAffiliated(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<int> &AffiliatedVertex);
	bool inBox(trimesh::vec4 bbox, trimesh::vec2 p);
	bool isBBoxConstrained(int bbox_id);
	bool isBBoxConstrained(int bbox_id, int constraint_mode);

	void BBox2Vertex(trimesh::vec4 bbox, std::vector<trimesh::vec2> &vertex);
	void BBox2Vertex2(trimesh::vec4 bbox, std::vector<trimesh::vec2> &vertex);
	cv::Point vec2cvPoint(trimesh::vec2 p, int scale, int offset_w, int offset_h);
};

#endif