#include "Optimizer.h"

Optimizer::Optimizer()
{
	m_WeightPosition = 200.0;
	//m_WeightPosition = 0;
	//m_WeightScale = 0.0002;
	m_WeightScale = 0.001;
	//m_WeightPathWidth = 0.001;
	m_WeightPathWidth = 0.0003;
	m_WeightStructure = 50.0;
	//m_WeightStructure = 0;
	m_WeightVisible = 0.001;

	m_WeightAffiliatePosition = 1;

	m_WeightVector = std::vector<double>(6, 0);
	m_WeightVector[0] = m_WeightPosition;
	m_WeightVector[1] = m_WeightScale;
	m_WeightVector[2] = m_WeightPathWidth;
	m_WeightVector[3] = m_WeightAffiliatePosition;
	m_WeightVector[4] = m_WeightStructure;
	m_WeightVector[5] = m_WeightVisible;

	m_phiDelta = 1e-6;
	m_barrierSigma = -6.0;
	m_distanceEpsilon = 1e-10;
	m_PathWidth = 0.25;

	m_MoveRangeR = 0.5;
	m_MoveExpS = 1.0;
	m_NeiborDistanceSquare = 9.0;
	m_barrierEpsilon = 0.03;
	m_barrierEpsilon2 = 0.03;

	m_deleteRegularityThreshold = 1e5;
	m_ScalePenalty = 10;
}

Optimizer::~Optimizer()
{

}

void Optimizer::IniteValue(std::vector<double> x)
{
	m_IniteValue = x;
	m_ResultValue = x;
}

void Optimizer::IniteValue(double scale)
{

}

void Optimizer::ImportOriginalPosition(std::vector<trimesh::vec2> p)
{
	m_OriginalPosition = p;
}

void Optimizer::ImportOriginalScale(std::vector<trimesh::vec2> s)
{
	m_OriginalScale = s;
	m_OriginalScaleStore = s;
}

void Optimizer::ImportScaleThreshold(std::vector<trimesh::vec4> t)
{
	m_ScaleThreshold = t;
	m_ScaleThresholdStore = t;
	m_DynamicScaleThreshold = t;
}

void Optimizer::ImportAffiliatedPair(std::vector<trimesh::ivec2> af_p, std::vector<std::vector<int>> af_v)
{
	m_AffiliatedPair = af_p;
	m_AffiliatedVertex = af_v;
}

void Optimizer::ImportApartBBoxPair(std::vector<std::vector<trimesh::ivec2>> ap_p, std::vector<double> ap_d)
{
	m_ApartBBoxPair = ap_p;
	m_ApartDistance = ap_d;
}
 
void Optimizer::ImportAdjacentPair(std::vector<trimesh::ivec3> ad_p)
{
	m_AdjacentPair = ad_p;
}

void Optimizer::ImportStructure(std::vector<trimesh::ivec3> s)
{

	std::ofstream testStructure("data/testStucture.txt");
	m_BBoxPositionStructure = s;

	m_BBoxStructureDistance.clear();
	trimesh::vec3 triangle_edge;
	
	for (int i = 0; i < m_BBoxPositionStructure.size(); ++i)
	{
		triangle_edge[0] = (m_OriginalPosition[m_BBoxPositionStructure[i][1]] - m_OriginalPosition[m_BBoxPositionStructure[i][0]]).length();
		triangle_edge[1] = (m_OriginalPosition[m_BBoxPositionStructure[i][2]] - m_OriginalPosition[m_BBoxPositionStructure[i][1]]).length();
		triangle_edge[2] = (m_OriginalPosition[m_BBoxPositionStructure[i][0]] - m_OriginalPosition[m_BBoxPositionStructure[i][2]]).length();
		m_BBoxStructureDistance.push_back(triangle_edge);

		testStructure << m_BBoxPositionStructure[i][0] << " " << m_BBoxPositionStructure[i][1] << " " << m_BBoxPositionStructure[i][2] << "   ";
		testStructure << triangle_edge[0] << " " << triangle_edge[1] << " " << triangle_edge[2] << std::endl; 
	}
}

void Optimizer::ImportStructure(std::vector<std::vector<int>> s_g)
{
	std::ofstream sf("data/test/sf.txt");

	m_StructurePair.clear();
	m_StructureDistanceSquare.clear();

	double point_dis_2 = 0;
	for (int i = 0; i < s_g.size(); ++i)
	{
		for (int j = 0; j < s_g[i].size(); ++j)
		{
			for (int k = j + 1; k < s_g[i].size(); ++k)
			{
				point_dis_2 = (m_OriginalPosition[s_g[i][j]][0] - m_OriginalPosition[s_g[i][k]][0]) * (m_OriginalPosition[s_g[i][j]][0] - m_OriginalPosition[s_g[i][k]][0])
					+ (m_OriginalPosition[s_g[i][j]][1] - m_OriginalPosition[s_g[i][k]][1]) * (m_OriginalPosition[s_g[i][j]][1] - m_OriginalPosition[s_g[i][k]][1]);

				if (point_dis_2 < m_NeiborDistanceSquare)
				{
					m_StructurePair.push_back(trimesh::ivec2(s_g[i][j], s_g[i][k]));
					m_StructureDistanceSquare.push_back(point_dis_2);

					sf << s_g[i][j] << " " << s_g[i][k] << " " << point_dis_2 << std::endl;
				}
			}
		}
	}
}

void Optimizer::ImportGroupStructure(std::vector<trimesh::ivec3> g_s, std::vector<trimesh::vec3> g_d)
{
	m_GroupPositionStructure = g_s;
	m_GroupStructureDistance = g_d;
}

void Optimizer::ImportVisibleInfo(std::vector<trimesh::ivec2> observe_related_ID, std::vector<std::vector<trimesh::vec2>> observe_field,
	std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_ID)
{
	m_ObserveRelatedID = observe_related_ID;
	m_ObserveField = observe_field;
	m_TargetPoint = target_point;
	m_OcclusionRelatedID = occlusion_related_ID;
}

void Optimizer::ImportRegularityStructure(std::vector<trimesh::ivec3> r)
{
	m_RegularityStructure = r;
}

void Optimizer::OptimizeNewton(std::vector<double> &optimal_energy)
{
	std::vector<double> x(m_IniteValue);
	std::vector<double> y(m_IniteValue);
	std::vector<double> dif(x.size(), 0);

// 	std::cout << "x size is " << x.size() << std::endl;
// 	for (int i = 0; i < x.size(); ++i)
// 	{
// 		std::cout << x[i] << "  ";
// 	}

	std::vector<std::vector<double>> hessian(x.size(), dif);
	std::vector<double> p(x.size(), 0);
	double tmax;
	double scale = 0.8;
	std::vector<double> energy(6, 0);
	double energy_sum = 0;
	double energy_delta = 0;
	trimesh::vec2 stepsize(2, 1e6);
	int max_time = 0;

	std::ofstream energy_out("data/energy.txt");
	//std::ofstream barrier_energy_test("data/test/barrier_energy.txt");
	//std::ofstream p_out("data/p.txt");
	//std::ofstream dif_out("data/dif.txt");
	//std::ofstream hessian_out("data/hessian.txt");
	//updateDynamicScaleThreshold(x);
	//ObjectFunction(x, energy, barrier_energy_test);

	updateScaleTolerance(x);

	ObjectFunction(x, energy);
	difObjectFunction(x, dif);
	HessianObjectFunction(x, hessian);

// 	for (int i = 0; i < x.size(); ++i)
// 	{
// 		dif_out << dif[i] << std::endl;
// 		for (int j = 0; j < x.size(); ++j)
// 		{
// 			hessian_out << hessian[i][j] << "  ";
// 		}
// 		hessian_out << std::endl;
// 	}

	for (int i = 0; i < energy.size(); ++i)
	{
		energy_sum = energy_sum + energy[i];
		//std::cout << energy[i] << ",  ";
	}
	//std::cout << " sum is " << energy_sum << std::endl;

	LinearConstraint(x.size());
	addSpecialLinearConstraint("data/Scene/specialConstraint.txt", x.size());
	ConstractNewtonMatrix(hessian);

// 	m_ResultValue = x;
// 	int result_file_num_a = result_file_num / 100;
// 	int result_file_num_b = (result_file_num - result_file_num_a * 100) / 10;
// 	std::string temp_result_name = "data/output/" + std::to_string(result_file_num_a)
// 		+ std::to_string(result_file_num_b) + std::to_string(result_file_num % 10) + ".txt";
// 	OutputResult(result_name);
// 	result_file_num++;


	int result_file_num = 0;
	while (1)
	{
		m_ResultValue = x;
		int result_file_num_a = (result_file_num+5) / 100;
		int result_file_num_b = ((result_file_num + 5) - result_file_num_a * 100) / 10;
		std::string result_name = "data/output/" + std::to_string(result_file_num_a)
			+ std::to_string(result_file_num_b) + std::to_string((result_file_num + 5) % 10) + ".txt";
		OutputResult(result_name);
		result_file_num++;

		if (max_time % 100 == 0)
		{
			std::cout << "iterate " << max_time << " times" << std::endl;
		}

		energy_out << "iterate time: " << max_time << "    energy = [ ";
		for (int i = 0; i < energy.size(); ++i)
		{
			energy_out << energy[i]<<" ";
		}
		energy_out << "],   sum = " << energy_sum << std::endl;

		max_time++;
		CalculateDescentDirection2(p, dif, hessian);
		//CalculateDescentDirectionLU(p, dif, hessian);

//  		for (int i = 0; i < p.size(); ++i)
//  		{
//  			p_out << p[i] << std::endl;
//  		}

		tmax = CalculateTmax(x, p);
		//tmax = 1.99;
		//LineSearch(x, p, tmax, stepsize, energy_sum, energy, y);
		double tempstepsize;
		//LineSearch(x, p, energy_sum, energy, tempstepsize, y);
		LineSearch(x, p, tmax, energy_sum, energy, tempstepsize, y);

		energy_out << "tmax = " << tmax << "          stepsize = " << tempstepsize << "  ";
		//energy_out <<"  stepsize = " << tempstepsize << "  ";

// 		energy_delta = energy_sum;
// 		for (int i = 0; i < energy.size(); ++i)
// 		{
// 			energy_delta = energy_delta - energy[i];
// 		}
// 		energy_out << "energy_delta = " << energy_delta << std::endl;

// 		if (std::abs(energy_delta) < 1e-4)
// 		{
// 			break;
// 		}
		if (max_time > 500)
		{
			break;
		}

//		energy_sum = energy_sum - energy_delta;

		x = y;
		//updateDynamicScaleThreshold(x);
		//ObjectFunction(x, energy, barrier_energy_test);

		updateScaleTolerance(x);

		ObjectFunction(x, energy);
		energy_sum = 0;
		for (int i = 0; i < energy.size(); ++i)
		{
			energy_sum = energy_sum + energy[i];
		}
		difObjectFunction(x, dif);
		HessianObjectFunction(x, hessian);

// 		m_ResultValue = x;
// 		int result_file_num_a = result_file_num / 100;
// 		int result_file_num_b = (result_file_num - result_file_num_a * 100) / 10;
// 		std::string result_name = "data/output/" + std::to_string(result_file_num_a)
// 			+ std::to_string(result_file_num_b) + std::to_string(result_file_num % 10) + ".txt";
// 		OutputResult(result_name);
// 		result_file_num++;
	}

	m_ResultValue = x;
	optimal_energy = energy;
}

void Optimizer::OptimizeRegularity(std::vector<double> &optimal_energy)
{

	double inite_energy_sum = 0;
	for (int i = 0; i < optimal_energy.size(); ++i)
	{
		inite_energy_sum += optimal_energy[i];
	}
	if (inite_energy_sum > m_deleteRegularityThreshold)
	{
		while (1)
		{
			if (m_RegularityStructure.size() == 0)
			{
				break;
			}

			std::vector<double> tempEnergy(6, 0);
			double tempEnergySum = 0;
			int delete_id;
			double min_energy = inite_energy_sum;
			for (int i = 0; i < m_RegularityStructure.size(); ++i)
			{
				deleteRegularity(m_RegularityStructure, i);
				OptimizeNewton(tempEnergy);
				tempEnergySum = 0;
				for (int j = 0; j < tempEnergy.size(); ++j)
				{
					tempEnergySum += tempEnergy[j];
				}
				if (tempEnergySum < min_energy)
				{
					min_energy = tempEnergySum;
					delete_id = i;
				}
				resetRegularity();
			}
			deleteRegularity(m_RegularityStructure, delete_id);

			m_OriginalScaleStore = m_OriginalScale;
			//m_ScaleThresholdStore = m_ScaleThreshold;

			std::cout << "delete regularity id = " << delete_id <<"    min energy = "<<min_energy<< std::endl;
			m_RegularityStructure.erase(m_RegularityStructure.begin() + delete_id);
			if (min_energy < m_deleteRegularityThreshold)
			{
				break;
			}
		}
	}

}

void Optimizer::OptimizeRegularity(std::vector<double> &optimal_energy, int &stop_flag)
{
	double penalty = 0;
	penalty = FunctionPenalty(m_ResultValue);
	std::cout << "penalty = " << penalty << std::endl;
	double min_penalty = penalty;

	int delete_id = 0;

	if (m_RegularityStructure.size() == 0)
	{
		stop_flag = 0;
	}
	if (penalty >= 100)
	{
		for (int i = 0; i < m_RegularityStructure.size(); ++i)
		{
			deleteRegularity(m_RegularityStructure, i);
			OptimizeNewton(optimal_energy);
			double temp_penalty = FunctionPenalty(m_ResultValue);
			if (temp_penalty<min_penalty)
			{
				min_penalty = temp_penalty;
				delete_id = i;
			} 
			std::cout << "temp id = " << i << "  temp penalty = " << temp_penalty << std::endl;
			resetRegularity();
		}

		if (penalty - min_penalty >= 50)
		{
			m_DeleteRegularityID.push_back(m_RegularityStructure[delete_id][0]);
			deleteRegularity(m_RegularityStructure, delete_id);
			OptimizeNewton(optimal_energy);
			double temp_penalty = FunctionPenalty(m_ResultValue);
			std::cout << "  current penalty = " << temp_penalty << std::endl;
			m_OriginalScaleStore = m_OriginalScale;
			m_RegularityStructure.erase(m_RegularityStructure.begin() + delete_id);
			stop_flag = 1;
			double temp_energy = 0;
			for (int j = 0; j < optimal_energy.size(); ++j)
			{
				temp_energy += optimal_energy[j];
			}
			std::cout << "delete id = " << delete_id << "  optimal energy = " << temp_energy << std::endl;
		}
		else
		{
			stop_flag = 0;
		}

		std::cout << "current deleted regularity structure has: " << std::endl;
		for (int i = 0; i < m_DeleteRegularityID.size(); ++i)
		{
			std::cout << m_DeleteRegularityID[i] << " ";
		}
		std::cout << std::endl;
	}
	
	else
	{
		stop_flag = 0;
	}

	if (m_RegularityStructure.size() == 0)
	{
		stop_flag = 0;
	}
}

void Optimizer::OptimizeRegularity(std::vector<double> &optimal_energy, int &stop_flag, std::ofstream &info_file)
{
	double penalty = 0;
	penalty = FunctionPenalty(m_ResultValue, info_file);
	info_file << "penalty = " << penalty << std::endl;
	std::cout << "penalty = " << penalty << std::endl;
	double min_penalty = penalty;

	int delete_id = 0;

	if (m_RegularityStructure.size() == 0)
	{
		stop_flag = 0;
	}
	if (penalty >= 100)
	{
		for (int i = 0; i < m_RegularityStructure.size(); ++i)
		{
			deleteRegularity(m_RegularityStructure, i);
			OptimizeNewton(optimal_energy);
			double temp_penalty = FunctionPenalty(m_ResultValue, info_file);
			if (temp_penalty < min_penalty)
			{
				min_penalty = temp_penalty;
				delete_id = i;
			}
			info_file << "temp id = " << i << "  temp penalty = " << temp_penalty << std::endl;
			std::cout << "temp id = " << i << "  temp penalty = " << temp_penalty << std::endl;
			resetRegularity();
		}

		if (penalty - min_penalty >= 40)
		{
			m_DeleteRegularityID.push_back(m_RegularityStructure[delete_id][0]);
			deleteRegularity(m_RegularityStructure, delete_id);
			OptimizeNewton(optimal_energy);
			double temp_penalty = FunctionPenalty(m_ResultValue, info_file);
			info_file << "  current penalty = " << temp_penalty << std::endl;
			std::cout << "  current penalty = " << temp_penalty << std::endl;
			m_OriginalScaleStore = m_OriginalScale;
			m_RegularityStructure.erase(m_RegularityStructure.begin() + delete_id);
			stop_flag = 1;
			double temp_energy = 0;
			for (int j = 0; j < optimal_energy.size(); ++j)
			{
				temp_energy += optimal_energy[j];
			}
			info_file << "delete id = " << delete_id << "  optimal energy = " << temp_energy << std::endl;
			std::cout << "delete id = " << delete_id << "  optimal energy = " << temp_energy << std::endl;
		}
		else
		{
			stop_flag = 0;
		}

		info_file << "current deleted regularity structure has: " << std::endl;
		std::cout<< "current deleted regularity structure has: " << std::endl;
		for (int i = 0; i < m_DeleteRegularityID.size(); ++i)
		{
			info_file << m_DeleteRegularityID[i] << " ";
			std::cout << m_DeleteRegularityID[i] << " ";
		}
		info_file << std::endl;
		std::cout << std::endl;
	}

	else
	{
		stop_flag = 0;
	}

	if (m_RegularityStructure.size() == 0)
	{
		stop_flag = 0;
	}
}

std::vector<double> Optimizer::getResult()
{
	return m_ResultValue;
}

void Optimizer::OutputResult(std::string result_path) 
{
	std::ofstream result_file(result_path);
	for (int i = 0; i < m_ResultValue.size(); ++i)
	{
		result_file << m_ResultValue[i] << " ";
		if (i % 4 == 3)
		{
			result_file << std::endl;
		}
	}
}

void Optimizer::OutputPAndS(std::string result_path)
{
	std::ofstream result_file(result_path);
	int bbox_num = m_ResultValue.size() / 4;
	for (int i = 0; i < bbox_num; ++i)
	{
		result_file << m_ResultValue[i * 4] << " 0.0 " << m_ResultValue[i * 4 + 1] << "                 "
			<< m_ResultValue[i * 4 + 2] / m_OriginalScale[i][0] << " 1.0 "
			<< m_ResultValue[i * 4 + 3] / m_OriginalScale[i][1] << std::endl;
	}
}

void Optimizer::testScaleThreshold(std::string test_path)
{
	std::ofstream test_file(test_path);
	for (int i = 0; i < m_ScaleThreshold.size(); ++i)
	{
		double temp_scale_x = m_ResultValue[i * 4 + 2] / m_OriginalScale[i][0];
		double temp_scale_y = m_ResultValue[i * 4 + 3] / m_OriginalScale[i][1];
		test_file << m_ScaleThreshold[i] << "     " << temp_scale_x << " " << temp_scale_y << "    ";
		test_file << temp_scale_x - m_ScaleThreshold[i][0] << " " << m_ScaleThreshold[i][1] - temp_scale_x << "     ";
		test_file << temp_scale_y - m_ScaleThreshold[i][2] << " " << m_ScaleThreshold[i][3] - temp_scale_y << std::endl;
	}


	for (int i = 0; i < m_OriginalScale.size(); ++i)
	{
		test_file << m_OriginalScale[i][0] <<"  "<< m_OriginalScale[i][1] << std::endl;
	}
}

void Optimizer::testPathWidth(std::string test_path)
{
	std::ofstream test_file(test_path);
	int bbox_id1, bbox_id2;
	for (int i = 0; i < m_ApartBBoxPair.size(); ++i)
	{
		test_file << "apart class " << i << " distance is " << m_ApartDistance[i] << std::endl;
		for (int j = 0; j < m_ApartBBoxPair[i].size(); ++j)
		{
			bbox_id1 = m_ApartBBoxPair[i][j][0];
			bbox_id2 = m_ApartBBoxPair[i][j][1];
			trimesh::vec4 bbox1(m_ResultValue[bbox_id1 * 4], m_ResultValue[bbox_id1 * 4 + 1], m_ResultValue[bbox_id1 * 4 + 2], m_ResultValue[bbox_id1 * 4 + 3]);
			trimesh::vec4 bbox2(m_ResultValue[bbox_id2 * 4], m_ResultValue[bbox_id2 * 4 + 1], m_ResultValue[bbox_id2 * 4 + 2], m_ResultValue[bbox_id2 * 4 + 3]);
			test_file << "bbox " << bbox_id1 << "  and bbox " << bbox_id2 << " distance is: " << BBoxDistance(bbox1, bbox2) << std::endl;
		}
	}
}

void Optimizer::testVisibleConstrain(std::string test_path)
{
	std::ofstream test_file(test_path);

	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);

	for (int i = 0; i < m_ObserveRelatedID.size(); ++i)
	{
		int observer_bbox_id = m_ObserveRelatedID[i][0];
		int target_bbox_id = m_ObserveRelatedID[i][1];
		trimesh::vec4 observe_bbox(m_ResultValue[observer_bbox_id * 4], m_ResultValue[observer_bbox_id * 4 + 1],
			m_ResultValue[observer_bbox_id * 4 + 2], m_ResultValue[observer_bbox_id * 4 + 3]);
		trimesh::vec4 target_bbox(m_ResultValue[target_bbox_id * 4], m_ResultValue[target_bbox_id * 4 + 1],
			m_ResultValue[target_bbox_id * 4 + 2], m_ResultValue[target_bbox_id * 4 + 3]);
		std::vector<trimesh::vec2> observe_triangle(3, trimesh::vec2(0, 0));
		trimesh::vec2 target_position(0, 0);
		for (int j = 0; j < 3; ++j)
		{
			observe_triangle[j][0] = observe_bbox[0] + observe_bbox[2] * m_ObserveField[i][j][0];
			observe_triangle[j][1] = observe_bbox[1] + observe_bbox[3] * m_ObserveField[i][j][1];
		}
		target_position[0] = target_bbox[0] + target_bbox[2] * m_TargetPoint[i][0];
		target_position[1] = target_bbox[1] + target_bbox[3] * m_TargetPoint[i][1];

		test_file << "observe bbox " << observer_bbox_id << "observe field  ";
		for (int j = 0; j < 3; ++j)
		{
			test_file << observe_triangle[j] << "  ";
		}
		test_file << std::endl;
		test_file << "target " << target_bbox_id << "  target position " << target_position;
		if (isPinTriangle(target_position, observe_triangle[0], observe_triangle[1], observe_triangle[2]))
		{
			test_file << "  in" << std::endl;
		}
		else
		{
			test_file << "  out" << std::endl;
		}

		for (int j = 0; j < m_OcclusionRelatedID[i].size(); ++j)
		{
			test_file << "occlusion box " << m_OcclusionRelatedID[i][j];
			int in_out_flag = 0;
			trimesh::vec4 occlusion_bbox(m_ResultValue[m_OcclusionRelatedID[i][j] * 4], m_ResultValue[m_OcclusionRelatedID[i][j] * 4 + 1],
				m_ResultValue[m_OcclusionRelatedID[i][j] * 4 + 2], m_ResultValue[m_OcclusionRelatedID[i][j] * 4 + 3]);

			for (int k = 0; k < 4; ++k)
			{
				trimesh::vec2 occlusion_position(0, 0);
				occlusion_position[0] = occlusion_bbox[0] + sign_a[k] * occlusion_bbox[2];
				occlusion_position[1] = occlusion_bbox[1] + sign_b[k] * occlusion_bbox[3];
				if (isPinTriangle(occlusion_position, observe_triangle[0], observe_triangle[1], observe_triangle[2]))
				{
					in_out_flag = 1;
					break;
				}
			}
			if (in_out_flag)
			{
				test_file << " in " << std::endl;
			}
			else
			{
				test_file << " out " << std::endl;
			}
		}
		test_file << std::endl;
	}
}

void Optimizer::testEnergy(std::string value_path)
{
	std::ifstream x_file(value_path);
	m_ResultValue = m_IniteValue;
	for (int i = 0; i < m_IniteValue.size(); ++i)
	{
		x_file >> m_ResultValue[i];
	}
	std::vector<double> result_energy(4, 0);
	result_energy[0] = FunctionPosition(m_ResultValue) * m_WeightPosition;
	result_energy[1] = FunctionScale(m_ResultValue) * m_WeightScale;
	result_energy[2] = FunctionPathWidth(m_ResultValue, m_ApartBBoxPair, m_ApartDistance) * m_WeightPathWidth;
	result_energy[3] = FunctionStructure(m_ResultValue, m_StructurePair, m_StructureDistanceSquare) * m_WeightStructure;
	double energy_sum = result_energy[0] + result_energy[1] + result_energy[2] + result_energy[3];

	std::cout << std::endl;
	std::cout << "position energy = " << result_energy[0] << std::endl;
	// m_DynamicScaleThreshold = m_ScaleThreshold;
	std::cout << "scale energy = " << result_energy[1] << std::endl;
	std::cout << "path width energy = " << result_energy[2] << std::endl;
	std::cout << "structure energy = " << result_energy[3] << std::endl;
	std::cout << "energy_sum = " << energy_sum << std::endl;
}

double Optimizer::FunctionPosition(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	double offset = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		offset = (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
		sum += std::exp(m_MoveExpS*(offset - m_MoveRangeR));
	}
	return sum;
}

double Optimizer::FunctionPosition(std::vector<double> &x, std::ofstream &energy_test)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	double offset = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		offset = (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
		sum += std::exp(m_MoveExpS*(offset - m_MoveRangeR));
	}
	
	int id = 27;
	double tempEnergy = (x[id * 4] - m_OriginalPosition[id][0]) * (x[id * 4] - m_OriginalPosition[id][0])
		+ (x[id * 4 + 1] - m_OriginalPosition[id][1])*(x[id * 4 + 1] - m_OriginalPosition[id][1]);
	tempEnergy = std::exp(m_MoveExpS*(tempEnergy - m_MoveRangeR));
	energy_test << "position energy: " << tempEnergy << std::endl;
	return sum;
}

void Optimizer::difFunctionPosition(std::vector<double> &x, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox_num = x.size() / 4;
	double offset = 0;
	double f = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		offset = (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
		f = std::exp(m_MoveExpS*(offset - m_MoveRangeR));

		dif[i * 4] = (x[i * 4] - m_OriginalPosition[i][0]) * 2.0 * f;
		dif[i * 4 + 1] = (x[i * 4 + 1] - m_OriginalPosition[i][1]) * 2.0 * f;
	}
}

void Optimizer::HessianFunctionPosition(std::vector<double> &x, std::vector<std::vector<double>> &hessian)
{
	int bbox_num = x.size() / 4;
	std::vector<double> zero_v(bbox_num * 4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(bbox_num * 4, zero_v);

	double offset = 0;
	double f = 0;

	for (int i = 0; i < bbox_num; ++i)
	{
		offset = (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
		f = std::exp(m_MoveExpS*(offset - m_MoveRangeR));

		hessian[i * 4][i * 4] = 2.0 * f + (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0]) * 4.0 * f;
		hessian[i * 4 + 1][i * 4 + 1] = 2.0 * f + (x[i * 4 + 1] - m_OriginalPosition[i][1]) * (x[i * 4 + 1] - m_OriginalPosition[i][1]) * 4.0 * f;
		hessian[i * 4 + 1][i * 4] = hessian[i * 4][i * 4 + 1] = (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4 + 1] - m_OriginalPosition[i][1]) * 4.0 * f;
	}
}

double Optimizer::FunctionPosition_Old(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		sum += (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
	}
	return sum;
}

double Optimizer::FunctionPosition_Old(std::vector<double> &x, std::ofstream &energy_test)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		sum += (x[i * 4] - m_OriginalPosition[i][0]) * (x[i * 4] - m_OriginalPosition[i][0])
			+ (x[i * 4 + 1] - m_OriginalPosition[i][1])*(x[i * 4 + 1] - m_OriginalPosition[i][1]);
	}

	int id = 27;
	double tempEnergy = (x[id * 4] - m_OriginalPosition[id][0]) * (x[id * 4] - m_OriginalPosition[id][0])
		+ (x[id * 4 + 1] - m_OriginalPosition[id][1])*(x[id * 4 + 1] - m_OriginalPosition[id][1]);
	energy_test << "position energy: " << tempEnergy << std::endl;
	return sum;
}

void Optimizer::difFunctionPosition_Old(std::vector<double> &x, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox_num = x.size() / 4;
	for (int i = 0; i < bbox_num; ++i)
	{
		dif[i * 4] = (x[i * 4] - m_OriginalPosition[i][0]) * 2.0;
		dif[i * 4 + 1] = (x[i * 4 + 1] - m_OriginalPosition[i][1]) * 2.0;
		dif[i * 4 + 2] = 0;
		dif[i * 4 + 3] = 0;
	}
}

void Optimizer::HessianFunctionPosition_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian)
{
	int bbox_num = x.size() / 4;
	std::vector<double> zero_v(bbox_num * 4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(bbox_num * 4, zero_v);
	for (int i = 0; i < bbox_num; ++i)
	{
		hessian[i * 4][i * 4] = 2.0;
		hessian[i * 4 + 1][i * 4 + 1] = 2.0;
	}
}

void Optimizer::updateDynamicScaleThreshold(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double scale_x, scale_y;
	trimesh::vec2 dynamic_scale_x, dynamic_scale_y;
	for (int i = 0; i < bbox_num; ++i)
	{
		scale_x = x[i * 4 + 2] / m_OriginalScale[i][0];
		scale_y = x[i * 4 + 3] / m_OriginalScale[i][1];
		dynamic_scale_x = adjustThreshold(scale_x, m_ScaleThreshold[i][0], m_ScaleThreshold[i][1]);
		dynamic_scale_y = adjustThreshold(scale_y, m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
		m_DynamicScaleThreshold[i] = trimesh::vec4(dynamic_scale_x[0], dynamic_scale_x[1], dynamic_scale_y[0], dynamic_scale_y[1]);
	}
}

trimesh::vec2 Optimizer::adjustThreshold(double s, double s_min, double s_max)
{
	trimesh::vec2 new_threshold(0, 0);

	if (s > s_min)
	{
		new_threshold[0] = s_min;
	}
	else
	{
		new_threshold[0] = s - 0.0001;
	}

	if (s < s_max)
	{
		new_threshold[1] = s_max;
	}
	else
	{
		new_threshold[1] = s + 0.0001;
	}

	return new_threshold;
}

double Optimizer::FunctionScale(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		sum += function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
			+ function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
	}
	return sum;
}

double Optimizer::FunctionScale(std::vector<double> &x, std::ofstream &energy_test)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		sum += function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
			+ function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
	}

	int id = 27;
	double tempEnergy = function_phi(x[id * 4 + 2] / m_OriginalScale[id][0], m_ScaleThreshold[id][0], m_ScaleThreshold[id][1])
		+ function_phi(x[id * 4 + 3] / m_OriginalScale[id][1], m_ScaleThreshold[id][2], m_ScaleThreshold[id][3]);
	energy_test << "scale energy: " << tempEnergy << std::endl;
	return sum;
}

void Optimizer::difFunctionScale(std::vector<double> &x, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox_num = x.size() / 4;
	for (int i = 0; i < bbox_num; ++i)
	{
		dif[i * 4] = 0;
		dif[i * 4 + 1] = 0;
		dif[i * 4 + 2] = dif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0],
			m_ScaleThreshold[i][0], m_ScaleThreshold[i][1]) / m_OriginalScale[i][0];
		dif[i * 4 + 3] = dif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1],
			m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]) / m_OriginalScale[i][1];
	}
}

void Optimizer::HessianFunctionScale(std::vector<double> &x, std::vector<std::vector<double>> &hessian)
{
	int bbox_num = x.size() / 4;
	std::vector<double> zero_v(bbox_num * 4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(bbox_num * 4, zero_v);
	for (int i = 0; i < bbox_num; ++i)
	{
		hessian[i * 4 + 2][i * 4 + 2] = ddif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0],
			m_ScaleThreshold[i][0], m_ScaleThreshold[i][1]) / (m_OriginalScale[i][0] * m_OriginalScale[i][0]);
		hessian[i * 4 + 3][i * 4 + 3] = ddif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1],
			m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]) / (m_OriginalScale[i][1] * m_OriginalScale[i][1]);
	}
}

void Optimizer::updateScaleTolerance(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double s_min = 1e10;
	double s_ratio_x, s_ratio_y;
	double ds_x_min, ds_x_max, ds_y_min, ds_y_max;
	for (int i = 0; i < bbox_num; ++i)
	{
		s_ratio_x = x[i * 4 + 2] / m_OriginalScale[i][0];
		s_ratio_y = x[i * 4 + 3] / m_OriginalScale[i][1];
		ds_x_min = s_ratio_x / m_ScaleThreshold[i][0] - 1.0;
		ds_x_max = 1.0 - s_ratio_x / m_ScaleThreshold[i][1];
		ds_y_min = s_ratio_y / m_ScaleThreshold[i][2] - 1.0;
		ds_y_max = 1.0 - s_ratio_y / m_ScaleThreshold[i][3];

		double new_min = std::min(std::min(ds_x_min, ds_x_max), std::min(ds_y_min, ds_y_max));
		if (s_min > new_min)
		{
			s_min = new_min;
		}
	}

	if (s_min < 0)
	{
		m_distanceEpsilon = s_min * s_min * 1e-10;
		if (m_distanceEpsilon < -2 * s_min * 1e-8)
		{
			m_distanceEpsilon = -2 * s_min * 1e-8;
		}
	}
	else
	{
		m_distanceEpsilon = 1e-10;
	}
	//std::cout << "m_distanceEpsilon = " << m_distanceEpsilon << std::endl;
}

double Optimizer::FunctionScale_Old(std::vector<double> &x)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		//sum += function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
		//	+ function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
		sum += function_phi_Old(x[i * 4 + 2] / m_OriginalScale[i][0], m_DynamicScaleThreshold[i][0],
			m_DynamicScaleThreshold[i][1])
			+ function_phi_Old(x[i * 4 + 3] / m_OriginalScale[i][1], m_DynamicScaleThreshold[i][2],
			m_DynamicScaleThreshold[i][3]);
	}
	return sum;
}

double Optimizer::FunctionScale_Old(std::vector<double> &x, std::ofstream &energy_test)
{
	int bbox_num = x.size() / 4;
	double sum = 0;
	for (int i = 0; i < bbox_num; ++i)
	{
		sum += function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
			+ function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
		//sum += function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_DynamicScaleThreshold[i][0], m_DynamicScaleThreshold[i][1])
		//	+ function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_DynamicScaleThreshold[i][2], m_DynamicScaleThreshold[i][3]);
	}

	int id = 27;
	double tempEnergy = function_phi(x[id * 4 + 2] / m_OriginalScale[id][0], m_ScaleThreshold[id][0],m_ScaleThreshold[id][1])
		+ function_phi(x[id * 4 + 3] / m_OriginalScale[id][1], m_ScaleThreshold[id][2], m_ScaleThreshold[id][3]);
	energy_test << "scale energy: " << tempEnergy << std::endl;
	return sum;
}

void Optimizer::difFunctionScale_Old(std::vector<double> &x, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox_num = x.size() / 4;
	for (int i = 0; i < bbox_num; ++i)
	{
		dif[i * 4] = 0;
		dif[i * 4 + 1] = 0;
		//dif[i * 4 + 2] = dif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1]) / m_OriginalScale[i][0];
		//dif[i * 4 + 3] = dif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]) / m_OriginalScale[i][1];
		dif[i * 4 + 2] = dif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_DynamicScaleThreshold[i][0], m_DynamicScaleThreshold[i][1]) / m_OriginalScale[i][0];
		dif[i * 4 + 3] = dif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_DynamicScaleThreshold[i][2], m_DynamicScaleThreshold[i][3]) / m_OriginalScale[i][1];
	}
}

void Optimizer::HessianFunctionScale_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian)
{
	int bbox_num = x.size() / 4;
	std::vector<double> zero_v(bbox_num * 4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(bbox_num * 4, zero_v);
	for (int i = 0; i < bbox_num; ++i)
	{
		//hessian[i * 4 + 2][i * 4 + 2] = ddif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1]) / (m_OriginalScale[i][0] * m_OriginalScale[i][0]);
		//hessian[i * 4 + 3][i * 4 + 3] = ddif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]) / (m_OriginalScale[i][1] * m_OriginalScale[i][1]);
		hessian[i * 4 + 2][i * 4 + 2] = ddif_function_phi(x[i * 4 + 2] / m_OriginalScale[i][0], m_DynamicScaleThreshold[i][0], m_DynamicScaleThreshold[i][1]) / (m_OriginalScale[i][0] * m_OriginalScale[i][0]);
		hessian[i * 4 + 3][i * 4 + 3] = ddif_function_phi(x[i * 4 + 3] / m_OriginalScale[i][1], m_DynamicScaleThreshold[i][2], m_DynamicScaleThreshold[i][3]) / (m_OriginalScale[i][1] * m_OriginalScale[i][1]);
	}
}

double Optimizer::FunctionPathWidth(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
	std::vector<double> &apart_distance)
{
	int apart_class_num = apart_pair.size();
	double sum = 0;

	double test_sum = 0;
	double test_temp = 0;

	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		int bbox1ID, bbox2ID;
		double apart_distance = m_ApartDistance[i];
		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			sum += function_barrier(bbox1, bbox2, apart_distance, m_barrierEpsilon);

 			if (bbox1ID == 14 && bbox2ID == 17)
 			{
 				//test_temp = function_barrier_test(bbox1, bbox2, apart_distance, m_barrierEpsilon);
 				//test_sum += test_temp;
 				//std::cout << bbox1ID << " and " << bbox2ID << " path energy is " << test_temp << std::endl;
				//std::cout << "bbox1 = " << bbox1 << "       bbox2 = " << bbox2 << "    apart distance = " << apart_distance << "   epsilon = " << m_barrierEpsilon << std::endl;
 			}
		}
	}

	//std::cout << "test_sum = " << test_sum << std::endl;

	return sum;
}

double Optimizer::FunctionPathWidth(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
	std::vector<double> &apart_distance, std::ofstream &energy_test)
{
	int apart_class_num = apart_pair.size();
	double sum = 0;
	double temp_energy = 0;
	energy_test << "barrier energy: ";
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		int bbox1ID, bbox2ID;
		double apart_distance = m_ApartDistance[i];
		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			temp_energy = function_barrier(bbox1, bbox2, apart_distance, m_barrierEpsilon);
			sum += temp_energy;
			//energy_test << bbox1ID << " " << bbox2ID << " " << temp_energy << "     ";
		}
		//energy_test << std::endl;
	}


	int bbox1ID_test, bbox2ID_test;
	bbox1ID_test = 28;
	bbox2ID_test = 27;
	double energy_temp;
	double apart_distance_test = 0.05;
	trimesh::vec4 bbox1_test(x[bbox1ID_test * 4], x[bbox1ID_test * 4 + 1], x[bbox1ID_test * 4 + 2], x[bbox1ID_test * 4 + 3]);
	trimesh::vec4 bbox2_test(x[bbox2ID_test * 4], x[bbox2ID_test * 4 + 1], x[bbox2ID_test * 4 + 2], x[bbox2ID_test * 4 + 3]);
	energy_temp = function_barrier(bbox1_test, bbox2_test, apart_distance_test, m_barrierEpsilon);
	energy_test << bbox1ID_test << " " << bbox2ID_test << " " << energy_temp << std::endl;

	return sum;
}

void Optimizer::difFunctionPathWidth(std::vector<double> &x, std::vector<double> &dif,
	std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox1ID, bbox2ID;
	std::vector<double> local_dif(8, 0);

	int apart_class_num = apart_pair.size();
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		double apart_distance = m_ApartDistance[i];

		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			dif_function_barrier(bbox1, bbox2, apart_distance, m_barrierEpsilon, local_dif);
			for (int k = 0; k < 4; ++k)
			{
				dif[bbox1ID * 4 + k] += local_dif[k];
				dif[bbox2ID * 4 + k] += local_dif[k + 4];
			}
		}
	}

}

void Optimizer::HessianFunctionPathWidth(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	int bbox1ID, bbox2ID;
	std::vector<std::vector<double>> local_hessian;

	int apart_class_num = apart_pair.size();
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		double apart_distance = m_ApartDistance[i];

		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			positive_hessian_function_barrier(bbox1, bbox2, apart_distance, m_barrierEpsilon, local_hessian);

			for (int k = 0; k < 4; ++k)
			{
				for (int w = 0; w < 4; ++w)
				{
					hessian[bbox1ID * 4 + k][bbox1ID * 4 + w] += local_hessian[k][w];
					hessian[bbox1ID * 4 + k][bbox2ID * 4 + w] += local_hessian[k][w + 4];
					hessian[bbox2ID * 4 + k][bbox1ID * 4 + w] += local_hessian[k + 4][w];
					hessian[bbox2ID * 4 + k][bbox2ID * 4 + w] += local_hessian[k + 4][w + 4];
				}
			}
		}
	}

}

double Optimizer::FunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
	std::vector<double> &apart_distance)
{
	//	int bbox_num = x.size() / 4;
	//	double sum = 0;
	// 	for (int i = 0; i < bbox_num; ++i)
	// 	{
	// 		trimesh::vec4 bbox1(x[i * 4], x[i * 4 + 1], x[i * 4 + 2], x[i * 4 + 3]);
	// 		for (int j = 0; j < bbox_num; ++j)
	// 		{
	// 			trimesh::vec4 bbox2(x[j * 4], x[j * 4 + 1], x[j * 4 + 2], x[j * 4 + 3]);
	// 			if (i == j)
	// 			{
	// 				continue;
	// 			}
	// 			sum += function_elliptic_barrier(bbox1, bbox2, m_PathWidth, m_barrierSigma);
	// 		}
	// 	}

	int apart_class_num = apart_pair.size();
	double sum = 0;
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		int bbox1ID, bbox2ID;
		double apart_distance = m_ApartDistance[i];
		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			sum += function_elliptic_barrier(bbox1, bbox2, apart_distance, m_barrierSigma);
		}
	}
	return sum;
}

double Optimizer::FunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<trimesh::ivec2>> &apart_pair,
	std::vector<double> &apart_distance, std::ofstream &energy_test)
{
//	int bbox_num = x.size() / 4;
//	double sum = 0;
// 	for (int i = 0; i < bbox_num; ++i)
// 	{
// 		trimesh::vec4 bbox1(x[i * 4], x[i * 4 + 1], x[i * 4 + 2], x[i * 4 + 3]);
// 		for (int j = 0; j < bbox_num; ++j)
// 		{
// 			trimesh::vec4 bbox2(x[j * 4], x[j * 4 + 1], x[j * 4 + 2], x[j * 4 + 3]);
// 			if (i == j)
// 			{
// 				continue;
// 			}
// 			sum += function_elliptic_barrier(bbox1, bbox2, m_PathWidth, m_barrierSigma);
// 		}
// 	}

	int apart_class_num = apart_pair.size();
	double sum = 0;
	double temp_energy = 0;
	energy_test << "barrier energy: ";
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		int bbox1ID, bbox2ID;
		double apart_distance = m_ApartDistance[i];
		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			temp_energy = function_elliptic_barrier(bbox1, bbox2, apart_distance, m_barrierSigma);
			sum += temp_energy;
			//energy_test << bbox1ID << " " << bbox2ID << " " << temp_energy << "     ";
		}
		//energy_test << std::endl;
	}


	int bbox1ID_test, bbox2ID_test;
	bbox1ID_test = 28;
	bbox2ID_test = 27;
	double energy_temp;
	double apart_distance_test = 0.05;
	trimesh::vec4 bbox1_test(x[bbox1ID_test * 4], x[bbox1ID_test * 4 + 1], x[bbox1ID_test * 4 + 2], x[bbox1ID_test * 4 + 3]);
	trimesh::vec4 bbox2_test(x[bbox2ID_test * 4], x[bbox2ID_test * 4 + 1], x[bbox2ID_test * 4 + 2], x[bbox2ID_test * 4 + 3]);
	energy_temp = function_elliptic_barrier(bbox1_test, bbox2_test, apart_distance_test, m_barrierSigma);
	energy_test << bbox1ID_test << " " << bbox2ID_test << " " <<  energy_temp << std::endl;

	return sum;
}

void Optimizer::difFunctionPathWidth_Old(std::vector<double> &x, std::vector<double> &dif,
	std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int bbox1ID, bbox2ID;
	trimesh::vec4 local_dif;

	int apart_class_num = apart_pair.size();
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		double apart_distance = m_ApartDistance[i];

		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];
			
			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);
			
			local_dif = dif_function_elliptic_barrier(bbox1, bbox2, apart_distance, m_barrierSigma);
			for (int k = 0; k < 4; ++k)
			{
				dif[bbox2ID * 4 + k] += local_dif[k];
			}
		}
	}
	
}

void Optimizer::HessianFunctionPathWidth_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	int bbox1ID, bbox2ID;
	std::vector<std::vector<double>> local_hessian;

	int apart_class_num = apart_pair.size();
	for (int i = 0; i < apart_class_num; ++i)
	{
		int pair_num = apart_pair[i].size();
		double apart_distance = m_ApartDistance[i];

		for (int j = 0; j < pair_num; ++j)
		{
			bbox1ID = apart_pair[i][j][0];
			bbox2ID = apart_pair[i][j][1];

			trimesh::vec4 bbox1(x[bbox1ID * 4], x[bbox1ID * 4 + 1], x[bbox1ID * 4 + 2], x[bbox1ID * 4 + 3]);
			trimesh::vec4 bbox2(x[bbox2ID * 4], x[bbox2ID * 4 + 1], x[bbox2ID * 4 + 2], x[bbox2ID * 4 + 3]);

			hessian_function_elliptic_barrier(bbox1, bbox2, apart_distance, m_barrierSigma, local_hessian);

			for (int k = 0; k < 4; ++k)
			{
				for (int w = 0; w < 4; ++w)
				{
					hessian[bbox2ID * 4 + k][bbox2ID * 4 + w] += local_hessian[k][w];
				}
			}
		}
	}
	
}

double Optimizer::FunctionAffiliatedPosition(std::vector<double> &x, std::vector<trimesh::ivec2> &affiliated_pair,
	std::vector<std::vector<int>> &affiliated_vertex)
{
	int pair_num = affiliated_pair.size();
	double sum = 0;
	int bbox1_id, bbox2_id;
	trimesh::vec4 bbox1, bbox2;

	std::ofstream test_signal_distance("data/distest.txt");

	for (int i = 0; i < pair_num; ++i)
	{
		bbox1_id = affiliated_pair[i][0];
		bbox2_id = affiliated_pair[i][1];
		test_signal_distance << "bbox1 is " << bbox1_id << "   bbox2 is " << bbox2_id << "  ";
		bbox1 = trimesh::vec4(x[bbox1_id * 4], x[bbox1_id * 4 + 1], x[bbox1_id * 4 + 2], x[bbox1_id * 4 + 3]);
		bbox2 = trimesh::vec4(x[bbox2_id * 4], x[bbox2_id * 4 + 1], x[bbox2_id * 4 + 2], x[bbox2_id * 4 + 3]);
		std::vector<int> vertex_id = affiliated_vertex[i];
		std::vector<std::vector<double>> vertex_distance;
		if (vertex_id[0] >= 0)
		{
			testDistance(bbox1, bbox2, vertex_id, vertex_distance);
			for (int j = 0; j < vertex_id.size(); ++j)
			{
				test_signal_distance << vertex_id[j] << " ";
				for (int k = 0; k < 4; ++k)
				{
					test_signal_distance << vertex_distance[j][k] << "    ";
				}
			}
			test_signal_distance << std::endl;
			sum += function_constrain_affiliated_position_overlapping(bbox1, bbox2, vertex_id);
		}
		else
		{
			sum += function_constrain_affiliated_position_apart(bbox1, bbox2);
		}
	}
	return sum;
}

void Optimizer::difFunctionAffiliatedPosition(std::vector<double> &x, std::vector<double> &dif,
	std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	int pair_num = affiliated_pair.size();
	int bbox1_id, bbox2_id;
	trimesh::vec4 bbox1, bbox2;
	for (int i = 0; i < pair_num; ++i)
	{
		bbox1_id = affiliated_pair[i][0];
		bbox2_id = affiliated_pair[i][1];
		bbox1 = trimesh::vec4(x[bbox1_id * 4], x[bbox1_id * 4 + 1], x[bbox1_id * 4 + 2], x[bbox1_id * 4 + 3]);
		bbox2 = trimesh::vec4(x[bbox2_id * 4], x[bbox2_id * 4 + 1], x[bbox2_id * 4 + 2], x[bbox2_id * 4 + 3]);
		std::vector<int> vertex_id = affiliated_vertex[i];
		std::vector<double> current_dif(8, 0);
		if (vertex_id[0] >= 0)
		{
			dif_function_constrain_affiliated_position_overlapping(bbox1, bbox2, vertex_id, current_dif);
			for (int j = 0; j < 4; ++j)
			{
				dif[bbox1_id * 4 + j] += current_dif[j];
				dif[bbox2_id * 4 + j] += current_dif[j + 4];
			}
		}
		else
		{
			dif_function_constrain_affiliated_position_apart(bbox1, bbox2, current_dif);
			for (int j = 0; j < 4; ++j)
			{
				dif[bbox1_id * 4 + j] += current_dif[j];
				dif[bbox2_id * 4 + j] += current_dif[j + 4];
			}
		}
	}
}

void Optimizer::HessianFunctionAffiliatedPosition(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	int pair_num = affiliated_pair.size();
	int bbox1_id, bbox2_id;
	trimesh::vec4 bbox1, bbox2;
	for (int i = 0; i < pair_num; ++i)
	{
		bbox1_id = affiliated_pair[i][0];
		bbox2_id = affiliated_pair[i][1];
		bbox1 = trimesh::vec4(x[bbox1_id * 4], x[bbox1_id * 4 + 1], x[bbox1_id * 4 + 2], x[bbox1_id * 4 + 3]);
		bbox2 = trimesh::vec4(x[bbox2_id * 4], x[bbox2_id * 4 + 1], x[bbox2_id * 4 + 2], x[bbox2_id * 4 + 3]);
		std::vector<int> vertex_id = affiliated_vertex[i];
		std::vector<std::vector<double>> current_hessian;
		if (vertex_id[0] >= 0)
		{
			ddif_function_constrain_affiliated_position_apart(bbox1, bbox2, current_hessian);
			for (int j = 0; j < 4; ++j)
			{
				for (int k = 0; k < 4; ++k)
				{
					hessian[bbox1_id * 4 + j][bbox1_id * 4 + k] += current_hessian[j][k];
					hessian[bbox1_id * 4 + j][bbox2_id * 4 + k] += current_hessian[j][k + 4];
					hessian[bbox2_id * 4 + j][bbox1_id * 4 + k] += current_hessian[j + 4][k];
					hessian[bbox2_id * 4 + j][bbox2_id * 4 + k] += current_hessian[j + 4][k + 4];
				}
			}
		}
		else
		{
			ddif_function_constrain_affiliated_position_overlapping(bbox1, bbox2, vertex_id, current_hessian);
			for (int j = 0; j < 4; ++j)
			{
				for (int k = 0; k < 4; ++k)
				{
					hessian[bbox1_id * 4 + j][bbox1_id * 4 + k] += current_hessian[j][k];
					hessian[bbox1_id * 4 + j][bbox2_id * 4 + k] += current_hessian[j][k + 4];
					hessian[bbox2_id * 4 + j][bbox1_id * 4 + k] += current_hessian[j + 4][k];
					hessian[bbox2_id * 4 + j][bbox2_id * 4 + k] += current_hessian[j + 4][k + 4];
				}
			}
		}
	}
}

double Optimizer::FunctionStructure(std::vector<double> &x, std::vector<trimesh::ivec2> &structure_pair,
	std::vector<double> &structure_distance_square)
{
	double sum = 0;
	double d_2 = 0;
	for (int i = 0; i < structure_pair.size(); ++i)
	{
		d_2 = (x[structure_pair[i][0] * 4] - x[structure_pair[i][1] * 4]) * (x[structure_pair[i][0] * 4] - x[structure_pair[i][1] * 4])
			+ (x[structure_pair[i][0] * 4 + 1] - x[structure_pair[i][1] * 4 + 1]) * (x[structure_pair[i][0] * 4 + 1] - x[structure_pair[i][1] * 4 + 1]);
		sum += d_2 / structure_distance_square[i] + structure_distance_square[i] / d_2;
	}
	return sum;
}

void Optimizer::difFunctionStructure(std::vector<double> &x, std::vector<double> &dif,
	std::vector<trimesh::ivec2> &structure_pair, std::vector<double> &structure_distance_square)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	double d_2 = 0;
	double df = 0;

	for (int i = 0; i < m_StructurePair.size(); ++i)
	{
		d_2 = (x[structure_pair[i][0] * 4] - x[structure_pair[i][1] * 4]) * (x[structure_pair[i][0] * 4] - x[structure_pair[i][1] * 4])
			+ (x[structure_pair[i][0] * 4 + 1] - x[structure_pair[i][1] * 4 + 1]) * (x[structure_pair[i][0] * 4 + 1] - x[structure_pair[i][1] * 4 + 1]);

		df = -structure_distance_square[i] / (d_2 * d_2) + 1.0 / structure_distance_square[i];
		dif[structure_pair[i][0] * 4] += 2.0 * (x[structure_pair[i][0] * 4] - x[structure_pair[i][1] * 4]) * df;
		dif[structure_pair[i][1] * 4] += 2.0 * (x[structure_pair[i][1] * 4] - x[structure_pair[i][0] * 4]) * df;
		dif[structure_pair[i][0] * 4 + 1] += 2.0 * (x[structure_pair[i][0] * 4 + 1] - x[structure_pair[i][1] * 4 + 1]) * df;
		dif[structure_pair[i][1] * 4 + 1] += 2.0 * (x[structure_pair[i][1] * 4 + 1] - x[structure_pair[i][0] * 4 + 1]) * df;
	}
}

void Optimizer::PositiveHessianFunctionStructure(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<trimesh::ivec2> &structure_pair, std::vector<double> &structure_distance_square)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	double df = 0;

	for (int i = 0; i < structure_pair.size(); ++i)
	{
		df = 2.0 / structure_distance_square[i];

		hessian[structure_pair[i][0] * 4][structure_pair[i][0] * 4] += df;
		hessian[structure_pair[i][1] * 4][structure_pair[i][1] * 4] += df;
		hessian[structure_pair[i][0] * 4][structure_pair[i][1] * 4] += -df;
		hessian[structure_pair[i][1] * 4][structure_pair[i][0] * 4] += -df;

		hessian[structure_pair[i][0] * 4 + 1][structure_pair[i][0] * 4 + 1] += df;
		hessian[structure_pair[i][1] * 4 + 1][structure_pair[i][1] * 4 + 1] += df;
		hessian[structure_pair[i][0] * 4 + 1][structure_pair[i][1] * 4 + 1] += -df;
		hessian[structure_pair[i][1] * 4 + 1][structure_pair[i][0] * 4 + 1] += -df;
	}
}

double Optimizer::FunctionStructure_Old(std::vector<double> &x, std::vector<trimesh::ivec3> &structure_triangle,
	std::vector<trimesh::vec3> &triangle_edge)
{
	double engergy = 0;
	std::vector<trimesh::vec2> triangle1(3, trimesh::vec2(0, 0));

	for (int i = 0; i < structure_triangle.size(); ++i)
	{
		for (int j = 0; j < 3; ++j)
		{
			triangle1[j][0] = x[structure_triangle[i][j] * 4];
			triangle1[j][1] = x[structure_triangle[i][j] * 4 + 1];
		}
		engergy += function_triangle_similarity(triangle1, triangle_edge[i]);
	}

	return engergy;
}

double Optimizer::FunctionStructure_Old(std::vector<double> &x, std::vector<trimesh::ivec3> &structure_triangle,
	std::vector<trimesh::vec3> &triangle_edge, std::ofstream &energy_test)
{
	double engergy = 0;
	std::vector<trimesh::vec2> triangle1(3, trimesh::vec2(0, 0));

	for (int i = 0; i < structure_triangle.size(); ++i)
	{
		for (int j = 0; j < 3; ++j)
		{
			triangle1[j][0] = x[structure_triangle[i][j] * 4];
			triangle1[j][1] = x[structure_triangle[i][j] * 4 + 1];
		}
		engergy += function_triangle_similarity(triangle1, triangle_edge[i]);
	}

	return engergy;
}

void Optimizer::difFunctionStructure_Old(std::vector<double> &x, std::vector<double> &dif,
	std::vector<trimesh::ivec3> &structure_triangle, std::vector<trimesh::vec3> &triangle_edge)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	std::vector<trimesh::vec2> triangle1(3, trimesh::vec2(0, 0));

	for (int i = 0; i < structure_triangle.size(); ++i)
	{
		for (int j = 0; j < 3; ++j)
		{
			triangle1[j][0] = x[structure_triangle[i][j] * 4];
			triangle1[j][1] = x[structure_triangle[i][j] * 4 + 1];
		}
		std::vector<double> temp_dif;
		dif_function_triangle_similarity(triangle1, triangle_edge[i], temp_dif);
		for (int j = 0; j < 3; ++j)
		{
			dif[structure_triangle[i][j] * 4] += temp_dif[2 * j];
			dif[structure_triangle[i][j] * 4 + 1] += temp_dif[2 * j + 1];
		}
	}
}

void Optimizer::HessianFunctionStructure_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<trimesh::ivec3> &structure_triangle, std::vector<trimesh::vec3> &triangle_edge)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	std::vector<trimesh::vec2> triangle1(3, trimesh::vec2(0, 0));

	for (int i = 0; i < structure_triangle.size(); ++i)
	{
		for (int j = 0; j < 3; ++j)
		{
			triangle1[j][0] = x[structure_triangle[i][j] * 4];
			triangle1[j][1] = x[structure_triangle[i][j] * 4 + 1];
		}
		std::vector<std::vector<double>> temp_hessian;
		ddif_function_triangle_similarity(triangle1, triangle_edge[i], temp_hessian);
		
		std::vector<int> hessian_id(6, 0);
		for (int j = 0; j < 3; ++j)
		{
			hessian_id[j * 2] = structure_triangle[i][j] * 4;
			hessian_id[j * 2 + 1] = structure_triangle[i][j] * 4 + 1;
		}
		for (int j = 0; j < 6; ++j)
		{
			for (int k = 0; k < 6; ++k)
			{
				hessian[hessian_id[j]][hessian_id[k]] += temp_hessian[j][k];
			}
		}
	}
}

double Optimizer::FunctionVisible(std::vector<double> &x, std::vector<trimesh::ivec2> &observe_related_id,
	std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point,
	std::vector<std::vector<int>> occlusion_related_id)
{
	double visible_energy = 0;

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		int observer_bbox_id = observe_related_id[i][0];
		int target_bbox_id = observe_related_id[i][1];
		trimesh::vec4 observe_bbox(x[observer_bbox_id * 4], x[observer_bbox_id * 4 + 1],
			x[observer_bbox_id * 4 + 2], x[observer_bbox_id * 4 + 3]);
		trimesh::vec4 target_bbox(x[target_bbox_id * 4], x[target_bbox_id * 4 + 1],
			x[target_bbox_id * 4 + 2], x[target_bbox_id * 4 + 3]);
		visible_energy += function_keep_visible(observe_bbox, observe_field[i], target_bbox, target_point[i]);

		for (int j = 0; j < occlusion_related_id[i].size(); ++j)
		{
			trimesh::vec4 occlusion_bbox(x[occlusion_related_id[i][j] * 4], x[occlusion_related_id[i][j] * 4 + 1],
				x[occlusion_related_id[i][j] * 4 + 2], x[occlusion_related_id[i][j] * 4 + 3]);
			visible_energy += function_prevent_occlusion(observe_bbox, observe_field[i], occlusion_bbox);
		}
	}

	return visible_energy;
}

void Optimizer::difFunctionVisible(std::vector<double> &x, std::vector<double> &dif, std::vector<trimesh::ivec2> &observe_related_id,
	std::vector<std::vector<trimesh::vec2>> observe_field, std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_id)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		int observer_bbox_id = observe_related_id[i][0];
		int target_bbox_id = observe_related_id[i][1];
		trimesh::vec4 observe_bbox(x[observer_bbox_id * 4], x[observer_bbox_id * 4 + 1],
			x[observer_bbox_id * 4 + 2], x[observer_bbox_id * 4 + 3]);
		trimesh::vec4 target_bbox(x[target_bbox_id * 4], x[target_bbox_id * 4 + 1],
			x[target_bbox_id * 4 + 2], x[target_bbox_id * 4 + 3]);

		std::vector<double> temp_dif(8, 0);
 		dif_function_keep_visible(observe_bbox, observe_field[i], target_bbox, target_point[i], temp_dif);
 
 		for (int j = 0; j < 4; ++j)
 		{
 			dif[observer_bbox_id * 4 + j] += temp_dif[j];
 			dif[target_bbox_id * 4 + j] += temp_dif[j + 4];
 		}

		for (int j = 0; j < occlusion_related_id[i].size(); ++j)
		{
			trimesh::vec4 occlusion_bbox(x[occlusion_related_id[i][j] * 4], x[occlusion_related_id[i][j] * 4 + 1],
				x[occlusion_related_id[i][j] * 4 + 2], x[occlusion_related_id[i][j] * 4 + 3]);
			dif_function_prevent_occlusion(observe_bbox, observe_field[i], occlusion_bbox, temp_dif);

			for (int k = 0; k < 4; ++k)
			{
				dif[observer_bbox_id * 4 + k] += temp_dif[k];
				dif[occlusion_related_id[i][j] * 4 + k] += temp_dif[k + 4];
			}
		}
	}
}

void Optimizer::HessianFunctionVisible(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<std::vector<trimesh::vec2>> observe_field,
	std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_id)
{
	hessian.clear();
	std::vector<double> zero_v(x.size(), 0);
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		int observer_bbox_id = observe_related_id[i][0];
		int target_bbox_id = observe_related_id[i][1];
		trimesh::vec4 observe_bbox(x[observer_bbox_id * 4], x[observer_bbox_id * 4 + 1],
			x[observer_bbox_id * 4 + 2], x[observer_bbox_id * 4 + 3]);
		trimesh::vec4 target_bbox(x[target_bbox_id * 4], x[target_bbox_id * 4 + 1],
			x[target_bbox_id * 4 + 2], x[target_bbox_id * 4 + 3]);

		std::vector<std::vector<double>> temp_hessian(8, std::vector<double>(8, 0));
		positive_hessian_function_keep_visible(observe_bbox, observe_field[i], target_bbox, target_point[i], temp_hessian);

		for (int j = 0; j < 4; ++j)
		{
			for (int k = 0; k < 4; ++k)
			{
				hessian[observer_bbox_id * 4 + j][observer_bbox_id * 4 + k] += temp_hessian[j][k];
				hessian[target_bbox_id * 4 + j][observer_bbox_id * 4 + k] += temp_hessian[j + 4][k];
				hessian[observer_bbox_id * 4 + j][target_bbox_id * 4 + k] += temp_hessian[j][k + 4];
				hessian[target_bbox_id * 4 + j][target_bbox_id * 4 + k] += temp_hessian[j + 4][k + 4];
			}
		}

		for (int j = 0; j < occlusion_related_id[i].size(); ++j)
		{
			trimesh::vec4 occlusion_bbox(x[occlusion_related_id[i][j] * 4], x[occlusion_related_id[i][j] * 4 + 1],
				x[occlusion_related_id[i][j] * 4 + 2], x[occlusion_related_id[i][j] * 4 + 3]);
			positive_hessian_function_prevent_occlusion(observe_bbox, observe_field[i], occlusion_bbox, temp_hessian);

			for (int k = 0; k < 4; ++k)
			{
				for (int w = 0; w < 4; ++w)
				{
					hessian[observer_bbox_id * 4 + k][observer_bbox_id * 4 + w] += temp_hessian[k][w];
					hessian[occlusion_related_id[i][j] * 4 + k][observer_bbox_id * 4 + w] += temp_hessian[k + 4][w];
					hessian[observer_bbox_id * 4 + k][occlusion_related_id[i][j] * 4 + w] += temp_hessian[k][w + 4];
					hessian[occlusion_related_id[i][j] * 4 + k][occlusion_related_id[i][j] * 4 + w] += temp_hessian[k + 4][w + 4];
				}
			}
		}
	}
}

double Optimizer::FunctionVisible_Old(std::vector<double> &x, std::vector<trimesh::ivec2> &observe_related_id,
	std::vector<trimesh::vec3> observe_field)
{
	double sum = 0;
	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		trimesh::vec2 observe_position(x[observe_related_id[i][0] * 4], x[observe_related_id[i][0] * 4 + 1]);
		trimesh::vec2 target_position(x[observe_related_id[i][1] * 4], x[observe_related_id[i][1] * 4 + 1]);
		sum += function_visible_target_in_field(observe_position, observe_field[i], target_position);
	}
	return sum;
}

void Optimizer::difFunctionVisible_Old(std::vector<double> &x, std::vector<double> &dif,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		trimesh::vec2 observe_position(x[observe_related_id[i][0] * 4], x[observe_related_id[i][0] * 4 + 1]);
		trimesh::vec2 target_position(x[observe_related_id[i][1] * 4], x[observe_related_id[i][1] * 4 + 1]);
		std::vector<double> current_dif(4, 0);
		dif_function_visible_target_in_field(observe_position, observe_field[i], target_position, current_dif);
		dif[observe_related_id[i][0] * 4] += current_dif[0];
		dif[observe_related_id[i][0] * 4 + 1] += current_dif[1];
		dif[observe_related_id[i][1] * 4] += current_dif[2];
		dif[observe_related_id[i][1] * 4 + 1] += current_dif[3];
	}
}

void Optimizer::HessianFunctionVisible_Old(std::vector<double> &x, std::vector<std::vector<double>> &hessian,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field)
{
	std::vector<double> zero_v(x.size(), 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(x.size(), zero_v);

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		trimesh::vec2 observe_position(x[observe_related_id[i][0] * 4], x[observe_related_id[i][0] * 4 + 1]);
		trimesh::vec2 target_position(x[observe_related_id[i][1] * 4], x[observe_related_id[i][1] * 4 + 1]);
		std::vector<std::vector<double>> current_hessian;
		ddif_function_visible_target_in_field(observe_position, observe_field[i], target_position, current_hessian);
		trimesh::ivec4 hessian_id(observe_related_id[i][0] * 4, observe_related_id[i][0] * 4 + 1,
			observe_related_id[i][1] * 4, observe_related_id[i][1] * 4 + 1);
		for (int j = 0; j < 4; ++j)
		{
			for (int k = 0; k < 4; ++k)
			{
				hessian[hessian_id[j]][hessian_id[k]] += current_hessian[j][k];
			}
		}
	}
}

double Optimizer::FunctionPenalty(std::vector<double> &x)
{
	double penalty = 0;
	for (int i = 0; i < m_ScaleThreshold.size(); ++i)
	{
		double add_penalty = ScalePenalty(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
			+ ScalePenalty(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
		penalty += add_penalty;

		if (add_penalty > 0)
		{
			std::cout << "scale " << i << " add penalty " << add_penalty << std::endl;
		}
// 		if (i == 30)
// 		{
// 			std::cout << x[i * 4 + 2] << " " << m_OriginalScale[i][0] << " " << m_OriginalScaleStore[i][0] 
// 				<< " " << x[i * 4 + 3] << " " << m_OriginalScale[i][1] << " " << m_OriginalScaleStore[i][1];
// 			std::cout << "scale " << 30 << " add penalty " << add_penalty << std::endl;
// 		}
	}
	return penalty;
}

double Optimizer::FunctionPenalty(std::vector<double> &x, std::ofstream &info_file)
{
	double penalty = 0;
	for (int i = 0; i < m_ScaleThreshold.size(); ++i)
	{
		double add_penalty = ScalePenalty(x[i * 4 + 2] / m_OriginalScale[i][0], m_ScaleThreshold[i][0], m_ScaleThreshold[i][1])
			+ ScalePenalty(x[i * 4 + 3] / m_OriginalScale[i][1], m_ScaleThreshold[i][2], m_ScaleThreshold[i][3]);
		penalty += add_penalty;

		if (add_penalty > 0)
		{
			info_file << "scale " << i << " add penalty " << add_penalty << std::endl;
		}
	}
	return penalty;
}

double Optimizer::ScalePenalty(double scale, double threshold_min, double threshold_max)
{
	double d_min = scale - threshold_min;
	double d_max = scale - threshold_max;
	if (d_min > -1e-4 && d_max < 1e-4)
	{
		return 0;
	}
	else if (d_min < 0)
	{
		return (((int)((-d_min) / 0.01) + 1) * m_ScalePenalty);
	}
	else
	{
		return (((int)(d_max / 0.01) + 1) * m_ScalePenalty);
	}

// 	else if ((d_min > -0.01 && d_min < -1e-4) || (d_max < 0.01 && d_max > 1e-4))
// 	{
// 		return m_ScalePenalty * 0.1;
// 	}
// 	else
// 	{
// 		return m_ScalePenalty;
// 	}
}

double Optimizer::PathWidthPenalty(trimesh::vec4 bbox1, trimesh::vec4 bbox2, double path_width)
{
	double distance = BBoxDistance(bbox1, bbox2);
	if (distance < path_width)
	{
		return 0;
	}
	else
	{
		return m_PathWidthPenalty;
	}
}

double Optimizer::function_phi(double x, double x_min, double x_max)
{
	double result;
	result = function_distance_energy(x / x_min - 1.0) + function_distance_energy(1.0 - x / x_max);
	return result; 
}

double Optimizer::dif_function_phi(double x, double x_min, double x_max)
{
	double dif = 0;
	dif = dif_function_distance_energy(x / x_min - 1.0) / x_min - dif_function_distance_energy(1.0 - x / x_max) / x_max;
	return dif;
}

double Optimizer::ddif_function_phi(double x, double x_min, double x_max)
{
	double ddif = 0;
	ddif = ddif_function_distance_energy(x / x_min - 1.0) / (x_min * x_min) + ddif_function_distance_energy(1.0 - x / x_max) / (x_max * x_max);
	return ddif;
}

double Optimizer::function_phi_Old(double x, double x_min, double x_max)
{
	double sum = 0;

	if (x > x_min)
	{
		sum -= std::log(x / x_min - 1.0 + m_phiDelta);
	}
	else
	{
		sum += 1000000.0;
	}

	if (x < x_max)
	{
		sum -= std::log(-x / x_max + 1.0 + m_phiDelta);
	}
	else
	{
		sum += 1000000.0;
	}
	return sum;
}

double Optimizer::dif_function_phi_Old(double x, double x_min, double x_max)
{
	double dif = 0;

	if (x > x_min)
	{
		dif += 1.0 / (x_min - x - m_phiDelta);
	}
	else
	{
		dif -= 10000000.0;
	}

	if (x < x_max)
	{
		dif += 1.0 / (x_max - x + m_phiDelta);
	}
	else
	{
		dif += 10000000.0;
	}
	return dif;
}

double Optimizer::ddif_function_phi_Old(double x, double x_min, double x_max)
{
	double ddif = 0;

	if (x > x_min)
	{
		ddif += 1.0 / ((x_min - x) * (x_min - x) + m_phiDelta);
	}
	else
	{
		ddif += 10000000.0;
	}

	if (x < x_max)
	{
		ddif += 1.0 / ((x - x_max)*(x - x_max) + m_phiDelta);
	}
	else
	{
		ddif += 10000000.0;
	}
	return ddif;
}

double Optimizer::function_barrier_d(double d1, double d2, double l, double epsilon)
{
	double d = d1 + d2 - l;
	//std::cout << " d = " << d<<"  ";
	if (d > epsilon)
	{
		return 0;
	}
	else
	{
		return (epsilon / d - 1.0) * (epsilon / d - 1.0);
	}
}

double Optimizer::dif_function_barrier_d(double d1, double d2, double l, double epsilon)
{
	double d = d1 + d2 - l;
	if (d > epsilon)
	{
		return 0;
	}
	else
	{
		return (epsilon / d - 1.0) * (-2.0 * epsilon / (d * d));
		//return (epsilon / d - 1.0) * (-2.0 / (d * d));
	}
}

double Optimizer::ddif_function_barrier_d(double d1, double d2, double l, double epsilon)
{
	double d = d1 + d2 - l;
	if (d > epsilon)
	{
		return 0;
	}
	else
	{
		return (6.0 * epsilon / d - 4.0) * epsilon / (d * d * d);
		//return (6.0 * epsilon / d - 4.0) / (d * d * d);
	}
}

double Optimizer::function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon)
{
	double sum = 0;

	std::vector<std::vector<double>> vertex_dis(4, std::vector<double>(4, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);
	trimesh::vec4 l_length(bbox_barrier[2], bbox_barrier[3], bbox_barrier[2], bbox_barrier[3]);
	double dx, dy, v_d;
	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			dx = bbox[0] + sign_a[i] * bbox[2] - bbox_barrier[0] - sign_a[j] * bbox_barrier[2] - distance * sign_a[j];
			dy = bbox[1] + sign_b[i] * bbox[3] - bbox_barrier[1] - sign_b[j] * bbox_barrier[3] - distance * sign_b[j];
			v_d = std::sqrt(dx * dx + dy * dy);
			//std::cout << "dx = " << dx << "   dy = " << dy << "   v_d = " << v_d << std::endl;
			vertex_dis[j][i] = v_d;
		}
		l_length[i] = 2.0 * (l_length[i] + distance);
	}

	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			double temp_f = function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);
			sum += temp_f;
			//std::cout << vertex_dis[i][j] << "   " << vertex_dis[(i + 1) % 4][j] << "   " << l_length[i] <<"  "<< "temp_f = " << temp_f << std::endl;
			//sum += function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);
		}
	}

	return sum;
}

double Optimizer::function_barrier_test(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon)
{
	double sum = 0;

	std::vector<std::vector<double>> vertex_dis(4, std::vector<double>(4, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);
	trimesh::vec4 l_length(bbox_barrier[2], bbox_barrier[3], bbox_barrier[2], bbox_barrier[3]);
	double dx, dy, v_d;
	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			dx = bbox[0] + sign_a[i] * bbox[2] - bbox_barrier[0] - sign_a[j] * bbox_barrier[2] - distance * sign_a[j];
			dy = bbox[1] + sign_b[i] * bbox[3] - bbox_barrier[1] - sign_b[j] * bbox_barrier[3] - distance * sign_b[j];
			v_d = std::sqrt(dx * dx + dy * dy);
			//std::cout << "dx = " << dx << "   dy = " << dy << "   v_d = " << v_d << std::endl;
			vertex_dis[j][i] = v_d;
		}
		l_length[i] = 2.0 * (l_length[i] + distance);
	}

	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			double temp_f = function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);
			sum += temp_f;
			std::cout << vertex_dis[i][j] << "   " << vertex_dis[(i + 1) % 4][j] << "   " << l_length[i] <<"  "<< "temp_f = " << temp_f << std::endl;
			//sum += function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);
		}
	}

	return sum;
}

void Optimizer::dif_function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double epsilon,
	std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(8, 0);

	double df = 0;

	std::vector<std::vector<double>> vertex_dis(4, std::vector<double>(4, 0));
	std::vector<std::vector<double>> dx(4, std::vector<double>(4, 0));
	std::vector<std::vector<double>> dy(4, std::vector<double>(4, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);
	trimesh::vec4 l_length(bbox_barrier[2], bbox_barrier[3], bbox_barrier[2], bbox_barrier[3]);
	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			dx[j][i] = bbox[0] + sign_a[i] * bbox[2] - bbox_barrier[0] - sign_a[j] * bbox_barrier[2] - distance * sign_a[j];
			dy[j][i] = bbox[1] + sign_b[i] * bbox[3] - bbox_barrier[1] - sign_b[j] * bbox_barrier[3] - distance * sign_b[j];
			vertex_dis[j][i] = std::sqrt(dx[j][i] * dx[j][i] + dy[j][i] * dy[j][i]);
		}
		l_length[i] = 2.0 * (l_length[i] + distance);
	}

	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			df = dif_function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);
			dif[0] += (-dx[i][j] / vertex_dis[i][j] - dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[1] += (-dy[i][j] / vertex_dis[i][j] - dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[2] += (-sign_a[i] * dx[i][j] / vertex_dis[i][j] - sign_a[(i + 1) % 4] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[3] += (-sign_b[i] * dy[i][j] / vertex_dis[i][j] - sign_b[(i + 1) % 4] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;

			dif[4] += (dx[i][j] / vertex_dis[i][j] + dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[5] += (dy[i][j] / vertex_dis[i][j] + dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			//dif[6] += (sign_a[i] * dx[i][j] / vertex_dis[i][j] + sign_a[(i + 1) % 4] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			//dif[7] += (sign_b[i] * dy[i][j] / vertex_dis[i][j] + sign_b[(i + 1) % 4] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[6] += (sign_a[j] * dx[i][j] / vertex_dis[i][j] + sign_a[j] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;
			dif[7] += (sign_b[j] * dy[i][j] / vertex_dis[i][j] + sign_b[j] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j]) * df;

			if (i % 2 == 0)
			{
				dif[2] -= 2.0 * df;
			}
			else
			{
				dif[3] -= 2.0 * df;
			}
		}

	}
}

void Optimizer::positive_hessian_function_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
	double epsilon, std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(8, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(8, zero_v);

	double ddf = 0;
	std::vector<double> dif_d(8, 0);

	std::vector<std::vector<double>> vertex_dis(4, std::vector<double>(4, 0));
	std::vector<std::vector<double>> dx(4, std::vector<double>(4, 0));
	std::vector<std::vector<double>> dy(4, std::vector<double>(4, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);
	trimesh::vec4 l_length(bbox_barrier[2], bbox_barrier[3], bbox_barrier[2], bbox_barrier[3]);
	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			dx[j][i] = bbox[0] + sign_a[i] * bbox[2] - bbox_barrier[0] - sign_a[j] * bbox_barrier[2] - distance * sign_a[j];
			dy[j][i] = bbox[1] + sign_b[i] * bbox[3] - bbox_barrier[1] - sign_b[j] * bbox_barrier[3] - distance * sign_b[j];
			vertex_dis[j][i] = std::sqrt(dx[j][i] * dx[j][i] + dy[j][i] * dy[j][i]);
		}
		l_length[i] = 2.0 * (l_length[i] + distance);
	}

	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			ddf = ddif_function_barrier_d(vertex_dis[i][j], vertex_dis[(i + 1) % 4][j], l_length[i], epsilon);

			dif_d[0] = -dx[i][j] / vertex_dis[i][j] - dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[1] = -dy[i][j] / vertex_dis[i][j] - dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[2] = -sign_a[i] * dx[i][j] / vertex_dis[i][j] - sign_a[(i + 1) % 4] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[3] = -sign_b[i] * dy[i][j] / vertex_dis[i][j] - sign_b[(i + 1) % 4] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];

			dif_d[4] = dx[i][j] / vertex_dis[i][j] + dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[5] = dy[i][j] / vertex_dis[i][j] + dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			//dif_d[6] = sign_a[i] * dx[i][j] / vertex_dis[i][j] + sign_a[(i + 1) % 4] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			//dif_d[7] = sign_b[i] * dy[i][j] / vertex_dis[i][j] + sign_b[(i + 1) % 4] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[6] = sign_a[j] * dx[i][j] / vertex_dis[i][j] + sign_a[j] * dx[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];
			dif_d[7] = sign_b[j] * dy[i][j] / vertex_dis[i][j] + sign_b[j] * dy[(i + 1) % 4][j] / vertex_dis[(i + 1) % 4][j];

			if (i % 2 == 0)
			{
				dif_d[2] -= 2.0;
			}
			else
			{
				dif_d[3] -= 2.0;
			}

			for (int k = 0; k < 8; ++k)
			{
				for (int w = 0; w < 8; ++w)
				{
					hessian[k][w] += dif_d[k] * dif_d[w] * ddf;
				}
			}
		}

	}
}

double Optimizer::function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance, double barrier_delta)
{
	double Ew_coe = (bbox_barrier[2] + distance) * 2.0;
	double Eh_coe = (bbox_barrier[3] + distance) * 2.0;
	Ew_coe = 1.0 / (Ew_coe * Ew_coe);
	Eh_coe = 1.0 / (Eh_coe * Eh_coe);

	std::vector<trimesh::vec2> offset_p(4, trimesh::vec2(0, 0));
	offset_p[0][0] = offset_p[3][0] = bbox[0] - bbox[2] - bbox_barrier[0];
	offset_p[0][1] = offset_p[1][1] = bbox[1] - bbox[3] - bbox_barrier[1];
	offset_p[1][0] = offset_p[2][0] = bbox[0] + bbox[2] - bbox_barrier[0];
	offset_p[2][1] = offset_p[3][1] = bbox[1] + bbox[3] - bbox_barrier[1];
	
	double Eb = 0;	
	Eb += function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[0], barrier_delta);
	Eb += function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[1], barrier_delta);
	Eb += function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[2], barrier_delta);
	Eb += function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[3], barrier_delta);

	return Eb;
}

double Optimizer::function_elliptic_barrier(double Ew_coe, double Eh_coe, trimesh::vec2 p, double barrier_delta)
{
	double energy = 0;
	energy = std::exp(m_barrierSigma * (Ew_coe * p[0] * p[0] + Eh_coe * p[1] * p[1]));
	return energy;
}

trimesh::vec4 Optimizer::dif_function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox,
	double distance, double barrier_delta)
{
	trimesh::vec4 dif_Eb(0, 0, 0, 0);

	double Ew_coe = (bbox_barrier[2] + distance) * 2.0;
	double Eh_coe = (bbox_barrier[3] + distance) * 2.0;
	Ew_coe = 1.0 / (Ew_coe * Ew_coe);
	Eh_coe = 1.0 / (Eh_coe * Eh_coe);

	std::vector<trimesh::vec2> offset_p(4, trimesh::vec2(0, 0));
	offset_p[0][0] = offset_p[3][0] = bbox[0] - bbox[2] - bbox_barrier[0];
	offset_p[0][1] = offset_p[1][1] = bbox[1] - bbox[3] - bbox_barrier[1];
	offset_p[1][0] = offset_p[2][0] = bbox[0] + bbox[2] - bbox_barrier[0];
	offset_p[2][1] = offset_p[3][1] = bbox[1] + bbox[3] - bbox_barrier[1];

	trimesh::vec4 Eb(0, 0, 0, 0);
	Eb[0] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[0], barrier_delta);
	Eb[1] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[1], barrier_delta);
	Eb[2] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[2], barrier_delta);
	Eb[3] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[3], barrier_delta);

	dif_Eb[0] = 2 * m_barrierSigma * Ew_coe * (offset_p[0][0] * Eb[0] + offset_p[1][0] * Eb[1]
		+ offset_p[2][0] * Eb[2] + offset_p[3][0] * Eb[3]);
	dif_Eb[1] = 2 * m_barrierSigma * Eh_coe * (offset_p[0][1] * Eb[0] + offset_p[1][1] * Eb[1]
		+ offset_p[2][1] * Eb[2] + offset_p[3][1] * Eb[3]);
	dif_Eb[2] = 2 * m_barrierSigma * Ew_coe * (-offset_p[0][0] * Eb[0] + offset_p[1][0] * Eb[1]
		+ offset_p[2][0] * Eb[2] - offset_p[3][0] * Eb[3]);
	dif_Eb[3] = 2 * m_barrierSigma * Eh_coe * (-offset_p[0][1] * Eb[0] - offset_p[1][1] * Eb[1]
		+ offset_p[2][1] * Eb[2] + offset_p[3][1] * Eb[3]);

	return dif_Eb;
}

void Optimizer::hessian_function_elliptic_barrier(trimesh::vec4 bbox_barrier, trimesh::vec4 bbox, double distance,
	double barrier_delta, std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(4, zero_v);

	double Ew_coe = (bbox_barrier[2] + distance) * 2.0;
	double Eh_coe = (bbox_barrier[3] + distance) * 2.0;
	Ew_coe = 1.0 / (Ew_coe * Ew_coe);
	Eh_coe = 1.0 / (Eh_coe * Eh_coe);

	std::vector<trimesh::vec2> offset_p(4, trimesh::vec2(0, 0));
	offset_p[0][0] = offset_p[3][0] = bbox[0] - bbox[2] - bbox_barrier[0];
	offset_p[0][1] = offset_p[1][1] = bbox[1] - bbox[3] - bbox_barrier[1];
	offset_p[1][0] = offset_p[2][0] = bbox[0] + bbox[2] - bbox_barrier[0];
	offset_p[2][1] = offset_p[3][1] = bbox[1] + bbox[3] - bbox_barrier[1];

	trimesh::vec4 Eb(0, 0, 0, 0);
	Eb[0] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[0], barrier_delta);
	Eb[1] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[1], barrier_delta);
	Eb[2] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[2], barrier_delta);
	Eb[3] = function_elliptic_barrier(Ew_coe, Eh_coe, offset_p[3], barrier_delta);

	double dif_Ew_coe = 2 * m_barrierSigma * Ew_coe;
	double dif_Eh_coe = 2 * m_barrierSigma * Eh_coe;
	std::vector<trimesh::vec4> dif_Eb_coe(4, trimesh::vec4(0,0,0,0));

	dif_Eb_coe[0][0] = dif_Ew_coe * offset_p[0][0];
	dif_Eb_coe[0][1] = dif_Ew_coe * offset_p[1][0];
	dif_Eb_coe[0][2] = dif_Ew_coe * offset_p[2][0];
	dif_Eb_coe[0][3] = dif_Ew_coe * offset_p[3][0];

	dif_Eb_coe[1][0] = dif_Eh_coe * offset_p[0][1];
	dif_Eb_coe[1][1] = dif_Eh_coe * offset_p[1][1];
	dif_Eb_coe[1][2] = dif_Eh_coe * offset_p[2][1];
	dif_Eb_coe[1][3] = dif_Eh_coe * offset_p[3][1];

	dif_Eb_coe[2][0] = -dif_Ew_coe * offset_p[0][0];
	dif_Eb_coe[2][1] = dif_Ew_coe * offset_p[1][0];
	dif_Eb_coe[2][2] = dif_Ew_coe * offset_p[2][0];
	dif_Eb_coe[2][3] = -dif_Ew_coe * offset_p[3][0];

	dif_Eb_coe[3][0] = -dif_Eh_coe * offset_p[0][1];
	dif_Eb_coe[3][1] = -dif_Eh_coe * offset_p[1][1];
	dif_Eb_coe[3][2] = dif_Eh_coe * offset_p[2][1];
	dif_Eb_coe[3][3] = dif_Eh_coe * offset_p[3][1];

	for (int i = 0; i < 4; ++i)
	{
		for (int j = i; j < 4; ++j)
		{
			for (int k = 0; k < 4; ++k)
			{
				hessian[i][j] += dif_Eb_coe[i][k] * dif_Eb_coe[j][k] * Eb[k];
			}
			hessian[j][i] = hessian[i][j];
		}
	}

	hessian[0][0] += dif_Ew_coe * (Eb[0] + Eb[1] + Eb[2] + Eb[3]);
	hessian[1][1] += dif_Eh_coe * (Eb[0] + Eb[1] + Eb[2] + Eb[3]);
	hessian[2][2] += dif_Ew_coe * (Eb[0] + Eb[1] + Eb[2] + Eb[3]);
	hessian[3][3] += dif_Eh_coe * (Eb[0] + Eb[1] + Eb[2] + Eb[3]);

	hessian[0][2] += dif_Ew_coe * (-Eb[0] + Eb[1] + Eb[2] - Eb[3]);
	hessian[1][3] += dif_Ew_coe * (-Eb[0] - Eb[1] + Eb[2] + Eb[3]);
	hessian[2][0] = hessian[0][2];
	hessian[3][1] = hessian[1][3];
}

double Optimizer::function_distance_energy(double distance)
{
	return 1.0 / (distance + std::sqrt(distance * distance + m_distanceEpsilon));
}

double Optimizer::dif_function_distance_energy(double distance)
{
	double temp = std::sqrt(distance * distance + m_distanceEpsilon);
	return -1.0 / ((distance + temp) * temp);
}

double Optimizer::ddif_function_distance_energy(double distance)
{
	double temp = distance * distance + m_distanceEpsilon;
	return 1.0 / (temp * std::sqrt(temp));
}

inline double Optimizer::sign_distance(trimesh::vec3 boundary_coe, trimesh::vec2 p)
{
	return boundary_coe[0] * p[0] + boundary_coe[1] * p[1] + boundary_coe[2];
}

double Optimizer::function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
	std::vector<int> vertex_id)
{
	double sum = 0;
	std::vector<trimesh::vec3> boundary_coe(4, trimesh::vec3(0, 0, 0));
	boundary_coe[0] = trimesh::vec3(1, 0, -bbox1[0] + bbox1[2]);
	boundary_coe[1] = trimesh::vec3(0, 1, -bbox1[1] + bbox1[3]);
	boundary_coe[2] = trimesh::vec3(-1, 0, bbox1[0] + bbox1[2]);
	boundary_coe[3] = trimesh::vec3(0, -1, bbox1[1] + bbox1[3]);

	for (int i = 0; i < vertex_id.size(); ++i)
	{
		int current_id = vertex_id[i];
		trimesh::vec2 current_vertex;
		switch (current_id)
		{
		case 0:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 1:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 2:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		default:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		}
		for (int j = 0; j < 4; ++j)
		{
			sum += function_distance_energy(sign_distance(boundary_coe[j], current_vertex));
		}
	}

	return sum;
}

void Optimizer::testDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<int> vertex_id,
	std::vector<std::vector<double>> &vertex_distance)
{
	vertex_distance.clear();
	vertex_distance = std::vector<std::vector<double>>(vertex_id.size(), std::vector<double>(4, 0));
	std::vector<trimesh::vec3> boundary_coe(4, trimesh::vec3(0, 0, 0));
	boundary_coe[0] = trimesh::vec3(1, 0, -bbox1[0] + bbox1[2]);
	boundary_coe[1] = trimesh::vec3(0, 1, -bbox1[1] + bbox1[3]);
	boundary_coe[2] = trimesh::vec3(-1, 0, bbox1[0] + bbox1[2]);
	boundary_coe[3] = trimesh::vec3(0, -1, bbox1[1] + bbox1[3]);

	for (int i = 0; i < vertex_id.size(); ++i)
	{
		int current_id = vertex_id[i];
		trimesh::vec2 current_vertex;
		switch (current_id)
		{
		case 0:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 1:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 2:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		default:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		}
		for (int j = 0; j < 4; ++j)
		{
			vertex_distance[i][j] = sign_distance(boundary_coe[j], current_vertex);
		}
	}
}

void Optimizer::dif_function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
	std::vector<int> vertex_id, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(8, 0);
	std::vector<trimesh::vec4> boundary_dif_coe(4, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec3> boundary_coe(4, trimesh::vec3(0, 0, 0));
	std::vector<trimesh::vec4> vertex_dif_coe(4, trimesh::vec4(1, 1, 0, 0));

	boundary_coe[0] = trimesh::vec3(1, 0, -bbox1[0] + bbox1[2]);
	boundary_coe[1] = trimesh::vec3(0, 1, -bbox1[1] + bbox1[3]);
	boundary_coe[2] = trimesh::vec3(-1, 0, bbox1[0] + bbox1[2]);
	boundary_coe[3] = trimesh::vec3(0, -1, bbox1[1] + bbox1[3]);

	boundary_dif_coe[0][0] = -1;
	boundary_dif_coe[0][2] = 1;
	boundary_dif_coe[1][1] = -1;
	boundary_dif_coe[1][3] = 1;
	boundary_dif_coe[2][0] = 1;
	boundary_dif_coe[2][2] = 1;
	boundary_dif_coe[3][1] = 1;
	boundary_dif_coe[3][3] = 1;

	vertex_dif_coe[0][2] = -1;
	vertex_dif_coe[0][3] = -1;
	vertex_dif_coe[1][2] = 1;
	vertex_dif_coe[1][3] = -1;
	vertex_dif_coe[2][2] = 1;
	vertex_dif_coe[2][3] = 1;
	vertex_dif_coe[3][2] = -1;
	vertex_dif_coe[3][3] = 1;

	for (int i = 0; i < vertex_id.size(); ++i)
	{
		int current_id = vertex_id[i];
		trimesh::vec2 current_vertex;
		switch (current_id)
		{
		case 0:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 1:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 2:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		default:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		}
		for (int j = 0; j < 4; ++j)
		{
			double current_distance_energy_dif = dif_function_distance_energy(sign_distance(boundary_coe[j], current_vertex));

			for (int k = 0; k < 4; ++k)
			{
				dif[k] += boundary_dif_coe[j][k] * current_distance_energy_dif;
				dif[k + 4] += boundary_coe[j][k % 2] * vertex_dif_coe[current_id][k] * current_distance_energy_dif;
			}
		}
	}
}

void Optimizer::ddif_function_constrain_affiliated_position_overlapping(trimesh::vec4 bbox1, trimesh::vec4 bbox2,
	std::vector<int> vertex_id, std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(8, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(8, zero_v);


	std::vector<trimesh::vec4> boundary_dif_coe(4, trimesh::vec4(0, 0, 0, 0));
	std::vector<trimesh::vec3> boundary_coe(4, trimesh::vec3(0, 0, 0));
	std::vector<trimesh::vec4> vertex_dif_coe(4, trimesh::vec4(1, 1, 0, 0));

	boundary_coe[0] = trimesh::vec3(1, 0, -bbox1[0] + bbox1[2]);
	boundary_coe[1] = trimesh::vec3(0, 1, -bbox1[1] + bbox1[3]);
	boundary_coe[2] = trimesh::vec3(-1, 0, bbox1[0] + bbox1[2]);
	boundary_coe[3] = trimesh::vec3(0, -1, bbox1[1] + bbox1[3]);

	boundary_dif_coe[0][0] = -1;
	boundary_dif_coe[0][2] = 1;
	boundary_dif_coe[1][1] = -1;
	boundary_dif_coe[1][3] = 1;
	boundary_dif_coe[2][0] = 1;
	boundary_dif_coe[2][2] = 1;
	boundary_dif_coe[3][1] = 1;
	boundary_dif_coe[3][3] = 1;

	vertex_dif_coe[0][2] = -1;
	vertex_dif_coe[0][3] = -1;
	vertex_dif_coe[1][2] = 1;
	vertex_dif_coe[1][3] = -1;
	vertex_dif_coe[2][2] = 1;
	vertex_dif_coe[2][3] = 1;
	vertex_dif_coe[3][2] = -1;
	vertex_dif_coe[3][3] = 1;

	for (int i = 0; i < vertex_id.size(); ++i)
	{
		int current_id = vertex_id[i];
		trimesh::vec2 current_vertex;
		switch (current_id)
		{
		case 0:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 1:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] - bbox2[3];
			break;
		case 2:
			current_vertex[0] = bbox2[0] + bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		default:
			current_vertex[0] = bbox2[0] - bbox2[2];
			current_vertex[1] = bbox2[1] + bbox2[3];
			break;
		}
		for (int j = 0; j < 4; ++j)
		{
			double current_distance_energy_ddif = ddif_function_distance_energy(sign_distance(boundary_coe[j], current_vertex));
			std::vector<double> current_dif(8, 0);
			for (int k = 0; k < 4; ++k)
			{
				current_dif[k] = boundary_dif_coe[j][k];
				current_dif[k + 4] = boundary_coe[j][k % 2] * vertex_dif_coe[current_id][k];
			}
			for (int k1 = 0; k1 < 8; ++k1)
			{
				for (int k2 = 0; k2 < 8; ++k2)
				{
					hessian[k1][k2] += current_distance_energy_ddif * current_dif[k1] * current_dif[k2];
				}
			}
		}
	}
}

double Optimizer::function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2)
{
	return 0;
}

void Optimizer::dif_function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(8, 0);
}

void Optimizer::ddif_function_constrain_affiliated_position_apart(trimesh::vec4 bbox1, trimesh::vec4 bbox2, std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(8, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(8, zero_v);
}

double Optimizer::function_triangle_similarity(std::vector<trimesh::vec2> triangle1, trimesh::vec3 triangle2)
{
	double energy = 0;
	double edge_length2_difference = 0;
	double edge_length2 = 0;
	for (int i = 0; i < 3; ++i)
	{
		edge_length2 = (triangle1[(i + 1) % 3][0] - triangle1[i][0])*(triangle1[(i + 1) % 3][0] - triangle1[i][0])
			+ (triangle1[(i + 1) % 3][1] - triangle1[i][1])*(triangle1[(i + 1) % 3][1] - triangle1[i][1]);
		edge_length2_difference = edge_length2 - triangle2[i] * triangle2[i];
		energy += edge_length2_difference * edge_length2_difference;
	}

	return energy;
}

void Optimizer::dif_function_triangle_similarity(std::vector<trimesh::vec2>triangle1, trimesh::vec3 triangle2, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(6, 0);
	double edge_length2_difference = 0;
	double edge_length2 = 0;
	for (int i = 0; i < 3; ++i)
	{
		edge_length2 = (triangle1[(i + 1) % 3][0] - triangle1[i][0]) * (triangle1[(i + 1) % 3][0] - triangle1[i][0])
			+ (triangle1[(i + 1) % 3][1] - triangle1[i][1]) * (triangle1[(i + 1) % 3][1] - triangle1[i][1]);
		edge_length2_difference = edge_length2 - triangle2[i] * triangle2[i];
		dif[i * 2] += -4.0 * edge_length2_difference * (triangle1[(i + 1) % 3][0] - triangle1[i][0]);
		dif[i * 2 + 1] += -4.0 * edge_length2_difference * (triangle1[(i + 1) % 3][1] - triangle1[i][1]);
		dif[((i + 1) % 3) * 2] += 4.0 * edge_length2_difference * (triangle1[(i + 1) % 3][0] - triangle1[i][0]);
		dif[((i + 1) % 3) * 2 + 1] += 4.0 * edge_length2_difference * (triangle1[(i + 1) % 3][1] - triangle1[i][1]);
	}
}

void Optimizer::ddif_function_triangle_similarity(std::vector<trimesh::vec2> triangle1, trimesh::vec3 triangle2,
	std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(6, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(6, zero_v);

	double edge_length2_difference = 0;
	double edge_length2 = 0;
	double dxdx, dxdy, dydy;
	for (int i = 0; i < 3; ++i)
	{
		edge_length2 = (triangle1[(i + 1) % 3][0] - triangle1[i][0])*(triangle1[(i + 1) % 3][0] - triangle1[i][0])
			+ (triangle1[(i + 1) % 3][1] - triangle1[i][1])*(triangle1[(i + 1) % 3][1] - triangle1[i][1]);
		edge_length2_difference = edge_length2 - triangle2[i] * triangle2[i];

		dxdx = 4.0 * edge_length2_difference + 8.0 * (triangle1[(i + 1) % 3][0] - triangle1[i][0])*(triangle1[(i + 1) % 3][0] - triangle1[i][0]);
		dxdy = 8.0 * (triangle1[(i + 1) % 3][0] - triangle1[i][0]) * (triangle1[(i + 1) % 3][1] - triangle1[i][1]);
		dydy = 4.0 * edge_length2_difference + 8.0 * (triangle1[(i + 1) % 3][1] - triangle1[i][1])*(triangle1[(i + 1) % 3][1] - triangle1[i][1]);

		hessian[i * 2][i * 2] += dxdx;
		hessian[i * 2][((i + 1) % 3) * 2] += -dxdx;
		hessian[((i + 1) % 3) * 2][i * 2] += -dxdx;
		hessian[((i + 1) % 3) * 2][((i + 1) % 3) * 2] += dxdx;
		hessian[i * 2 + 1][i * 2 + 1] += dydy;
		hessian[i * 2 + 1][((i + 1) % 3) * 2 + 1] += -dydy;
		hessian[((i + 1) % 3) * 2 + 1][i * 2 + 1] += -dydy;
		hessian[((i + 1) % 3) * 2 + 1][((i + 1) % 3) * 2 + 1] += dydy;
		hessian[i * 2][i * 2 + 1] += dxdy;
		hessian[i * 2 + 1][i * 2] += dxdy;
		hessian[((i + 1) % 3) * 2][i * 2 + 1] += -dxdy;
		hessian[i * 2 + 1][((i + 1) % 3) * 2] += -dxdy;
		hessian[i * 2][((i + 1) % 3) * 2 + 1] += -dxdy;
		hessian[((i + 1) % 3) * 2 + 1][i * 2] += -dxdy;
		hessian[((i + 1) % 3) * 2][((i + 1) % 3) * 2 + 1] += dxdy;
		hessian[((i + 1) % 3) * 2 + 1][((i + 1) % 3) * 2] += dxdy;
	}
}

double Optimizer::function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field, trimesh::vec2 target_position)
{
	double energy = 0;
	trimesh::vec3 boundary1_coe, boundary2_coe;
	visible_boundary(observe_position, observe_field, boundary1_coe, boundary2_coe);

	energy = function_distance_energy(sign_distance(boundary1_coe, target_position))
		+ function_distance_energy(sign_distance(boundary2_coe, target_position));
	return energy;
}

void Optimizer::visible_boundary(trimesh::vec2 p, trimesh::vec3 field, trimesh::vec3 &b1, trimesh::vec3 &b2)
{
	trimesh::vec2 view_direction(field[0], field[1]);
	view_direction.normalize();

	trimesh::vec4 rotate_M;
	rotate_M[0] = rotate_M[3] = std::cos(field[2]);
	rotate_M[1] = -std::sin(field[2]);
	rotate_M[2] = -rotate_M[1];

	trimesh::vec2 b1_v, b2_v;
	b1_v[0] = rotate_M[0] * view_direction[0] + rotate_M[1] * view_direction[1];
	b1_v[1] = rotate_M[2] * view_direction[0] + rotate_M[3] * view_direction[1];
	b2_v[0] = rotate_M[0] * view_direction[0] + rotate_M[1] * view_direction[1];
	b2_v[1] = rotate_M[2] * view_direction[0] + rotate_M[3] * view_direction[1];

	b1[0] = b1_v[1];
	b1[1] = -b1_v[0];
	b1[2] = -b1_v[1] * p[0] + b1_v[0] * p[1];

	b2[0] = -b2_v[1];
	b2[1] = b2_v[0];
	b2[2] = b2_v[1] * p[0] - b2_v[0] * p[1];
}

void Optimizer::dif_function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field,
	trimesh::vec2 target_position, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(4, 0);

	trimesh::vec3 boundary1_coe, boundary2_coe;
	visible_boundary(observe_position, observe_field, boundary1_coe, boundary2_coe);
	double boundary1_distance_energy_dif = dif_function_distance_energy(sign_distance(boundary1_coe, target_position));
	double boundary2_distance_energy_dif = dif_function_distance_energy(sign_distance(boundary2_coe, target_position));
	dif[0] = -boundary1_coe[0] * boundary1_distance_energy_dif - boundary2_coe[0] * boundary2_distance_energy_dif;
	dif[1] = -boundary1_coe[1] * boundary1_distance_energy_dif - boundary2_coe[1] * boundary2_distance_energy_dif;
	dif[2] = boundary1_coe[0] * boundary1_distance_energy_dif + boundary2_coe[0] * boundary2_distance_energy_dif;
	dif[3] = boundary1_coe[1] * boundary1_distance_energy_dif + boundary2_coe[1] * boundary2_distance_energy_dif;
}

void Optimizer::ddif_function_visible_target_in_field(trimesh::vec2 observe_position, trimesh::vec3 observe_field,
	trimesh::vec2 target_position, std::vector<std::vector<double>> &hessian)
{
	std::vector<double> zero_v(4, 0);
	hessian.clear();
	hessian = std::vector<std::vector<double>>(4, zero_v);

	trimesh::vec3 boundary1_coe, boundary2_coe;
	visible_boundary(observe_position, observe_field, boundary1_coe, boundary2_coe);
	double boundary1_distance_energy_ddif = ddif_function_distance_energy(sign_distance(boundary1_coe, target_position));
	double boundary2_distance_energy_ddif = ddif_function_distance_energy(sign_distance(boundary2_coe, target_position));
	trimesh::vec4 current_dif1(-boundary1_coe[0], -boundary1_coe[1], boundary1_coe[0], boundary1_coe[1]);
	trimesh::vec4 current_dif2(-boundary2_coe[0], -boundary2_coe[1], boundary2_coe[0], boundary2_coe[1]);
	for (int i = 0; i < 4; ++i)
	{
		for (int j = 0; j < 4; ++j)
		{
			hessian[i][j] = boundary1_distance_energy_ddif * current_dif1[i] * current_dif1[j]
				+ boundary2_distance_energy_ddif*current_dif2[i] * current_dif2[j];
		}
	}
}

double Optimizer::function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 target_bbox, trimesh::vec2 target_point)
{
	double visible_energy = 0;

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	trimesh::vec2 target_position(0, 0);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	target_position[0] = target_bbox[0] + target_point[0] * target_bbox[2];
	target_position[1] = target_bbox[1] + target_point[1] * target_bbox[3];

	trimesh::vec3 dis_t_to_v(0, 0, 0);
	trimesh::vec3 length_v(0, 0, 0);
	for (int i = 0; i < 3; ++i)
	{
		dis_t_to_v[i] = (target_position - region_v[i]).length();
		length_v[i] = (region_v[(i + 1) % 3] - region_v[i]).length();
	}
	for (int i = 0; i < 3; ++i)
	{
		visible_energy += function_barrier_d(dis_t_to_v[i], dis_t_to_v[(i + 1) % 3], length_v[i], m_barrierEpsilon2);
	}

	return visible_energy;
}

void Optimizer::dif_function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 target_bbox, trimesh::vec2 target_point, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(8, 0);

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	trimesh::vec2 target_position(0, 0);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	target_position[0] = target_bbox[0] + target_point[0] * target_bbox[2];
	target_position[1] = target_bbox[1] + target_point[1] * target_bbox[3];

	trimesh::vec3 dis_t_to_v(0, 0, 0);
	trimesh::vec3 length_v(0, 0, 0);
	std::vector<std::vector<double>> dis_t_to_v_dif(3, std::vector<double>(8, 0));
	std::vector<std::vector<double>> length_v_dif(3, std::vector<double>(8, 0));
	for (int i = 0; i < 3; ++i)
	{
		dis_t_to_v[i] = (target_position - region_v[i]).length();
		length_v[i] = (region_v[(i + 1) % 3] - region_v[i]).length();
	}
	for (int i = 0; i < 3; ++i)
	{
		dis_t_to_v_dif[i][0] = -(region_v[i][0] - target_position[0]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][1] = -(region_v[i][1] - target_position[1]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][2] = -(region_v[i][0] - target_position[0]) * visible_region[i][0] / dis_t_to_v[i];
		dis_t_to_v_dif[i][3] = -(region_v[i][1] - target_position[1]) * visible_region[i][1] / dis_t_to_v[i];

		dis_t_to_v_dif[i][4] = -(target_position[0] - region_v[i][0]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][5] = -(target_position[1] - region_v[i][1]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][6] = -(target_position[0] - region_v[i][0]) * target_point[0] / dis_t_to_v[i];
		dis_t_to_v_dif[i][7] = -(target_position[1] - region_v[i][1]) * target_point[1] / dis_t_to_v[i];

		length_v_dif[i][2] = (region_v[i][0] - region_v[(i + 1) % 3][0]) * (-visible_region[i][0] + visible_region[(i + 1) % 3][0]) / length_v[i];
		length_v_dif[i][3] = (region_v[i][1] - region_v[(i + 1) % 3][1]) * (-visible_region[i][1] + visible_region[(i + 1) % 3][1]) / length_v[i];
	}


	for (int i = 0; i < 3; ++i)
	{
		double df = dif_function_barrier_d(dis_t_to_v[i], dis_t_to_v[(i + 1) % 3], length_v[i], m_barrierEpsilon2);
		for (int j = 0; j < 8; ++j)
		{
			dif[j] += (dis_t_to_v_dif[i][j] + dis_t_to_v_dif[(i + 1) % 3][j] - length_v_dif[i][j]) * df;
		}
	}
}

void Optimizer::positive_hessian_function_keep_visible(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 target_bbox, trimesh::vec2 target_point, std::vector<std::vector<double>> &hessian)
{
	hessian.clear();
	hessian = std::vector<std::vector<double>>(8, std::vector<double>(8, 0));

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	trimesh::vec2 target_position(0, 0);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	target_position[0] = target_bbox[0] + target_point[0] * target_bbox[2];
	target_position[1] = target_bbox[1] + target_point[1] * target_bbox[3];

	trimesh::vec3 dis_t_to_v(0, 0, 0);
	trimesh::vec3 length_v(0, 0, 0);
	std::vector<std::vector<double>> dis_t_to_v_dif(3, std::vector<double>(8, 0));
	std::vector<std::vector<double>> length_v_dif(3, std::vector<double>(8, 0));
	std::vector<std::vector<double>> dif(3, std::vector<double>(8, 0));
	for (int i = 0; i < 3; ++i)
	{
		dis_t_to_v[i] = (target_position - region_v[i]).length();
		length_v[i] = (region_v[(i + 1) % 3] - region_v[i]).length();
	}
	for (int i = 0; i < 3; ++i)
	{
		dis_t_to_v_dif[i][0] = -(region_v[i][0] - target_position[0]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][1] = -(region_v[i][1] - target_position[1]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][2] = -(region_v[i][0] - target_position[0]) * visible_region[i][0] / dis_t_to_v[i];
		dis_t_to_v_dif[i][3] = -(region_v[i][1] - target_position[1]) * visible_region[i][1] / dis_t_to_v[i];

		dis_t_to_v_dif[i][4] = -(target_position[0] - region_v[i][0]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][5] = -(target_position[1] - region_v[i][1]) / dis_t_to_v[i];
		dis_t_to_v_dif[i][6] = -(target_position[0] - region_v[i][0]) * target_point[0] / dis_t_to_v[i];
		dis_t_to_v_dif[i][7] = -(target_position[1] - region_v[i][1]) * target_point[1] / dis_t_to_v[i];

		length_v_dif[i][2] = (region_v[i][0] - region_v[(i + 1) % 3][0]) * (-visible_region[i][0] + visible_region[(i + 1) % 3][0]) / length_v[i];
		length_v_dif[i][3] = (region_v[i][1] - region_v[(i + 1) % 3][1]) * (-visible_region[i][1] + visible_region[(i + 1) % 3][1]) / length_v[i];
		
	}

	for (int i = 0; i < 3; ++i)
	{
		for (int j = 0; j < 8; ++j)
		{
			dif[i][j] = (dis_t_to_v_dif[i][j] + dis_t_to_v_dif[(i + 1) % 3][j] - length_v_dif[i][j]);
		}
	}

	for (int i = 0; i < 3; ++i)
	{
		double ddf = ddif_function_barrier_d(dis_t_to_v[i], dis_t_to_v[(i + 1) % 3], length_v[i], m_barrierEpsilon2);
		for (int j = 0; j < 8; ++j)
		{
			for (int k = 0; k < 8; ++k)
			{
				hessian[j][k] = dif[i][j] * dif[i][k] * ddf;
			}
		}
	}
}

double Optimizer::function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 occlusion_bbox)
{
	double occlusion_energy = 0;

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> occlusion_position(4, trimesh::vec2(0, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	for (int i = 0; i < 4; ++i)
	{
		occlusion_position[i][0] = occlusion_bbox[0] + sign_a[i] * occlusion_bbox[2];
		occlusion_position[i][1] = occlusion_bbox[1] + sign_b[i] * occlusion_bbox[3];
	}

	for (int i = 0; i < 4; ++i)
	{
		trimesh::vec3 dis_t_to_v(0, 0, 0);
		trimesh::vec3 length_v(0, 0, 0);
		for (int j = 0; j < 3; ++j)
		{
			dis_t_to_v[j] = (occlusion_position[i] - region_v[j]).length();
			length_v[j] = (region_v[(j + 1) % 3] - region_v[j]).length();
		}
		for (int j = 0; j < 3; ++j)
		{
			occlusion_energy += function_barrier_d(dis_t_to_v[j], dis_t_to_v[(j + 1) % 3], length_v[j], m_barrierEpsilon);
		}
	}

	return occlusion_energy;
}

void Optimizer::dif_function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 occlusion_bbox, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(8, 0);

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> occlusion_position(4, trimesh::vec2(0, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	for (int i = 0; i < 4; ++i)
	{
		occlusion_position[i][0] = occlusion_bbox[0] + sign_a[i] * occlusion_bbox[2];
		occlusion_position[i][1] = occlusion_bbox[1] + sign_b[i] * occlusion_bbox[3];
	}

	for (int i = 0; i < 4; ++i)
	{
		trimesh::vec3 dis_t_to_v(0, 0, 0);
		trimesh::vec3 length_v(0, 0, 0);
		std::vector<std::vector<double>> dis_t_to_v_dif(3, std::vector<double>(8, 0));
		std::vector<std::vector<double>> length_v_dif(3, std::vector<double>(8, 0));
		for (int j = 0; j < 3; ++j)
		{
			dis_t_to_v[j] = (occlusion_position[i] - region_v[j]).length();
			length_v[j] = (region_v[(j + 1) % 3] - region_v[j]).length();
		}
		for (int j = 0; j < 3; ++j)
		{
			dis_t_to_v_dif[j][0] = -(region_v[j][0] - occlusion_position[i][0]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][1] = -(region_v[j][1] - occlusion_position[i][1]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][2] = -(region_v[j][0] - occlusion_position[i][0]) * visible_region[j][0] / dis_t_to_v[j];
			dis_t_to_v_dif[j][3] = -(region_v[j][1] - occlusion_position[i][1]) * visible_region[j][1] / dis_t_to_v[j];

			dis_t_to_v_dif[j][4] = -(occlusion_position[i][0] - region_v[j][0]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][5] = -(occlusion_position[i][1] - region_v[j][1]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][6] = -(occlusion_position[i][0] - region_v[j][0]) * sign_a[i] / dis_t_to_v[j];
			dis_t_to_v_dif[j][7] = -(occlusion_position[i][1] - region_v[j][1]) * sign_b[i] / dis_t_to_v[j];

			length_v_dif[j][2] = (region_v[j][0] - region_v[(j + 1) % 3][0]) * (-visible_region[j][0] + visible_region[(j + 1) % 3][0]) / length_v[j];
			length_v_dif[j][3] = (region_v[j][1] - region_v[(j + 1) % 3][1]) * (-visible_region[j][1] + visible_region[(j + 1) % 3][1]) / length_v[j];
		}
		for (int j = 0; j < 3; ++j)
		{
			double df = dif_function_barrier_d(dis_t_to_v[j], dis_t_to_v[(j + 1) % 3], length_v[j], m_barrierEpsilon);

			for (int k = 0; k < 8; ++k)
			{
				dif[k] += (dis_t_to_v_dif[j][k] + dis_t_to_v_dif[(j + 1) % 3][k] - length_v_dif[j][k]) * df;
			}
		}
	}
}

void Optimizer::positive_hessian_function_prevent_occlusion(trimesh::vec4 observe_bbox, std::vector<trimesh::vec2> visible_region,
	trimesh::vec4 occlusion_bbox, std::vector<std::vector<double>> &hessian)
{
	hessian.clear();
	hessian = std::vector<std::vector<double>>(8, std::vector<double>(8, 0));

	std::vector<trimesh::vec2> region_v(3, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> occlusion_position(4, trimesh::vec2(0, 0));
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);

	for (int i = 0; i < 3; ++i)
	{
		region_v[i][0] = observe_bbox[0] + observe_bbox[2] * visible_region[i][0];
		region_v[i][1] = observe_bbox[1] + observe_bbox[3] * visible_region[i][1];
	}
	for (int i = 0; i < 4; ++i)
	{
		occlusion_position[i][0] = occlusion_bbox[0] + sign_a[i] * occlusion_bbox[2];
		occlusion_position[i][1] = occlusion_bbox[1] + sign_b[i] * occlusion_bbox[3];
	}

	for (int i = 0; i < 4; ++i)
	{
		trimesh::vec3 dis_t_to_v(0, 0, 0);
		trimesh::vec3 length_v(0, 0, 0);
		std::vector<std::vector<double>> dis_t_to_v_dif(3, std::vector<double>(8, 0));
		std::vector<std::vector<double>> length_v_dif(3, std::vector<double>(8, 0));
		std::vector<std::vector<double>> dif(3, std::vector<double>(8, 0));
		for (int j = 0; j < 3; ++j)
		{
			dis_t_to_v[j] = (occlusion_position[i] - region_v[j]).length();
			length_v[j] = (region_v[(j + 1) % 3] - region_v[j]).length();
		}
		for (int j = 0; j < 3; ++j)
		{
			dis_t_to_v_dif[j][0] = -(region_v[j][0] - occlusion_position[i][0]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][1] = -(region_v[j][1] - occlusion_position[i][1]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][2] = -(region_v[j][0] - occlusion_position[i][0]) * visible_region[j][0] / dis_t_to_v[j];
			dis_t_to_v_dif[j][3] = -(region_v[j][1] - occlusion_position[i][1]) * visible_region[j][1] / dis_t_to_v[j];

			dis_t_to_v_dif[j][4] = -(occlusion_position[i][0] - region_v[j][0]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][5] = -(occlusion_position[i][1] - region_v[j][1]) / dis_t_to_v[j];
			dis_t_to_v_dif[j][6] = -(occlusion_position[i][0] - region_v[j][0]) * sign_a[i] / dis_t_to_v[j];
			dis_t_to_v_dif[j][7] = -(occlusion_position[i][1] - region_v[j][1]) * sign_b[i] / dis_t_to_v[j];

			length_v_dif[j][2] = (region_v[j][0] - region_v[(j + 1) % 3][0]) * (-visible_region[j][0] + visible_region[(j + 1) % 3][0]) / length_v[j];
			length_v_dif[j][3] = (region_v[j][1] - region_v[(j + 1) % 3][1]) * (-visible_region[j][1] + visible_region[(j + 1) % 3][1]) / length_v[j];
		}

		for (int j = 0; j < 3; ++j)
		{
			for (int k = 0; k < 8; ++k)
			{
				dif[j][k] = (dis_t_to_v_dif[j][k] + dis_t_to_v_dif[(j + 1) % 3][k] - length_v_dif[j][k]);
			}
		}

		for (int j = 0; j < 3; ++j)
		{
			double ddf = ddif_function_barrier_d(dis_t_to_v[j], dis_t_to_v[(j + 1) % 3], length_v[j], m_barrierEpsilon);

			for (int k = 0; k < 8; ++k)
			{
				for (int w = 0; w < 8; ++w)
				{
					hessian[k][w] += dif[j][k] * dif[j][w] * ddf;
				}
			}
		}
	}
}

void Optimizer::ObjectFunction(std::vector<double> &x, std::vector<double> &energy)
{
	energy[3] = 0;

	energy[0] = FunctionPosition(x) * m_WeightPosition;
	//energy[0] = 0;
	energy[1] = FunctionScale(x) * m_WeightScale;
	//energy[1] = 0;
	energy[2] = FunctionPathWidth(x, m_ApartBBoxPair, m_ApartDistance) * m_WeightPathWidth;
	//energy[2] = 0;
	energy[4] = FunctionStructure(x, m_StructurePair, m_StructureDistanceSquare) * m_WeightStructure;
	//energy[4] = 0;	
	energy[5] = FunctionVisible(x, m_ObserveRelatedID, m_ObserveField, m_TargetPoint, m_OcclusionRelatedID) * m_WeightVisible;
	//energy[5] = 0;
}

void Optimizer::ObjectFunction(std::vector<double> &x, std::vector<double> &energy, std::ofstream &energy_test)
{
	energy[0] = FunctionPosition(x, energy_test) * m_WeightPosition;
	energy[1] = FunctionScale(x, energy_test) * m_WeightScale;
	energy[2] = FunctionPathWidth(x, m_ApartBBoxPair, m_ApartDistance, energy_test) * m_WeightPathWidth;
	//energy[3] = FunctionAffiliatedPosition(x, m_AffiliatedPair, m_AffiliatedVertex) * m_WeightAffiliatePosition;
	energy[3] = 0;
	energy[4] = FunctionStructure(x, m_StructurePair, m_StructureDistanceSquare) * m_WeightStructure;
	//energy[4] = (FunctionStructure(x, m_BBoxPositionStructure, m_BBoxStructureDistance, energy_test) + FunctionStructure(x, m_GroupPositionStructure, m_GroupStructureDistance)) * m_WeightStructure;
	//energy[5] = FunctionVisible(x, m_ObserveRelatedID, m_ObserveField);
	energy[5] = 0;
}

void Optimizer::difObjectFunction(std::vector<double> &x, std::vector<double> &dif)
{
	dif.clear();
	dif = std::vector<double>(x.size(), 0);
	std::vector<std::vector<double>> dif_vector(6, std::vector<double>(x.size(), 0));
	std::vector<double> add_dif(x.size(), 0);

	difFunctionPosition(x, dif_vector[0]);
	difFunctionScale(x, dif_vector[1]);
	difFunctionPathWidth(x, dif_vector[2], m_ApartBBoxPair, m_ApartDistance);
	difFunctionStructure(x, dif_vector[4], m_StructurePair, m_StructureDistanceSquare);
	difFunctionVisible(x, dif_vector[5], m_ObserveRelatedID, m_ObserveField, m_TargetPoint, m_OcclusionRelatedID);
	
	for (int i = 0; i < x.size(); ++i)
	{
		//for (int j = 1; j < 3; ++j)
		for (int j = 0; j < 6; ++j)
		{
			dif[i] += dif_vector[j][i] * m_WeightVector[j];
		}
	}
}

void Optimizer::HessianObjectFunction(std::vector<double> &x, std::vector<std::vector<double>> &hessian)
{
	hessian.clear();
	std::vector<double> vector_zero(x.size(), 0);
	std::vector<std::vector<double>> hessian_zero(x.size(), vector_zero);
	std::vector<std::vector<std::vector<double>>> hessian_vector(6, hessian_zero);
	std::vector<std::vector<double>> add_hessian(x.size(), vector_zero);
	hessian = std::vector < std::vector < double >>(x.size(), vector_zero);

	HessianFunctionPosition(x, hessian_vector[0]);
	HessianFunctionScale(x, hessian_vector[1]);
	HessianFunctionPathWidth(x, hessian_vector[2], m_ApartBBoxPair, m_ApartDistance);
	PositiveHessianFunctionStructure(x, hessian_vector[4], m_StructurePair, m_StructureDistanceSquare);
	HessianFunctionVisible(x, hessian_vector[5], m_ObserveRelatedID, m_ObserveField, m_TargetPoint, m_OcclusionRelatedID);

	for (int i = 0; i < x.size(); ++i)
	{
		for (int j = 0; j < x.size(); ++j)
		{
			//for (int k = 1; k < 3; ++k)
			for (int k = 0; k < 6; ++k)
			{
				hessian[i][j] += hessian_vector[k][i][j] * m_WeightVector[k];
			}
		}
	}
}

void Optimizer::LinearConstraint(int variable_num)
{

	m_MatrixA.clear() ;

	int pair_num = m_AdjacentPair.size();
	for (int i = 0; i < pair_num; ++i)
	{
		std::vector<double> constraint_v(variable_num, 0);
		int bbox1_id = m_AdjacentPair[i][0];
		int bbox2_id = m_AdjacentPair[i][1];
		if (m_AdjacentPair[i][2] == 0)
		{
			constraint_v[bbox1_id * 4] = -1;
			constraint_v[bbox1_id * 4 + 2] = 1;
			constraint_v[bbox2_id * 4] = 1;
			constraint_v[bbox2_id * 4 + 2] = 1;
		}
		else if (m_AdjacentPair[i][2] == 1)
		{
			constraint_v[bbox1_id * 4] = 1;
			constraint_v[bbox1_id * 4 + 2] = 1;
			constraint_v[bbox2_id * 4] = -1;
			constraint_v[bbox2_id * 4 + 2] = 1;
		}
		else if (m_AdjacentPair[i][2] == 2)
		{
			constraint_v[bbox1_id * 4 + 1] = -1; 
			constraint_v[bbox1_id * 4 + 3] = 1;
			constraint_v[bbox2_id * 4 + 1] = 1;
			constraint_v[bbox2_id * 4 + 3] = 1;
		}
		else
		{
			constraint_v[bbox1_id * 4 + 1] = 1;
			constraint_v[bbox1_id * 4 + 3] = 1;
			constraint_v[bbox2_id * 4 + 1] = -1;
			constraint_v[bbox2_id * 4 + 3] = 1;
		}
		m_MatrixA.push_back(constraint_v);
	}
}

void Optimizer::addSpecialLinearConstraint(std::string constraint_filepath, int variable_num)
{
	std::fstream constraint_file(constraint_filepath);
	int constraint_num;
	int nonzero_num;
	int nonzero_id;
	double nonzero_value;
	constraint_file >> constraint_num;
	for (int i = 0; i < constraint_num; ++i)
	{
		std::vector<double> constraint_v(variable_num, 0);
		constraint_file >> nonzero_num;
		for (int j = 0; j < nonzero_num; ++j)
		{
			constraint_file >> nonzero_id >> nonzero_value;
			constraint_v[nonzero_id] = nonzero_value;
		}
		m_MatrixA.push_back(constraint_v);
	}
}

void Optimizer::ConstractNewtonMatrix(std::vector<std::vector<double>> &hessian)
{
	int hessian_n = hessian.size();
	int A_h = m_MatrixA.size();
	int matrix_n = hessian_n + A_h;
	//std::cout << "matrix_n = " << matrix_n << std::endl;	
	m_MatrixNewton = Eigen::MatrixXd::Zero(matrix_n, matrix_n);
	for (int i = 0; i < hessian_n; ++i)
	{
		for (int j = 0; j < hessian_n; ++j)
		{
			m_MatrixNewton(i, j) = hessian[i][j];
		}
	}
	for (int i = 0; i < A_h; ++i)
	{
		for (int j = 0; j < hessian_n; ++j)
		{
			m_MatrixNewton(hessian_n + i, j) = m_MatrixA[i][j];
			m_MatrixNewton(j, hessian_n + i) = m_MatrixA[i][j];
		}
	}

	//testMatrixRank(m_MatrixA);
	//testMatrixRank(hessian);
	//testMatrixRank(m_MatrixNewton);
}

void Optimizer::UpdateNewtonMatrix(std::vector<std::vector<double>> &hessian)
{
	int hessian_n = hessian.size();
	for (int i = 0; i < hessian_n; ++i)
	{
		for (int j = 0; j < hessian_n; ++j)
		{
			m_MatrixNewton(i, j) = hessian[i][j];
		}
	}
}

void Optimizer::CalculateDescentDirection(std::vector<double> &p, std::vector<double> &dif, std::vector<std::vector<double>> &hessian)
{
	int matrix_num = hessian.size() + m_MatrixA.size();
	Eigen::VectorXd gradient = Eigen::VectorXd::Zero(matrix_num);
	Eigen::VectorXd direction = gradient;

	UpdateNewtonMatrix(hessian);

	for (int i = 0; i < dif.size(); ++i)
	{
		gradient[i] = -dif[i];
	}

	Eigen::MatrixXd newtonMatrix = m_MatrixNewton;

	

	Eigen::MatrixXd diagonalMatrix = Eigen::MatrixXd::Identity(matrix_num, matrix_num);

	for (int i = 0; i < m_MatrixA.size(); ++i)
	{
		diagonalMatrix(i + hessian.size(), i + hessian.size()) = 0;
	}

	double tao = 1e-5;
	while (tao < 1e10)
	{
		Eigen::LLT<Eigen::MatrixXd> Matrix_LLT(newtonMatrix);
		if (Matrix_LLT.info() != Eigen::Success)
		{
			newtonMatrix = m_MatrixNewton + diagonalMatrix * tao;
			tao *= 10.0;
		}
		else
		{
			direction = Matrix_LLT.solve(gradient);
			break;
		}
	}
	std::cout << " tao = " << tao << std::endl;
	for (int i = 0; i < dif.size(); ++i)
	{
		p[i] = direction[i];
	}
}

bool Optimizer::CalculateDescentDirection2(std::vector<double> &p, std::vector<double> &dif, std::vector<std::vector<double>> &hessian)
{
	bool positive_flag = true;
	int matrix_num = hessian.size() + m_MatrixA.size();
	Eigen::VectorXd gradient = Eigen::VectorXd::Zero(matrix_num);
	Eigen::VectorXd direction = gradient;

	//testMatrixRank(hessian);
	positive_flag = PositiveHessian(hessian);
	UpdateNewtonMatrix(hessian);

	for (int i = 0; i < dif.size(); ++i)
	{
		gradient[i] = -dif[i];
	}

	Eigen::FullPivLU<Eigen::MatrixXd> lu;
	lu.compute(m_MatrixNewton);
	direction = lu.solve(gradient);

	for (int i = 0; i < dif.size(); ++i)
	{
		p[i] = direction[i];
	}
	return positive_flag;
}

void Optimizer::CalculateDescentDirectionLU(std::vector<double> &p, std::vector<double> &dif, std::vector<std::vector<double>> &hessian)
{
	int matrix_num = hessian.size() + m_MatrixA.size();
	Eigen::VectorXd gradient = Eigen::VectorXd::Zero(matrix_num);
	Eigen::VectorXd direction = gradient;

	UpdateNewtonMatrix(hessian);

	for (int i = 0; i < dif.size(); ++i)
	{
		gradient[i] = -dif[i];
	}

	Eigen::FullPivLU<Eigen::MatrixXd> lu;
	lu.compute(m_MatrixNewton);
	direction = lu.solve(gradient);

	for (int i = 0; i < dif.size(); ++i)
	{
		p[i] = direction[i];
	}
}

double Optimizer::CalculateTmax(std::vector<double> &x, std::vector<double> &p)
{
	//return std::min(CalculateAffiliatedTMax(x, p, m_AffiliatedPair, m_AffiliatedVertex),
	//	CalculateVisibleTMax(x, p, m_ObserveRelatedID, m_ObserveField));

	//return CalculateAffiliatedTMax(x, p, m_AffiliatedPair, m_AffiliatedVertex);

	return CalculatePathBarrierTmax(x, p, m_ApartBBoxPair, m_ApartDistance);
	//return std::min(CalculatePathBarrierTmax(x, p, m_ApartBBoxPair, m_ApartDistance),
	//	CalculateVisibleTMax(x, p, m_ObserveRelatedID, m_ObserveField, m_TargetPoint, m_OcclusionRelatedID));
}

double Optimizer::CalculateAffiliatedTMax(std::vector<double> &x, std::vector<double> &p,
	std::vector<trimesh::ivec2> &affiliated_pair, std::vector<std::vector<int>> &affiliated_vertex)
{
	double tmax = 1e6;

	int bbox1_id, bbox2_id;
	trimesh::vec4 bbox1_x, bbox2_x, bbox1_p, bbox2_p;
	std::vector<int> vertex_id; 
	double coe_k, coe_b, temp_tmax;
	
	for (int i = 0; i < affiliated_pair.size(); ++i)
	{
		bbox1_id = affiliated_pair[i][0];
		bbox2_id = affiliated_pair[i][1];
		bbox1_x = trimesh::vec4(x[bbox1_id * 4], x[bbox1_id * 4 + 1], x[bbox1_id * 4 + 2], x[bbox1_id * 4 + 3]);
		bbox2_x = trimesh::vec4(x[bbox2_id * 4], x[bbox2_id * 4 + 1], x[bbox2_id * 4 + 2], x[bbox2_id * 4 + 3]);
		bbox1_p = trimesh::vec4(p[bbox1_id * 4], p[bbox1_id * 4 + 1], p[bbox1_id * 4 + 2], p[bbox1_id * 4 + 3]);
		bbox2_p = trimesh::vec4(p[bbox2_id * 4], p[bbox2_id * 4 + 1], p[bbox2_id * 4 + 2], p[bbox2_id * 4 + 3]);
		vertex_id = affiliated_vertex[i];

		double sum = 0;
		std::vector<trimesh::vec4> boundary_coe(4, trimesh::vec4(0, 0, 0, 0));
		boundary_coe[0] = trimesh::vec4(1, 0, -bbox1_x[0] + bbox1_x[2], -bbox1_p[0] + bbox1_p[2]);
		boundary_coe[1] = trimesh::vec4(0, 1, -bbox1_x[1] + bbox1_x[3], -bbox1_p[1] + bbox1_p[3]);
		boundary_coe[2] = trimesh::vec4(-1, 0, bbox1_x[0] + bbox1_x[2], bbox1_p[0] + bbox1_p[2]);
		boundary_coe[3] = trimesh::vec4(0, -1, bbox1_x[1] + bbox1_x[3], bbox1_p[1] + bbox1_p[3]);

		for (int j = 0; j < vertex_id.size(); ++j)
		{
			int current_id = vertex_id[j];
			trimesh::vec2 current_vertex_x, current_vertex_p;
			switch (current_id)
			{
			case 0:
				current_vertex_x[0] = bbox2_x[0] - bbox2_x[2];
				current_vertex_x[1] = bbox2_x[1] - bbox2_x[3];
				current_vertex_p[0] = bbox2_p[0] - bbox2_p[2];
				current_vertex_p[1] = bbox2_p[1] - bbox2_p[3];
				break;
			case 1:
				current_vertex_x[0] = bbox2_x[0] + bbox2_x[2];
				current_vertex_x[1] = bbox2_x[1] - bbox2_x[3];
				current_vertex_p[0] = bbox2_p[0] + bbox2_p[2];
				current_vertex_p[1] = bbox2_p[1] - bbox2_p[3];
				break;
			case 2:
				current_vertex_x[0] = bbox2_x[0] + bbox2_x[2];
				current_vertex_x[1] = bbox2_x[1] + bbox2_x[3];
				current_vertex_p[0] = bbox2_p[0] + bbox2_p[2];
				current_vertex_p[1] = bbox2_p[1] + bbox2_p[3];
				break;
			default:
				current_vertex_x[0] = bbox2_x[0] - bbox2_x[2];
				current_vertex_x[1] = bbox2_x[1] + bbox2_x[3];
				current_vertex_p[0] = bbox2_p[0] - bbox2_p[2];
				current_vertex_p[1] = bbox2_p[1] + bbox2_p[3];
				break;
			}
			for (int k = 0; k < 4; ++k)
			{
				coe_k = boundary_coe[k][0] * current_vertex_p[0] + boundary_coe[k][1] * current_vertex_p[1] + boundary_coe[k][3];
				coe_b = boundary_coe[k][0] * current_vertex_x[0] + boundary_coe[k][1] * current_vertex_x[1] + boundary_coe[k][2];
				if (coe_k < 0)
				{
					temp_tmax = -coe_b / coe_k;
					if (temp_tmax < tmax)
					{
						tmax = temp_tmax;
					}
				}
			}
		}
	}

	return tmax;
}

double Optimizer::CalculateVisibleTMax(std::vector<double> &x, std::vector<double> &p,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<std::vector<trimesh::vec2>> observe_field,
	std::vector<trimesh::vec2> target_point, std::vector<std::vector<int>> occlusion_related_id)
{
	double t_max = 1e6;

	double t_max_KV, t_max_PO;
	t_max_KV = CalculateKeepVisibleTmax(x, p, observe_related_id, observe_field, target_point);
	t_max_PO = CalculatePreventOcclusionTMax(x, p, observe_related_id, observe_field, occlusion_related_id);
	double min_t_max = std::min(t_max_KV, t_max_PO);
	//double min_t_max = t_max_PO;
	if (min_t_max < t_max)
	{
		t_max = min_t_max;
	}
	return t_max;
}

double Optimizer::CalculateKeepVisibleTmax(std::vector<double> &x, std::vector<double> &p,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<std::vector<trimesh::vec2>> observe_field,
	std::vector<trimesh::vec2> target_point)
{
	double a, b, c;
	int t_root_num = 0;
	trimesh::vec2 t_root(0, 0);
	double t_max = 1e6;

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		int o_b_id = observe_related_id[i][0];
		int t_b_id = observe_related_id[i][1];
		trimesh::vec4 observe_bbox_x(x[o_b_id * 4], x[o_b_id * 4 + 1], x[o_b_id * 4 + 2], x[o_b_id * 4 + 3]);
		trimesh::vec4 observe_bbox_p(p[o_b_id * 4], p[o_b_id * 4 + 1], p[o_b_id * 4 + 2], p[o_b_id * 4 + 3]);
		trimesh::vec4 target_bbox_x(x[t_b_id * 4], x[t_b_id * 4 + 1], x[t_b_id * 4 + 2], x[t_b_id * 4 + 3]);
		trimesh::vec4 target_bbox_p(p[t_b_id * 4], p[t_b_id * 4 + 1], p[t_b_id * 4 + 2], p[t_b_id * 4 + 3]);
		std::vector<trimesh::vec4> region_v(3, trimesh::vec4(0, 0, 0, 0));
		trimesh::vec4 target_position(0, 0, 0, 0);

		for (int j = 0; j < 3; ++j)
		{
			region_v[j][0] = observe_bbox_x[0] + observe_bbox_x[2] * observe_field[i][j][0];
			region_v[j][1] = observe_bbox_x[1] + observe_bbox_x[3] * observe_field[i][j][1];
			region_v[j][2] = observe_bbox_p[0] + observe_bbox_p[2] * observe_field[i][j][0];
			region_v[j][3] = observe_bbox_p[1] + observe_bbox_p[3] * observe_field[i][j][1];
		}
		target_position[0] = target_bbox_x[0] + target_point[i][0] * target_bbox_x[2];
		target_position[1] = target_bbox_x[1] + target_point[i][1] * target_bbox_x[3];
		target_position[2] = target_bbox_p[0] + target_point[i][0] * target_bbox_p[2];
		target_position[3] = target_bbox_p[1] + target_point[i][1] * target_bbox_p[3];

		for (int j = 0; j < 3; ++j)
		{
			a = (target_position[2] - region_v[j][2]) * (region_v[(j + 1) % 3][3] - region_v[j][3])
				- (target_position[3] - region_v[j][3]) * (region_v[(j + 1) % 3][2] - region_v[j][2]);
			b = (target_position[2] - region_v[j][2]) * (region_v[(j + 1) % 3][1] - region_v[j][1])
				+ (target_position[0] - region_v[j][0]) * (region_v[(j + 1) % 3][3] - region_v[j][3])
				- (target_position[3] - region_v[j][3]) * (region_v[(j + 1) % 3][0] - region_v[j][0])
				- (target_position[1] - region_v[j][1]) * (region_v[(j + 1) % 3][2] - region_v[j][2]);
			c = (target_position[0] - region_v[j][0]) * (region_v[(j + 1) % 3][1] - region_v[j][1])
				- (target_position[1] - region_v[j][1]) * (region_v[(j + 1) % 3][0] - region_v[j][0]);
			t_root_num = quadratic_root(a, b, c, t_root);
			if (t_root_num == 0)
			{
				
			}
			else if (t_root_num == 1)
			{
				if (t_root[0] > 0 && t_root[0]<t_max)
				{
					t_max = t_root[0];
				}
			}
			else
			{
				if (t_root[0] > 0 && t_root[0] < t_max)
				{
					t_max = t_root[0];
				}
				else if (t_root[1]>0 && t_root[1] < t_max)
				{
					t_max = t_root[1];
				}
			}
		}
	}

	return t_max;
}

double Optimizer::CalculatePreventOcclusionTMax(std::vector<double> &x, std::vector<double> &p,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<std::vector<trimesh::vec2>> observe_field,
	std::vector<std::vector<int>> occlusion_related_id)
{
	double a, b, c;
	int t_root_num = 0;
	trimesh::vec2 t_root(0, 0);
	double t_max = 1e6;
	trimesh::vec4 sign_a(-1, 1, 1, -1);
	trimesh::vec4 sign_b(-1, -1, 1, 1);

	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		int o_b_id = observe_related_id[i][0];
		trimesh::vec4 observe_bbox_x(x[o_b_id * 4], x[o_b_id * 4 + 1], x[o_b_id * 4 + 2], x[o_b_id * 4 + 3]);
		trimesh::vec4 observe_bbox_p(p[o_b_id * 4], p[o_b_id * 4 + 1], p[o_b_id * 4 + 2], p[o_b_id * 4 + 3]);
		std::vector<trimesh::vec4> region_v(3, trimesh::vec4(0, 0, 0, 0));

		for (int j = 0; j < 3; ++j)
		{
			region_v[j][0] = observe_bbox_x[0] + observe_bbox_x[2] * observe_field[i][j][0];
			region_v[j][1] = observe_bbox_x[1] + observe_bbox_x[3] * observe_field[i][j][1];
			region_v[j][2] = observe_bbox_p[0] + observe_bbox_p[2] * observe_field[i][j][0];
			region_v[j][3] = observe_bbox_p[1] + observe_bbox_p[3] * observe_field[i][j][1];
		}

		for (int j = 0; j < occlusion_related_id[i].size(); ++j)
		{
			int oc_b_id = occlusion_related_id[i][j];
			trimesh::vec4 occlusion_bbox_x(x[oc_b_id * 4], x[oc_b_id * 4 + 1], x[oc_b_id * 4 + 2], x[oc_b_id * 4 + 3]);
			trimesh::vec4 occlusion_bbox_p(p[oc_b_id * 4], p[oc_b_id * 4 + 1], p[oc_b_id * 4 + 2], p[oc_b_id * 4 + 3]);

			trimesh::vec4 occlusion_position(0, 0, 0, 0);
			for (int k = 0; k < 4; ++k)
			{
				occlusion_position[0] = occlusion_bbox_x[0] + sign_a[k] * occlusion_bbox_x[2];
				occlusion_position[1] = occlusion_bbox_x[1] + sign_b[k] * occlusion_bbox_x[3];
				occlusion_position[2] = occlusion_bbox_p[0] + sign_a[k] * occlusion_bbox_p[2];
				occlusion_position[3] = occlusion_bbox_p[1] + sign_b[k] * occlusion_bbox_p[3];

				for (int w = 0; w < 3; ++w)
				{
					a = (occlusion_position[2] - region_v[w][2]) * (region_v[(w + 1) % 3][3] - region_v[w][3])
						- (occlusion_position[3] - region_v[w][3]) * (region_v[(w + 1) % 3][2] - region_v[w][2]);
					b = (occlusion_position[2] - region_v[w][2]) * (region_v[(w + 1) % 3][1] - region_v[w][1])
						+ (occlusion_position[0] - region_v[w][0]) * (region_v[(w + 1) % 3][3] - region_v[w][3])
						- (occlusion_position[3] - region_v[w][3]) * (region_v[(w + 1) % 3][0] - region_v[w][0])
						- (occlusion_position[1] - region_v[w][1]) * (region_v[(w + 1) % 3][2] - region_v[w][2]);
					c = (occlusion_position[0] - region_v[w][0]) * (region_v[(w + 1) % 3][1] - region_v[w][1])
						- (occlusion_position[1] - region_v[w][1]) * (region_v[(w + 1) % 3][0] - region_v[w][0]);
					t_root_num = quadratic_root(a, b, c, t_root);
					if (t_root_num == 0)
					{

					}
					else if (t_root_num == 1)
					{
						if (t_root[0] > 0 && t_root[0] < t_max)
						{
							t_max = t_root[0];
						}
					}
					else
					{
						if (t_root[0] > 0 && t_root[0] < t_max)
						{
							t_max = t_root[0];
						}
						else if (t_root[1]>0 && t_root[1] < t_max)
						{
							t_max = t_root[1];
						}
					}
				}
			}
		}
	}

	return t_max;
}

double Optimizer::CalculateVisibleTMaxOld(std::vector<double> &x, std::vector<double> &p,
	std::vector<trimesh::ivec2> &observe_related_id, std::vector<trimesh::vec3> observe_field)
{
	double tmax = 1e6;

	trimesh::vec3 boundary1_coe, boundary2_coe;
	trimesh::vec2 observe_position_x,observe_position_p,target_position_x, target_position_p;
	double coe_k, coe_b, temp_tmax;
	for (int i = 0; i < observe_related_id.size(); ++i)
	{
		observe_position_x = trimesh::vec2(x[observe_related_id[i][0] * 4], x[observe_related_id[i][0] * 4 + 1]);
		observe_position_p = trimesh::vec2(p[observe_related_id[i][0] * 4], p[observe_related_id[i][0] * 4 + 1]);
		target_position_x = trimesh::vec2(x[observe_related_id[i][1] * 4], x[observe_related_id[i][1] * 4 + 1]);
		target_position_p = trimesh::vec2(p[observe_related_id[i][1] * 4], p[observe_related_id[i][1] * 4 + 1]);

		visible_boundary(observe_position_x, observe_field[i], boundary1_coe, boundary2_coe);

		coe_k = target_position_p[0] * boundary1_coe[0] + target_position_p[1] * boundary1_coe[1]
			- boundary1_coe[0] * observe_position_p[0] - boundary1_coe[1] * observe_position_p[1];
		coe_b = target_position_x[0] * boundary1_coe[0] + target_position_x[1] * boundary1_coe[1] + boundary1_coe[2];
		if (coe_k < 0)
		{
			temp_tmax = -coe_b / coe_k;
			if (temp_tmax < tmax)
			{
				tmax = temp_tmax;
			}
		}

		coe_k = target_position_p[0] * boundary2_coe[0] + target_position_p[1] * boundary2_coe[1]
			- boundary2_coe[0] * observe_position_p[0] - boundary2_coe[1] * observe_position_p[1];
		coe_b = target_position_x[0] * boundary2_coe[0] + target_position_x[1] * boundary2_coe[1] + boundary2_coe[2];
		if (coe_k < 0)
		{
			temp_tmax = -coe_b / coe_k;
			if (temp_tmax < tmax)
			{
				tmax = temp_tmax;
			}
		}
	}

	return tmax;
}

double Optimizer::CalculatePathBarrierTmax(std::vector<double> &x, std::vector<double> &p,
	std::vector<std::vector<trimesh::ivec2>> &apart_pair, std::vector<double> &apart_distance)
{
	double tmax = 1e6;

	int bbox1_ID, bbox2_ID;
	std::vector<trimesh::vec2> lx_C(2, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> ly_C(2, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> vx_C(4, trimesh::vec2(0, 0));
	std::vector<trimesh::vec2> vy_C(4, trimesh::vec2(0, 0));

	for (int i = 0; i < apart_pair.size(); ++i)
	{
		double distance = apart_distance[i];
		for (int j = 0; j < apart_pair[i].size(); ++j)
		{
			bbox1_ID = apart_pair[i][j][0];
			bbox2_ID = apart_pair[i][j][1];

			vl_Coefficient(lx_C, ly_C, vx_C, vy_C, x, p, bbox1_ID, bbox2_ID, distance);

			for (int k = 0; k < 4; ++k)
			{
				double t = 0;
				t = vertex2bboxTmax(lx_C, ly_C, vx_C[k], vy_C[k]);
				if (t < tmax)
				{
					tmax = t;
				}
			}
		}
	}

	return tmax;
}

void Optimizer::LineSearch(std::vector<double> &x, std::vector<double> &p, double tmax, trimesh::vec2 &stepsize,
	double energy_pre, std::vector<double> &energy, std::vector<double> &result)
{
	double tempt = std::min(1.0, tmax / 2.0);
	double temps = std::min(tempt, (double)(stepsize[1]));
	stepsize[0] = tempt;

	double energy_sum = 0;

	for (int i = 0; i < x.size(); ++i)
	{
		result[i] = x[i] + p[i] * temps;
	}
	ObjectFunction(result, energy);
	for (int i = 0; i < energy.size(); ++i)
	{
		energy_sum += energy[i];
	}

	if (energy_sum > energy_pre)
	{
		while (1)
		{
			temps *= 0.5;
			for (int i = 0; i < x.size(); ++i)
			{
				result[i] = x[i] + p[i] * temps;
			}
			ObjectFunction(result, energy);
			energy_sum = 0;
			for (int i = 0; i < energy.size(); ++i)
			{
				energy_sum += energy[i];
			}
			if (energy_sum < energy_pre)
			{
				break;
			}
			if (temps < 1e-16)
			{
				temps = 1.0;
				break;
			}
		}
	}
	else
	{
		while (1)
		{
			temps *= 2.0;
			if (temps > tempt)
			{
				temps = tempt;

				for (int i = 0; i < x.size(); ++i)
				{
					result[i] = x[i] + p[i] * temps;
				}
				ObjectFunction(result, energy);
				break;
			}

			for (int i = 0; i < x.size(); ++i)
			{
				result[i] = x[i] + p[i] * temps;
			}
			ObjectFunction(result, energy);
			energy_sum = 0;
			for (int i = 0; i < energy.size(); ++i)
			{
				energy_sum += energy[i];
			}
			if (energy_sum > energy_pre)
			{
				temps = temps * 0.5; 
				for (int i = 0; i < x.size(); ++i)
				{
					result[i] = x[i] + p[i] * temps;
				}
				ObjectFunction(result, energy);
				break;
			}
		}
	}
	stepsize[1] = temps;
}

void Optimizer::LineSearch(std::vector<double> &x, std::vector<double> &p, double energy_pre, std::vector<double> &energy,
	double &stepsize,std::vector<double> &result)
{
	std::ofstream linesearch("data/LS.txt");

	double temps = 1.0;

	double energy_sum = 0;

	for (int i = 0; i < x.size(); ++i)
	{
		result[i] = x[i] + p[i] * temps;
	}
	ObjectFunction(result, energy);

	

	for (int i = 0; i < energy.size(); ++i)
	{
		energy_sum += energy[i];
	}

	linesearch << "energy_pre = " << energy_pre << std::endl;
	linesearch << "temps =  " << temps << "    energy = [ ";
	for (int i = 0; i < energy.size(); ++i)
	{
		linesearch << energy[i] << " ";
	}
	linesearch << "],   sum = " << energy_sum << std::endl;

	if (energy_sum > energy_pre)
	{
		while (1)
		{
			temps *= 0.5;
			for (int i = 0; i < x.size(); ++i)
			{
				result[i] = x[i] + p[i] * temps;
			}
			ObjectFunction(result, energy);
			energy_sum = 0;
			for (int i = 0; i < energy.size(); ++i)
			{
				energy_sum += energy[i];
			}

			linesearch << "temps =  " << temps << "    energy = [ ";
			for (int i = 0; i < energy.size(); ++i)
			{
				linesearch << energy[i] << " ";
			}
			linesearch << "],   sum = " << energy_sum << std::endl;

			if (energy_sum < energy_pre)
			{
				break;
			}
			if (temps < 1e-8)
			{
				temps = 1.0;
				break;
			}
		}
	}
	stepsize = temps;
}

void Optimizer::LineSearch(std::vector<double> &x, std::vector<double> &p, double tmax, double energy_pre,
	std::vector<double> &energy, double &stepsize, std::vector<double> &result)
{
	std::ofstream linesearch("data/LS.txt");

	double temps = std::min(1.0, 0.8 * tmax);

	double energy_sum = 0;

	for (int i = 0; i < x.size(); ++i)
	{
		result[i] = x[i] + p[i] * temps;
	}
	ObjectFunction(result, energy);



	for (int i = 0; i < energy.size(); ++i)
	{
		energy_sum += energy[i];
	}

	linesearch << "energy_pre = " << energy_pre << std::endl;
	linesearch << "temps =  " << temps << "    energy = [ ";
	for (int i = 0; i < energy.size(); ++i)
	{
		linesearch << energy[i] << " ";
	}
	linesearch << "],   sum = " << energy_sum << std::endl;

	if (energy_sum > energy_pre)
	{
		while (1)
		{
			temps *= 0.5;
			for (int i = 0; i < x.size(); ++i)
			{
				result[i] = x[i] + p[i] * temps;
			}
			ObjectFunction(result, energy);
			energy_sum = 0;
			for (int i = 0; i < energy.size(); ++i)
			{
				energy_sum += energy[i];
			}

			linesearch << "temps =  " << temps << "    energy = [ ";
			for (int i = 0; i < energy.size(); ++i)
			{
				linesearch << energy[i] << " ";
			}
			linesearch << "],   sum = " << energy_sum << std::endl;

			if (energy_sum < energy_pre)
			{
				break;
			}
			if (temps < 1e-8)
			{
				temps = 1.0;
				break;
			}
		}
	}
	stepsize = temps;
}

void Optimizer::testMatrixRank(std::vector<std::vector<double>> matrix)
{
	int w, h;
	w = matrix[0].size();
	h = matrix.size();
	Eigen::MatrixXd M = Eigen::MatrixXd::Zero(h, w);
	for (int i = 0; i < matrix.size(); ++i)
	{
		for (int j = 0; j < matrix[i].size(); ++j)
		{
			M(i, j) = matrix[i][j];
		}
	}

	Eigen::FullPivLU<Eigen::MatrixXd> lu; 
	lu.compute(M);
	//Eigen::MatrixXd MLU = lu.matrixLU();
	int r = lu.rank();
	std::cout << "matrix h = " << h << "  w = " << w << "  rank = " << r << std::endl;
}

void Optimizer::testMatrixRank(Eigen::MatrixXd matrix)
{
	int w, h;
	h = matrix.rows();
	w = matrix.cols(); 
	Eigen::FullPivLU<Eigen::MatrixXd> lu;
	lu.compute(matrix);
	//Eigen::MatrixXd MLU = lu.matrixLU();
	int r = lu.rank();
	std::cout << "matrix h = " << h << "  w = " << w << "  rank = " << r << std::endl;
}

bool Optimizer::isHessianPositive(std::vector<std::vector<double>> &hessian)
{
	int n = hessian.size();
	double lamda = 1e-5;
	Eigen::MatrixXd M = Eigen::MatrixXd::Zero(n, n);
	for (int i = 0; i < n; ++i)
	{
		for (int j = 0; j < n; ++j)
		{
			M(i, j) = hessian[i][j];
		}
	}
	Eigen::LLT<Eigen::MatrixXd> ldlt;
	ldlt.compute(M);
	double det = M.determinant();
	if (Eigen::Success != ldlt.info())
	{
		//std::cout << "hessian det = " << det << " HESSIAN IS NO POSITIVE DEFINED !!!!!" << std::endl;
		std::cout << "hessian det = " << det << " HINPD! " << std::endl;
	}
}

bool Optimizer::PositiveHessian(std::vector<std::vector<double>> &hessian)
{
	bool positive_flag = true;
	int n = hessian.size();
	double lamda = 1e-5;
	Eigen::MatrixXd M = Eigen::MatrixXd::Zero(n, n);
	for (int i = 0; i < n; ++i)
	{
		for (int j = 0; j < n; ++j)
		{
			M(i, j) = hessian[i][j];
		}
	}


	for (int i = 0; i < 10; ++i)
	{
		Eigen::LLT<Eigen::MatrixXd> ldlt;
		ldlt.compute(M);
		//double det = M.determinant();
		//std::cout << "hessian det = " << det << std::endl;
		if (Eigen::Success != ldlt.info())
		{
			//std::cout << "    HESSIAN IS NOT POSITIVE DEFINED !!!!!" << std::endl;
			std::cout << " HINPD! " ;
			positive_flag = false;
			lamda *= 10;
			for (int j = 0; j < n; ++j)
			{
				M(j, j) = hessian[j][j] + lamda;
			}
		}
		else
		{
			for (int j = 0; j < n; ++j)
			{
				hessian[j][j] = M(j, j);
			}
			break;
		}
	}
	return positive_flag;
}

bool Optimizer::isPinTriangle(trimesh::vec2 p, trimesh::vec2 t1, trimesh::vec2 t2, trimesh::vec2 t3)
{
	double s = triangle_area(t1, t2, t3);
	std::cout << "s = " << s << std::endl;
	double s1 = triangle_area(p, t1, t2);
	std::cout << "s1 = " << s1 << std::endl;
	double s2 = triangle_area(p, t2, t3);
	std::cout << "s2 = " << s2 << std::endl;
	double s3 = triangle_area(p, t3, t1);
	std::cout << "s3 = " << s3 << std::endl;
	double d_s = s - s1 - s2 - s3;
	std::cout << "d_s = " << d_s << std::endl;

	if (std::abs(d_s) < 1e-3)
	{
		return true;
	}
	else
	{
		return false;
	}
}

double Optimizer::triangle_area(trimesh::vec2 t1, trimesh::vec2 t2, trimesh::vec2 t3)
{
	double l1 = (t1 - t2).length();
	double l2 = (t2 - t3).length();
	double l3 = (t3 - t1).length();
	double p = 0.5 * (l1 + l2 + l3);
	return std::sqrt(p * (p - l1) * (p - l2) * (p - l3));
}

void Optimizer::vl_Coefficient(std::vector<trimesh::vec2> &lx_C, std::vector<trimesh::vec2> &ly_C, std::vector<trimesh::vec2> &vx_C, std::vector<trimesh::vec2> &vy_C,
	std::vector<double> &x, std::vector<double> &p, int bbox1_ID, int bbox2_ID, double distance)
{
	lx_C[0][0] = p[bbox1_ID * 4] - p[bbox1_ID * 4 + 2];
	lx_C[0][1] = x[bbox1_ID * 4] - x[bbox1_ID * 4 + 2] - distance;
	lx_C[1][0] = p[bbox1_ID * 4] + p[bbox1_ID * 4 + 2];
	lx_C[1][1] = x[bbox1_ID * 4] + x[bbox1_ID * 4 + 2] + distance;
	ly_C[0][0] = p[bbox1_ID * 4 + 1] - p[bbox1_ID * 4 + 3];
	ly_C[0][1] = x[bbox1_ID * 4 + 1] - x[bbox1_ID * 4 + 3] - distance;
	ly_C[1][0] = p[bbox1_ID * 4 + 1] + p[bbox1_ID * 4 + 3];
	ly_C[1][1] = x[bbox1_ID * 4 + 1] + x[bbox1_ID * 4 + 3] + distance;

	vx_C[0][0] = vx_C[3][0] = p[bbox2_ID * 4] - p[bbox2_ID * 4 + 2];
	vx_C[0][1] = vx_C[3][1] = x[bbox2_ID * 4] - x[bbox2_ID * 4 + 2];
	vx_C[1][0] = vx_C[2][0] = p[bbox2_ID * 4] + p[bbox2_ID * 4 + 2];
	vx_C[1][1] = vx_C[2][1] = x[bbox2_ID * 4] + x[bbox2_ID * 4 + 2];

	vy_C[0][0] = vy_C[1][0] = p[bbox2_ID * 4 + 1] - p[bbox2_ID * 4 + 3];
	vy_C[0][1] = vy_C[1][1] = x[bbox2_ID * 4 + 1] - x[bbox2_ID * 4 + 3];
	vy_C[2][0] = vy_C[3][0] = p[bbox2_ID * 4 + 1] + p[bbox2_ID * 4 + 3];
	vy_C[2][1] = vy_C[3][1] = x[bbox2_ID * 4 + 1] + x[bbox2_ID * 4 + 3];
}

double Optimizer::vertex2bboxTmax(std::vector<trimesh::vec2> lx_C, std::vector<trimesh::vec2> ly_C, trimesh::vec2 vx_C, trimesh::vec2 vy_C)
{
	double t = 1e6;
	for (int i = 0; i < lx_C.size(); ++i)
	{
		double temp_t = 0;
		double da = vx_C[0] - lx_C[i][0];
		double db = vx_C[1] - lx_C[i][1];
		if (std::abs(da) > 1e-6)
		{
			temp_t = -db / da;
			if (temp_t > 0 && temp_t < t)
			{
				if (isOnSegment(ly_C[0], ly_C[1], vy_C, temp_t))
				{
					t = temp_t;
				}
			}
		}

		da = vy_C[0] - ly_C[i][0];
		db = vy_C[1] - ly_C[i][1];
		if (std::abs(da) > 1e-6)
		{
			temp_t = -db / da;
			if (temp_t > 0 && temp_t < t)
			{
				if (isOnSegment(lx_C[0], lx_C[1], vx_C, temp_t))
				{
					t = temp_t;
				}
			}
		}
	}

	return t;
}

bool Optimizer::isOnSegment(trimesh::vec2 bound1, trimesh::vec2 bound2, trimesh::vec2 v, double t)
{
	double b1, b2, vt;
	b1 = bound1[0] * t + bound1[1];
	b2 = bound2[0] * t + bound2[1];
	vt = v[0] * t + v[1];
	if (vt > b1 && vt < b2)
	{
		return true;
	}
	else
	{
		return false;
	}
}

void Optimizer::testFunction_phi()
{
	double s_min = 0.5;
	double s_max = 2.0;
	std::cout << "Phi(0.5) = " << function_phi(0.5, s_min, s_max) << "  dPhi(0.5) = " << dif_function_phi(0.5, s_min, s_max)
		<< "  ddPhi(0.5) = " << ddif_function_phi(0.5, s_min, s_max) << std::endl;
	std::cout << "Phi(0.501) = " << function_phi(0.501, s_min, s_max) << "  dPhi(0.501) = " << dif_function_phi(0.501, s_min, s_max)
		<< "  ddPhi(0.501) = " << ddif_function_phi(0.501, s_min, s_max) << std::endl;
	std::cout << "Phi(0.502) = " << function_phi(0.502, s_min, s_max) << "  dPhi(0.502) = " << dif_function_phi(0.502, s_min, s_max)
		<< "  ddPhi(0.502) = " << ddif_function_phi(0.502, s_min, s_max) << std::endl;
	std::cout << "Phi(0.51) = " << function_phi(0.51, s_min, s_max) << "  dPhi(0.51) = " << dif_function_phi(0.51, s_min, s_max)
		<< "  ddPhi(0.51) = " << ddif_function_phi(0.51, s_min, s_max) << std::endl;
	std::cout << "Phi(1.0) = " << function_phi(1.0, s_min, s_max) << "  dPhi(1.0) = " << dif_function_phi(1.0, s_min, s_max)
		<< "  ddPhi(1.0) = " << ddif_function_phi(1.0, s_min, s_max) << std::endl;
	std::cout << "Phi(1.90) = " << function_phi(1.90, s_min, s_max) << "  dPhi(1.90) = " << dif_function_phi(1.90, s_min, s_max)
		<< "  ddPhi(1.90) = " << ddif_function_phi(1.90, s_min, s_max) << std::endl;
	std::cout << "Phi(1.990) = " << function_phi(1.990, s_min, s_max) << "  dPhi(1.990) = " << dif_function_phi(1.990, s_min, s_max)
		<< "  ddPhi(1.990) = " << ddif_function_phi(1.990, s_min, s_max) << std::endl;
}

double Optimizer::BBoxDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2)
{
	trimesh::vec2 center_distance(std::abs(bbox1[0] - bbox2[0]), std::abs(bbox1[1] - bbox2[1]));
	trimesh::vec2 bbox_distance(center_distance[0] - bbox1[2] - bbox2[2], center_distance[1] - bbox1[3] - bbox2[3]);
	if (bbox_distance[0] <= 0 && bbox_distance[1] <= 0)
	{
		return std::max(bbox_distance[0], bbox_distance[1]);
	}
	else if (bbox_distance[0] > 0 && bbox_distance[1] > 0)
	{
		return bbox_distance.length();
		//return std::min(bbox_distance[0], bbox_distance[1]);
	}
	else
	{
		if (bbox_distance[0] > 0)
		{
			return bbox_distance[0];
		}
		else
		{
			return bbox_distance[1];
		}
	}
}

double Optimizer::BBoxDistance(trimesh::vec4 bbox1, trimesh::vec4 bbox2, int &apart_mode)
{
	trimesh::vec2 center_distance(std::abs(bbox1[0] - bbox2[0]), std::abs(bbox1[1] - bbox2[1]));
	trimesh::vec2 bbox_distance(center_distance[0] - bbox1[2] - bbox2[2], center_distance[1] - bbox1[3] - bbox2[3]);
	if (bbox_distance[0] <= 0 && bbox_distance[1] <= 0)
	{
		return std::max(bbox_distance[0], bbox_distance[1]);
	}
	else if (bbox_distance[0] > 0 && bbox_distance[1] > 0)
	{
		apart_mode = 2;
		return bbox_distance.length();
		//return std::min(bbox_distance[0], bbox_distance[1]);
	}
	else
	{
		if (bbox_distance[0] > 0)
		{
			apart_mode = 0;
			return bbox_distance[0];
		}
		else
		{
			apart_mode = 1;
			return bbox_distance[1];
		}
	}
}

void Optimizer::testFunctionBarrier()
{
	trimesh::vec4 bbox1(1, 1, 1, 1);
	trimesh::vec4 bbox2(2.50, 1.0, 0.05, 0.05);
	double test_energy = function_barrier(bbox1, bbox2, 0.4, 0.1);
	std::cout << "function barrier energy = " << test_energy << std::endl;
}

void Optimizer::deleteRegularity(std::vector<trimesh::ivec3> &regularity_structure, int id)
{
	int delete_id = regularity_structure[id][0];
	
	if (regularity_structure[id][2] == 0)
	{
		m_OriginalScale[delete_id][0] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_OriginalScale[delete_id][0];
		//m_ScaleThreshold[delete_id][0] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_ScaleThreshold[delete_id][0];
		//m_ScaleThreshold[delete_id][1] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_ScaleThreshold[delete_id][1];
	}
	else
	{
		m_OriginalScale[delete_id][1] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_OriginalScale[delete_id][1];
		//m_ScaleThreshold[delete_id][2] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_ScaleThreshold[delete_id][2];
		//m_ScaleThreshold[delete_id][3] = (double)(regularity_structure[id][1] - 1) / (double)(regularity_structure[id][1]) * m_ScaleThreshold[delete_id][3];
	}
}

void Optimizer::deleteRegularitySpecial()
{
	//deleteRegularity(m_RegularityStructure, 4);
	deleteRegularity(m_RegularityStructure, 1);
}

void Optimizer::resetRegularity()
{
	m_OriginalScale = m_OriginalScaleStore;
	//m_ScaleThreshold = m_ScaleThresholdStore;
}

int Optimizer::quadratic_root(double a, double b, double c, trimesh::vec2 &q_root)
{
	double delta = b * b - 4.0 * a * c;
	if (std::abs(a) < 1e-8)
	{
		if (std::abs(b) < 1e-8)
		{
			return 0;
		}
		else
		{
			q_root[0] = q_root[1] = -c / b;
			return 1;
		}
	}
	if (delta < -1e-6)
	{
		return 0;
	}
	else if (delta < 1e6)
	{
		q_root[0] = q_root[1] = -0.5 * b / a;
		return 1;
	}
	else
	{
		q_root[0] = 0.5 * (-b - std::sqrt(delta)) / a;
		q_root[1] = 0.5 * (-b + std::sqrt(delta)) / a;
		return 2;
	}
}